Package: vetr
Title: Trust, but Verify
Description: Declarative template-based framework for verifying that objects
  meet structural requirements, and auto-composing error messages when they do
  not.
Version: 0.2.22
Authors@R: c(
    person("Brodie", "Gaslam", email="brodie.gaslam@yahoo.com",
    role=c("aut", "cre")),
    person(given="Paxdiablo", role="cph", comment="Hash
    table implementation in src/pfhash.h"),
    person(given="R Core Team",
    email="R-core@r-project.org", role="cph",
    comment="Used/adapted several code snippets from R sources, see src/r-copied.c"),
    person("Michael","Chirico", role="ctb",
    email="michaelchirico4@gmail.com", comment = c(ORCID="0000-0003-0787-087X")
    ))
Depends: R (>= 3.2.0)
License: GPL (>= 2)
URL: https://github.com/brodieG/vetr
BugReports: https://github.com/brodieG/vetr/issues
VignetteBuilder: knitr
Imports: methods, stats, utils
Suggests: knitr, rmarkdown, unitizer
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-04-27 11:33:31 UTC; brodie
Author: Brodie Gaslam [aut, cre],
  Paxdiablo [cph] (Hash table implementation in src/pfhash.h),
  R Core Team [cph] (Used/adapted several code snippets from R sources,
    see src/r-copied.c),
  Michael Chirico [ctb] (ORCID: <https://orcid.org/0000-0003-0787-087X>)
Maintainer: Brodie Gaslam <brodie.gaslam@yahoo.com>
Repository: CRAN
Date/Publication: 2026-04-27 13:50:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-01 17:08:21 UTC; unix
Archs: vetr.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(abstract,array)
S3method(abstract,data.frame)
S3method(abstract,default)
S3method(abstract,environment)
S3method(abstract,ggplot)
S3method(abstract,list)
S3method(abstract,lm)
S3method(abstract,matrix)
S3method(abstract,ts)
S3method(nullify,default)
export(CHR)
export(CHR.1)
export(CPX)
export(CPX.1)
export(GT.0)
export(GTE.0)
export(INT)
export(INT.1)
export(INT.1.NEG)
export(INT.1.NEG.STR)
export(INT.1.POS)
export(INT.1.POS.STR)
export(INT.NEG)
export(INT.NEG.STR)
export(INT.POS)
export(INT.POS.STR)
export(LGL)
export(LGL.1)
export(LT.0)
export(LTE.0)
export(NO.INF)
export(NO.NA)
export(NUM)
export(NUM.1)
export(NUM.1.NEG)
export(NUM.1.POS)
export(NUM.NEG)
export(NUM.POS)
export(abstract)
export(alike)
export(all_bw)
export(bench_mark)
export(nullify)
export(tev)
export(type_alike)
export(type_of)
export(vet)
export(vet_token)
export(vetr)
export(vetr_settings)
importFrom(methods,new)
importFrom(stats,median)
importFrom(utils,modifyList)
useDynLib(vetr, .registration=TRUE, .fixes="VALC_")

<!-- README.md is generated from README.Rmd. Please edit that file 

library(rmarkdown)
render('README.Rmd', output_format=html_vignette(css='vignettes/styles.css'))
rmarkdown::render('README.Rmd', output_format=rmarkdown::md_document())

-->

# vetr

[![R build
status](https://github.com/brodieG/vetr/workflows/R-CMD-check/badge.svg)](https://github.com/brodieG/vetr/actions)
[![](https://codecov.io/github/brodieG/vetr/coverage.svg?branch=rc)](https://app.codecov.io/gh/brodieG/vetr?branch=rc)
[![](http://www.r-pkg.org/badges/version/vetr)](https://cran.r-project.org/package=vetr)
[![Project Status: WIP - Initial development is in progress, but there
has not yet been a stable, usable release suitable for the
public.](https://www.repostatus.org/badges/latest/wip.svg)](https://www.repostatus.org/#wip)
[![Dependencies
direct/recursive](https://tinyverse.netlify.app/badge/vetr)](https://tinyverse.netlify.app/)

## Trust, but Verify

### Easily

When you write functions that operate on S3 or unclassed objects you can
either trust that your inputs will be structured as expected, or
tediously check that they are.

`vetr` takes the tedium out of structure verification so that you can
trust, but verify. It lets you express structural requirements
declaratively with templates, and it auto-generates human-friendly error
messages as needed.

### Quickly

`vetr` is written in C to minimize overhead from parameter checks in
your functions. It has no dependencies.

## Declarative Checks with Templates

### Templates

Declare a template that an object should conform to, and let `vetr` take
care of the rest:

    library(vetr)
    tpl <- numeric(1L)
    vet(tpl, 1:3)
    ## [1] "`length(1:3)` should be 1 (is 3)"
    vet(tpl, "hello")
    ## [1] "`\"hello\"` should be type \"numeric\" (is \"character\")"
    vet(tpl, 42)
    ## [1] TRUE

The template concept is based on `vapply`, but generalizes to all S3
objects and adds some special features to facilitate comparison. For
example, zero length templates match any length:

    tpl <- integer()
    vet(tpl, 1L:3L)
    ## [1] TRUE
    vet(tpl, 1L)
    ## [1] TRUE

And for convenience short (&lt;= 100 length) integer-like numerics are
considered integer:

    tpl <- integer(1L)
    vet(tpl, 1)       # this is a numeric, not an integer
    ## [1] TRUE
    vet(tpl, 1.0001)
    ## [1] "`1.0001` should be type \"integer-like\" (is \"double\")"

`vetr` can compare recursive objects such as lists, or data.frames:

    tpl.iris <- iris[0, ]      # 0 row DF matches any number of rows in object
    iris.fake <- iris
    levels(iris.fake$Species)[3] <- "sibirica"   # tweak levels

    vet(tpl.iris, iris)
    ## [1] TRUE
    vet(tpl.iris, iris.fake)
    ## [1] "`levels(iris.fake$Species)[3]` should be \"virginica\" (is \"sibirica\")"

From our declared template `iris[0, ]`, `vetr` infers all the required
checks. In this case, `vet(iris[0, ], iris.fake, stop=TRUE)` is
equivalent to:

    stopifnot_iris <- function(x) {
      stopifnot(
        is.data.frame(x),
        is.list(x),
        length(x) == length(iris),
        identical(lapply(x, class), lapply(iris, class)),
        is.integer(attr(x, 'row.names')),
        identical(names(x), names(iris)),
        identical(typeof(x$Species), "integer"),
        identical(levels(x$Species), levels(iris$Species))
      )
    }
    stopifnot_iris(iris.fake)
    ## Error in stopifnot_iris(iris.fake): identical(levels(x$Species), levels(iris$Species)) is not TRUE

`vetr` saved us typing, and the time and thought needed to come up with
what needs to be compared.

You could just as easily have created templates for nested lists, or
data frames in lists. Templates are compared to objects with the `alike`
function. For a thorough description of templates and how they work see
the [`alike`
vignette](https://cran.r-project.org/package=vetr/vignettes/alike.html).
For template examples see `example(alike)`.

### Auto-Generated Error Messages

Let’s revisit the error message:

    vet(tpl.iris, iris.fake)
    ## [1] "`levels(iris.fake$Species)[3]` should be \"virginica\" (is \"sibirica\")"

It tells us:

-   The reason for the failure
-   What structure would be acceptable instead
-   The location of failure `levels(iris.fake$Species)[3]`

`vetr` does what it can to reduce the time from error to resolution. The
location of failure is generated such that you can easily copy it in
part or full to the R prompt for further examination.

## Vetting Expressions

You can combine templates with `&&` / `||`:

    vet(numeric(1L) || NULL, NULL)
    ## [1] TRUE
    vet(numeric(1L) || NULL, 42)
    ## [1] TRUE
    vet(numeric(1L) || NULL, "foo")
    ## [1] "`\"foo\"` should be `NULL`, or type \"numeric\" (is \"character\")"

Templates only check structure. When you need to check values use `.` to
refer to the object:

    vet(numeric(1L) && . > 0, -42)  # strictly positive scalar numeric
    ## [1] "`-42 > 0` is not TRUE (FALSE)"
    vet(numeric(1L) && . > 0, 42)
    ## [1] TRUE

If you do use the `.` symbol in your vetting expressions in your
packages, you will need to include `utils::globalVariables(".")` as a
top-level call to avoid the “no visible binding for global variable ‘.’”
R CMD check NOTE.

You can compose vetting expressions as language objects and combine
them:

    scalar.num.pos <- quote(numeric(1L) && . > 0)
    foo.or.bar <- quote(character(1L) && . %in% c('foo', 'bar'))
    vet.exp <- quote(scalar.num.pos || foo.or.bar)

    vet(vet.exp, 42)
    ## [1] TRUE
    vet(vet.exp, "foo")
    ## [1] TRUE
    vet(vet.exp, "baz")
    ## [1] "At least one of these should pass:"                         
    ## [2] "  - `\"baz\" %in% c(\"foo\", \"bar\")` is not TRUE (FALSE)" 
    ## [3] "  - `\"baz\"` should be type \"numeric\" (is \"character\")"

`all_bw` is available for value range checks (~10x faster than
`isTRUE(all(. >= x & . <= y))` for large vectors):

    vet(all_bw(., 0, 1), runif(5) + 1)
    ## [1] "`all_bw(runif(5) + 1, 0, 1)` is not TRUE (is chr: \"`1.643241` at index 1 not in `[0,1]`\")"

There are a number of predefined vetting tokens you can use in your
vetting expressions such as:

    vet(NUM.POS, -runif(5))    # positive numeric; see `?vet_token` for others
    ## [1] "`-runif(5)` should contain only positive values, but has negatives"

Vetting expressions are designed to be intuitive to use, but their
implementation is complex. We recommend you look at `example(vet)` for
usage ideas, or at the [“Non Standard Evaluation” section of the
vignette](https://cran.r-project.org/package=vetr/vignettes/vetr.html#non-standard-evaluation)
for the gory details.

## `vetr` in Functions

If you are vetting function inputs, you can use the `vetr` function,
which works just like `vet` except that it is streamlined for use within
functions:

    fun <- function(x, y) {
      vetr(numeric(1L), logical(1L))
      TRUE   # do work...
    }
    fun(1:2, "foo")
    ## Error in fun(x = 1:2, y = "foo"): For argument `x`, `length(1:2)` should be 1 (is 2)
    fun(1, "foo")
    ## Error in fun(x = 1, y = "foo"): For argument `y`, `"foo"` should be type "logical" (is "character")

`vetr` automatically matches the vetting expressions to the
corresponding arguments and fetches the argument values from the
function environment.

See
[vignette](https://cran.r-project.org/package=vetr/vignettes/vetr.html#in-functions)
for additional details on how the `vetr` function works.

## Additional Documentation

-   [`vetr`
    vignette](https://cran.r-project.org/package=vetr/vignettes/vetr.html),
    `?vet`, `?vetr`, `example(vet)`, `example(vetr)`.
-   [`alike`
    vignette](https://cran.r-project.org/package=vetr/vignettes/alike.html),
    `?alike`, and `example(alike)` for discussion of templates.
-   A survey of [parameter validation
    functions](http://htmlpreview.github.io/?https://github.com/brodieG/vetr/blob/master/extra/compare.html).

## Development Status

`vetr` is still in development, although most of the features are
considered mature. The most likely area of change is the treatment of
function and language templates (e.g. `alike(sum, max)`), and more
flexible treatment of list templates (e.g. in future lists may be
allowed to be different lengths so long as every named element in the
template exists in the object).

## Installation

This package is available on CRAN:

    install.packages('vetr')

It has no runtime dependencies.

For the development version use
`remotes::install_github('brodieg/vetr@development')` or:

    f.dl <- tempfile()
    f.uz <- tempfile()
    github.url <- 'https://github.com/brodieG/vetr/archive/development.zip'
    download.file(github.url, f.dl)
    unzip(f.dl, exdir=f.uz)
    install.packages(file.path(f.uz, 'vetr-development'), repos=NULL, type='source')
    unlink(c(f.dl, f.uz))

The master branch typically mirrors CRAN and should be stable.

## Alternatives

There are many alternatives available to `vetr`. We do a survey of the
following in our [parameter validation
functions](http://htmlpreview.github.io/?https://github.com/brodieG/vetr/blob/master/extra/compare.html)
review:

-   `stopifnot` by R Core
-   [`vetr`](https://github.com/brodieG/vetr) by Yours Truly
-   [`asserthat`](https://github.com/hadley/assertthat) by Hadley
    Wickham
-   [`assertive`](https://www.r-pkg.org/pkg/assertive) by Richie Cotton
-   [`checkmate`](https://github.com/mllg/checkmate) by Michel Lang

The following packages also perform related tasks, although we have not
used them and do not review them:

-   [`valaddin`](https://github.com/egnha/valaddin) v0.1.0 by Eugene Ha,
    a framework for augmenting existing functions with validation
    contracts. Currently the package is undergoing a major overhaul so
    we will add it to the comparison once the new release (v0.3.0) is
    out.
-   [`ensurer`](https://github.com/smbache/ensurer) v1.1 by Stefan M.
    Bache, a framework for flexibly creating and combining validation
    contracts. The development version adds an experimental method for
    creating type safe functions, but it is not published to CRAN so we
    do not test it here.
-   [`validate`](https://github.com/data-cleaning/validate) by Mark van
    der Loo and Edwin de Jonge, with a primary focus on validating data
    in data frames and similar data structures.
-   [`assertr`](https://github.com/tonyfischetti/assertr) by Tony
    Fischetti, also focused on data validation in data frames and
    similar structures.
-   [`types`](https://github.com/jimhester/types) by Jim Hester, which
    implements but does not enforce type hinting.
-   [`argufy`](https://github.com/gaborcsardi/argufy) by Gábor Csárdi,
    which implements parameter validation via roxygen tags (not released
    to CRAN).
-   [`typed`](https://moodymudskipper.github.io/typed/) by Antoine
    Fabri, which enforces types of symbols, function parameters, and
    return values.
-   [`erify`](https://github.com/flujoo/erify/) by Renfei Mao, with a
    focus on readable error messages.

## Acknowledgments

Thank you to:

-   R Core for developing and maintaining such a wonderful language.
-   CRAN maintainers, for patiently shepherding packages onto CRAN and
    maintaining the repository, and Uwe Ligges in particular for
    maintaining [Winbuilder](https://win-builder.r-project.org/).
-   Users and others who have reported bugs and/or helped contribute
    fixes (see NEWS.md).
-   Tomas Kalibera for [rchk](https://github.com/kalibera/rchk) and
    rcnst to help detect errors in compiled code, and in particular for
    his infinite patience in helping me resolve the issues he identified
    for me.
-   [Jim Hester](https://github.com/jimhester) because
    [covr](https://cran.r-project.org/package=covr) rocks.
-   [Dirk Eddelbuettel](https://github.com/eddelbuettel) and [Carl
    Boettiger](https://github.com/cboettig) for the
    [rocker](https://github.com/rocker-org/rocker) project, and [Gábor
    Csárdi](https://github.com/gaborcsardi) and the
    [R-consortium](https://r-consortium.org/) for
    [Rhub](https://github.com/r-hub), without which testing bugs on
    R-devel and other platforms would be a nightmare.
-   [Winston Chang](https://github.com/wch) for the
    [r-debug](https://hub.docker.com/r/wch1/r-debug/) docker container,
    in particular because of the valgrind level 2 instrumented version
    of R.
-   [Hadley Wickham](https://github.com/hadley/) and [Peter
    Danenberg](https://github.com/klutometis) for
    [roxygen2](https://cran.r-project.org/package=roxygen2).
-   [Yihui Xie](https://github.com/yihui) for
    [knitr](https://cran.r-project.org/package=knitr) and [J.J.
    Allaire](https://github.com/jjallaire) etal for
    [rmarkdown](https://cran.r-project.org/package=rmarkdown), and by
    extension John MacFarlane for [pandoc](https://pandoc.org/).
-   [Michel Lang](https://github.com/mllg) for pushing me to implement
    `all_bw` to compete with his own package
    [`checkmate`](https://cran.r-project.org/package=checkmate).
-   [Eugene Ha](https://github.com/egnha) for pointing me to several
    other relevant packages, which in turn led to the [survey of related
    packages](http://htmlpreview.github.io/?https://github.com/brodieG/vetr/blob/master/extra/compare.html).
-   Stefan M. Bache for the idea of having a function for testing
    objects directly (originally `vetr` only worked with function
    arguments), which I took from ensurer.
-   Olaf Mersmann for
    [microbenchmark](https://cran.r-project.org/package=microbenchmark),
    because microsecond matter, and [Joshua
    Ulrich](https://github.com/joshuaulrich) for making it lightweight.
-   All open source developers out there that make their work freely
    available for others to use.
-   [Github](https://github.com/), [Codecov](https://about.codecov.io/),
    [Vagrant](https://www.vagrantup.com/),
    [Docker](https://www.docker.com/), [Ubuntu](https://ubuntu.com/),
    [Brew](https://brew.sh/) for providing infrastructure that greatly
    simplifies open source development.
-   [Free Software Foundation](https://www.fsf.org/) for developing the
    GPL license and promotion of the free software movement.

## About the Author

Brodie Gaslam is a hobbyist programmer based on the US East Coast.

# Copyright (C) 2023  Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

#' Turn S3 Objects Into Templates
#'
#' Create templates for use by \code{\link{alike}}. Currently somewhat
#' experimental; behavior may change in future.
#'
#' \code{abstract} is intended to create templates for use by
#' \code{\link{alike}}.  The result of abstraction is often a partially
#' specified object.  This type of object may not be suited for use in typical
#' R computations and may cause errors (or worse) if you try to use them as
#' normal R objects.
#'
#' There is no guarantee that the \code{abstract}ed object is suitable for use
#' as a template to \code{alike} as is.  You may need to modify it further so
#' that it suits your purposes.
#'
#' \code{abstract} is an S3 generic.  The default method will
#' dispatch on implicit classes, so if you attempt to \code{abstract} an object
#' without an explicit \code{abstract} method, it will get abstracted based on
#' its implicit class.  If you define your own \code{abstract} method and do not
#' wish further abstraction based on implicit classes do not use
#' \code{\link{NextMethod}}.
#'
#' S4 and RC objects are returned unchanged.
#'
#' @section Time Series:
#'
#' `abstract` replaces the "tsp" attribute with a "tsp_vetr" attribute with the
#' components specified in `what` set to zero.  \code{\link{alike}} will
#' treat it as a "tsp" attribute except that the zero components become
#' wildcards.  If you manually create a template object with both
#' "tsp" and "tsp_vetr" attributes, "tsp_vetr" is treated as a regular
#' attribute.  `vetr` does not consider whether the "ts" class is also set
#' when attributing special semantics to the "tsp".  The "tsp_vetr" attribute is
#' required because R does not permit 0 values in "tsp".  Prior to R4.6.0 and
#' `vetr` 2.20, `vetr` constructed a "tsp" attribute with zeroes, but it no
#' longer does that.
#'
#' @export
#' @param x the object to abstract
#' @param ... arguments for methods that require further arguments
#' @param what, for time series which portion of the \code{ts} attribute to
#'   abstract, by default all three are abstracted, but you can select, any one,
#'   two, or all
#' @return abstracted object
#' @examples
#' iris.tpl <- abstract(iris)
#' alike(iris.tpl, iris[1:10, ])
#' alike(iris.tpl, transform(iris, Species=as.character(Species)))
#'
#' abstract(1:10)
#' abstract(matrix(1:9, nrow=3))
#' abstract(list(1:9, runif(10)))

abstract <- function(x, ...) UseMethod("abstract")

#' @rdname abstract
#' @export

abstract.data.frame <- function(x, ...) x[0, ]

#' @importFrom utils modifyList
#' @rdname abstract
#' @export

abstract.default <- function(x, ...) {
  if(isS4(x)) return(x);
  if(!is.null(class.exp <- attr(x, "class"))) {
    attr(x, "class") <- NULL
    x <- abstract(x, ...)  # handle implicit classes
    class(x) <- class.exp
  }
  if(!is.atomic(x)) return(x)
  attrs.old <- attributes(x)
  length(x) <- 0L
  attrs.new <- attributes(x)
  attributes(x) <- if(!is.null(attrs.new)) modifyList(attrs.old, attrs.new)
  else attrs.old
  x
}
#' @rdname abstract
#' @export

abstract.array <- function(x, ...) {
  if(!is.atomic(x)) NextMethod()
  ndims <- length(dim(x))
  length(x) <- 0L
  dim(x) <- rep(0L, ndims)
  # nocov start previous line should get rid of dimnames already so
  # there is no way to test this, but leaving it here just in case
  if(!is.null(dimnames(x)))
    dimnames(x) <- replicate(NULL, length(dimnames(x)), simplify=FALSE)
  # nocov end
  x
}
#' @rdname abstract
#' @export

abstract.matrix <-function(x, ...) abstract.array(x, ...)

#' @rdname abstract
#' @export

abstract.list <- function(x, ...) {
  for(i in seq_along(x)) {
    xi.abs <- abstract(x[[i]])
    if(is.null(xi.abs)) x <- nullify(x, i)
    else x[[i]] <- xi.abs
  }
  x
}
#' @rdname abstract
#' @export

abstract.lm <- function(x, ...) {
  names(attr(x$terms, "dataClasses")) <- NULL
  names(x$model) <- NULL
  names(attr(attr(x$model, "terms"), "dataClasses")) <- NULL
  attr(attr(x$model, "terms"), ".Environment") <- emptyenv()
  attr(x$terms, ".Environment") <- emptyenv()
  # zero length call should match any call
  x$call <- call(as.character(x$call[[1L]]))
  NextMethod()
}
# nocov start
#' Experimental Abstraction Method for GGPlot
#'
#' Not entirely sure this can ever work well since so much of \code{ggplot} is
#' done with \code{proto} objects and those do not really use meta data, which
#' makes \code{alike} rather useless.
#'
#' @keywords internal
#' @export

abstract.ggplot <- function(x, ...) {
  x <- NextMethod()
  x$data <- data.frame()
  x
}
# nocov end
#' @rdname abstract
#' @export

abstract.environment <- function(x, ...) x

#' @rdname abstract
#' @export

abstract.ts <- function(x, what=c("start", "end", "frequency"), ...) {
  what.valid <- c("start", "end", "frequency")
  if(
    !is.character(what) || any(is.na(what)) || !all(what %in% what.valid)
  )
    stop(
      "Argument `what` must be character with all values in ",
      deparse(what.valid)
    )
  tsp <- attr(x, "tsp")
  if(!is.numeric(tsp) || length(tsp) != 3L)
    stop("Argument `x` must have a \"tsp\" attribute that is numeric(3L)")
  attr(x, "tsp") <- NULL
  x <- abstract.default(x, ...)

  # Set the wildcard values to zero and store as "tsp_vetr" attribute
  # (not "tsp") since R validates tsp and rejects zero frequency.
  # The C comparison code recognizes both "tsp" and "tsp_vetr" as
  # time series template attributes.
  tsp[match(unique(what), what.valid)] <- 0
  attr(x, "tsp_vetr") <- tsp
  x
}
#' Set Element to NULL Without Removing It
#'
#' This function is required because there is no straightforward way to
#' over-write a value in a list with NULL without completely removing the entry
#' from the list as well.
#'
#' This returns a copy of the object modified with null slots; it does
#' not modify the input argument.
#'
#' Default method will attempt to convert non-list objects to lists
#' with \code{\link{as.list}}, and then back to whatever they were by using a
#' function with name \code{paste0("as.", class(obj)[[1L]])}
#' if it exists and works.  If the object cannot be coerced back
#' to its original type the corresponding list will be returned.
#'
#' If this is not appropriate for your object type you can write an S3 method
#' for it.
#'
#' @note attributes are copied from original object and re-applied to final
#'   object before return, which may
#'   not make sense in some circumstances.
#'
#' @export
#' @param obj the R object to NULL a value in
#' @param index an indexing vectors of values to NULL
#' @return object with selected values NULLified
#' @examples
#' nullify(list(1, 2, 3), 2)
#' nullify(call("fun", 1, 2, 3), 2)

nullify <- function(obj, index) {
  UseMethod("nullify")
}
#' @rdname nullify
#' @export

nullify.default <- function (obj, index) {
  not.list <- FALSE
  if(!is.list(obj)) {
    not.list <- TRUE
    class <- class(obj)
    if(inherits(try(obj <- as.list(obj)), "try-error"))
      stop("Could not coerce `obj` to list; see previous error.")
  }
  if(
    !is.character(index) &&
    !(is.numeric(index) && all(index >= 1L)) && !is.logical(index)
  ) {
    stop(
      "Argument `index` must be a valid subsetting index and if numeric must ",
      "be greater than or equal to one."
    )
  }
  seq.along <- seq_along(obj)
  if(is.numeric(index)) {
    if(min(index) < min(seq.along) || max(index) > max(seq.along)) {
      stop(
        "Argument `index` can only contain values that exist within ",
        "seq_along(`obj`)."
      )
    }
    vec.subset <- seq.along %in% index
  } else if(is.logical(index)) {
    if(length(index) > length(obj)) {
      stop("Argument `index` may be no longer than `obj` if it is logical.")
    }
    else if(length(obj) %% length(index) > 0L) {
      warning("Argument `obj` is not a multiple in length of argument `obj`")
    }
    vec.subset <- rep(index, ceiling(length(obj) / length(index)))[seq.along]
  } else if (is.character(index)) {
    ind.match <- index %in% names(obj)
    if(!all(ind.match)) {
      stop(
        "Argument `index` contains values not present in names of `obj` (",
        paste(index[!ind.match], collapse=", "), ")."
      )
    }
    vec.subset <- names(obj) %in% index
  }
  res <- ifelse(vec.subset, lapply(seq.along, function(x) NULL), obj)
  if(not.list) {  # try to reconvert object
    if(
      inherits(
        try(
          res.conv <- eval(call(paste0("as.", class[[1L]]), res)), silent=TRUE
        ),
        "try-error"
      )
    ) {
      warning("Unable to convert object back to class \"", class[[1L]],"\"")
    } else {
      res <- res.conv
    }
  }
  attrs.new <- attributes(res)
  attributes(res) <- if(!is.null(attrs.new))
    modifyList(attributes(obj), attrs.new) else attributes(obj)
  res
}

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

library(vetr)

unitizer_sect("Class Matching", {
  obj2 <- structure(numeric())
  obj1 <- structure(numeric(), class="hello")
  alike(obj1, obj2)
  obj2 <- structure(numeric(), class=c(letters[10:12], letters[1:3], letters[8:9]))
  obj1 <- structure(numeric(), class=letters[1:3])
  alike(obj1, obj2)
  alike(obj2, obj1)
  obj2 <- structure(numeric(), class=c("b", "a", "c"))
  alike(obj1, obj2)
  obj2 <- structure(numeric(), class=c("a", "b", "x", "c"))
  alike(obj1, obj2)
  obj2 <- structure(numeric(), class=c("a", "b", "c"))
  alike(obj1, obj2)      # TRUE
  obj2 <- structure(numeric(), class=c("x", "a", "b", "c"))
  alike(obj1, obj2)      # TRUE
  alike(obj1, obj2, settings=vetr_settings(attr.mode=1))   # FALSE
} )
unitizer_sect("S4", {
  # We used to define classes here, but under unitizer by virtue of vetr being
  # the first package on the search path, that is where the class definitions
  # ended up due to logic in topenv().  That seemed fragile so we switched to
  # defining in package.

  x <- new("vetr_foo")
  y <- new("vetr_foo")
  z <- new("vetr_bar")
  v <- new("vetr_baz")
  w <- structure(list(a=character(), b=numeric()), class="vetr_foo")

  alike(x, y)  # TRUE
  alike(x, z)  # FALSE
  alike(x, w)  # FALSE
  alike(w, x)  # FALSE
  alike(x, v)  # TRUE, because v contains x
  alike(v, x)  # FALSE

  # S4 nested in list

  lst.2 <- list(list(11, 21), list(31, list(41, list(51, list(61)))))
  lst.5 <- lst.6 <- lst.2
  lst.5[[2]][[2]][[1]] <- x
  lst.6[[2]][[2]][[1]] <- v

  alike(lst.5, lst.6)  # TRUE
  alike(lst.6, lst.5)  # FALSE, child class is target, so parent can't match

  # Borked S4
  v2 <- v
  class(v2) <- c("vetr_baz", "vetr_foo")
  alike(x, v2)

  # Stress test installation of `inherits`; right now the inherits command is
  # evaluated in the base environment, which seems to work since the arguments
  # are already evaluated, but it suggests `inherits` can look up S4 definitions
  # irrespective of where they are defined...

  inherits <- function(x, y) stop("pwned!!!")
  alike(y, v)  # TRUE
} )
unitizer_sect("R5", {
  Foo.1 <- vetr:::Foo$new()
  Foo.2 <- vetr:::Foo$new()
  Bar.1 <- vetr:::Bar$new()

  alike(Foo.1, Foo.2)
  alike(Foo.1, Bar.1)

} )
unitizer_sect("Non-Standard Class", {
  # Basically ensure that stuff still recurses even if they are lists/calls
  # but have another class

  var.1 <- list(1, 2, 3)
  var.2 <- list("hello", list(1, 2, 3), 5)
  class(var.1) <- "marbles"
  class(var.2) <- "marbles"
  # "mis-match at index [[1]]: should be integer instead of character"
  alike(var.1, var.2)
} )

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

library(vetr)

unitizer_sect("Atomic", {
  alike(integer(), 1:3)    # TRUE
  alike(integer(5L), 1:3)  # FALSE
  alike(integer(3L), 1:3)  # TRUE
  alike(numeric(), c(1, 2, 3))         # TRUE
  alike(numeric(), 1L)                 # TRUE
  alike(numeric(), c(1.1,.053,41.8))   # TRUE
  alike(integer(3L), 1:3 + .01)
  alike(integer(6L), seq(1/6, 1, 1/6) * 6)     # FALSE, not true integers
  alike(integer(4L), letters[1:4])
  alike(letters[1:4], c("hello", "goodbye", "ba", "da"))  # TRUE

  alike(integer(), NULL)    # FALSE, corner case test

  alike(c(a=1, b=2), 3)         # Length mismatch
  alike(c(a=1, b=2), c(1, 2))   # Names

  # Integer-like ness limits
  alike(1L, 2^31-1)
  alike(1L, 2^31)
  alike(1L, Inf)
  alike(1L, -2^31+1)
  alike(1L, -2^31)
  alike(1L, -Inf)
  alike(1L, NaN)
  alike(1L, NA_real_)
} )
unitizer_sect("lists", {
  lst <-   list(list( 1,  2), list( 3, list( 4, list( 5, list(6, 6.1, 6.2)))))
  lst.2 <- list(list(11, 21), list(31, list(41, list(51, list(61          )))))

  alike(lst, lst.2)     # length mismatch

  lst.3 <- lst.2
  lst.3[[2]][[2]][[2]][[2]] <- matrix(1:9, nrow=3)

  alike(lst, lst.3)     # object type mismatch
  alike(1:10, "hello")  # object type mismatch, no dive

  alike(lst, lst)       # obvious match

  lst.4 <- lst
  lst.4[[2]][[2]] <- list()

  alike(lst.4, lst)     # should match
  alike(lst, lst.4)     # should not match because template has more detail

  # Named lists

  lst.5 <- list(1, list(a = 1, b = 2, c = list(d = 1)))
  lst.6 <- list(1, list(a = 1, b = 2, c = list(d = "hello")))
  lst.5.1 <- list(1, list(a = 1, b = 2, `c d` = list(d = 1)))
  lst.6.1 <- list(1, list(a = 1, b = 2, `c d` = list(d = "hello")))

  alike(lst.5, lst.6)
  alike(lst.6, lst.5)

  alike(lst.5.1, lst.6.1)
  alike(lst.6.1, lst.5.1)

  # Pair lists

  alike(pairlist(a=1, b="character"), pairlist(a=1, b=letters))
  alike(pairlist(1, "character"), pairlist(1, letters))
})
unitizer_sect("NULL values as wildcards", {
  alike(NULL, 1:3)                  # not a wild card at top level
  alike(list(NULL), list(1:3))      # but yes when nested
  alike(list(NULL, NULL), list(list(list(1, 2, 3)), 1:25))
  alike(list(NULL), list(1, 2))
  alike(list(), list(1, 2))
})
unitizer_sect("Matrix / Arrays", {
  alike(matrix(integer(), ncol=7), matrix(1:21, nrow=3))
  alike(matrix(integer(), nrow=3), matrix(1:21, nrow=3))
  alike(matrix(character(), nrow=3), matrix(1:21, nrow=3))
  alike(matrix(integer(), nrow=4), matrix(1:21, nrow=3))
  alike(
    matrix(integer(), ncol=3, dimnames=list(NULL, c("R", "G", "B"))),
    matrix(1:21, ncol=3, dimnames=list(NULL, c("R", "G", "B")))
  )
  alike(
    matrix(integer(), nrow=3, dimnames=list(c("R", "G", "B"), NULL)),
    matrix(1:21, ncol=3, dimnames=list(NULL, c("R", "G", "B")))
  )
  alike(
    matrix(integer(), nrow=3, dimnames=list(c("R", "G", "B"), NULL)),
    matrix(1:9, nrow=3, dimnames=list(NULL, c("R", "G", "B")))
  )
  alike(
    matrix(integer(), nrow=3, dimnames=list(c("R", "G", "B"), NULL)),
    matrix(
      1:9, nrow=3, dimnames=list(c("R", "G", "B"),
      c("bacon", "turkey", "bravo"))
  ) )
  alike(matrix(1:9, nrow = 3), 1:9)

  # Adding tests from docs

  mx.tpl <- matrix(
    integer(), ncol=3, dimnames=list(row.id=NULL, c("R", "G", "B"))
  )
  mx.cur <- matrix(
    sample(0:255, 12), ncol=3, dimnames=list(row.id=1:4, rgb=c("R", "G", "B"))
  )
  mx.cur2 <-
    matrix(sample(0:255, 12), ncol=3, dimnames=list(1:4, c("R", "G", "B")))

  alike(mx.tpl, mx.cur)
  alike(mx.tpl, mx.cur2)

  # dimnames with attributes
  dn0 <- structure(list(c('a', 'b'), 'c'), attr1='a1', attrZ=1)
  dn1 <- structure(list(c('a', 'b'), 'c'), attr1='a1', attrBOB='aa', attrZ=1)
  dn2 <- structure(list(c('a', 'b'), 'c'), attr1='a1', attrJANE='bb', attrZ=1)
  mx.dn0 <- matrix(1:2, ncol=1, dimnames=dn0)
  mx.dn1 <- matrix(1:2, ncol=1, dimnames=dn1)
  mx.dn2 <- matrix(1:2, ncol=1, dimnames=dn2)
  alike(mx.dn0, mx.dn1)  # extra attr skip okay
  alike(mx.dn1, mx.dn0)  # but converse not true
  # Logic slightly different depending on lexical order of attributes
  # for these next two; result should be the similar.
  alike(mx.dn1, mx.dn2)
  alike(mx.dn2, mx.dn1)
} )
unitizer_sect("Data Frames", {
  alike(mtcars, 1:3)
  alike(1:3, mtcars)
  alike(
    data.frame(),
    data.frame(a=1:3, b=letters[1:3])
  )  # TRUE
  alike(       # TRUE, note this is recursive
    data.frame(a=integer(), b=factor()),
    data.frame(a=1:3, b=letters[1:3], stringsAsFactors=TRUE)
  )
  # FALSE mis-match at index[[1]]
  alike(
    data.frame(a=factor(), b=factor()),
    data.frame(a=1:3, b=letters[1:3])
  )
  # FALSE mis-match at index[[2]] (class)
  alike(
    list(NULL, structure("a", class="x")),
    list(NULL, structure("a", class="y"))
  )
  # TRUE, more complex nested structure
  # row.names / names (note use `structure` to get around `data.frame` checks)

  alike(
    list(
      integer(), data.frame(a=integer(), b=numeric()),
      matrix(integer(), nrow=3)
    ),
    list(1:10, data.frame(a=1:200, b=runif(20)), matrix(1:27, nrow=3))
  )

  df.tpl <- structure(
    list(1:4, factor(LETTERS[1:4], levels=LETTERS)),
    row.names=c("one", "", "", ""), names=c("id", ""), class="data.frame"
  )
  df.cur <- `row.names<-`(
    data.frame(id=5:8, val=factor(LETTERS[21:24], levels=LETTERS)),
    c("one", "two", "tre", "qtr")
  )
  df.cur2 <- `row.names<-`(
    data.frame(id=5:8, val=factor(LETTERS[21:24], levels=LETTERS)),
    c("uno", "due", "tre", "qtr")
  )
  alike(df.tpl, df.cur)   # TRUE
  alike(df.cur, df.tpl)   # Nope, names won't match reversed
  alike(df.tpl, df.cur2)  # Nope, row.names won't match

  # NA names

  df.tpl <-
    structure(list(1:4, letters[1:4]), names=c("id", NA), class="data.frame")
  df.cur <-
    structure(list(1:4, letters[1:4]), names=c("id", "val"), class="data.frame")

  alike(df.tpl, df.tpl)
  alike(df.tpl, df.cur)

  # special treatment

  alike(mtcars, iris)
  alike(mtcars, mtcars[1:10,])
  alike(mtcars[-5], mtcars)
})
unitizer_sect("Time Series", {
  ts.1 <- ts(runif(24), 1970, frequency=12)
  ts.2 <- ts(runif(24), 1970, frequency=4)
  ts.3 <- abstract(ts.1, "end")
  ts.4 <- abstract(ts.2, "frequency")

  alike(ts.1, ts.2)
  alike(ts.3, ts.1)
  alike(ts.1, ts.3)
  alike(ts.3, ts.2)

  ts.5 <- ts(matrix(runif(24 * 3), ncol=3), 1970, frequency=12)
  ts.6 <- ts(matrix(runif(12 * 3), ncol=3), 1970, frequency=12)

  alike(ts.5, ts.6)
  alike(ts.5, matrix(runif(24 * 3), ncol=3))
})
unitizer_sect("Factors", {
  f1 <- factor(letters[1:5])
  f2 <- factor(letters[1:5], levels=letters[5:1])
  f3 <- f1
  levels(f3)[[5]] <- ""
  f4 <- factor(c(letters[1:4], "f"))

  alike(f1, f2)   # FALSE
  alike(f1, f3)   # FALSE
  alike(f1, f4)   # FALSE
  alike(f3, f1)   # TRUE, wildcard matches anything
  alike(f3, f4)   # TRUE, wildcard matches anything
})
unitizer_sect("Environments / Pairlists", {
  env0 <- new.env()
  env1 <- list2env(list(a=character(), b=list(), c=NULL))
  env2 <- list2env(list(a="hello", b=iris, c=matrix(1:3)))
  env3 <- list2env(list(a="hello", b=iris))
  env4 <- list2env(list(a="hello", b=iris, c=logical(1L), d=logical(1L)))
  env5 <- list2env(list(b=iris, a="hello", c=matrix(1:3)))

  alike(env0, env2)  # zero length, matches anything
  alike(env1, env2)  # TRUE
  alike(env1, env3)  # length mismatch
  alike(env3, env1)  # component mismatch
  alike(env1, env4)  # TRUE length mismatch but longer allowed
  alike(env1, env5)  # order change, should still match

  # Test infinite recursion protection

  rec.env <- rec.env.cpy <- new.env()

  for(i in 1:50) {
    rec.env.cpy$a <- new.env()
    rec.env.cpy <- rec.env.cpy$a
  }
  rec.env.cpy$a <- rec.env;
  alike(rec.env, rec.env)

  plst1 <- pairlist(a=character(), b=list(), c=NULL)
  plst2 <- pairlist(a="hello", b=iris, c=matrix(1:3))
  plst3 <- pairlist(a="hello", b=iris)
  plst4 <- pairlist(a="hello", b=iris, c=logical(1L), d=logical(1L))
  plst5 <- pairlist(a=character(), b=list(), integer())
  plst6 <- pairlist(a=character(), b=list(), boogey=1:3)
  plst7 <- pairlist(a=character(), boogey=1:3, b=list())

  alike(plst1, plst2)  # TRUE
  alike(plst1, plst3)  # length mismatch
  alike(plst1, plst4)  # length mismatch
  alike(plst1, plst5)  # fail, missing name
  alike(plst5, plst6)  # TRUE, no name matches anything
  alike(plst5, plst7)  # FALSE, order matters in pair lists

  # Nesting

  env7 <- list2env(list(a=character(), b=plst1))
  env8 <- list2env(list(a=letters[1:3], b=plst2))
  env9 <- list2env(list(a=letters[1:3], b=plst5))

  alike(env7, env8)   # pass
  alike(env7, env9)   # fail

  # Overwhelm env nesting

  env.nest.1 <- env.nest.1.cpy <- new.env()
  env.nest.2 <- env.nest.2.cpy <- new.env()
  for(i in 1:26) {
    env.nest.1.cpy[[letters[i]]] <- new.env();
    env.nest.1.cpy <- env.nest.1.cpy[[letters[i]]]
    env.nest.2.cpy[[letters[i]]] <- new.env();
    env.nest.2.cpy <- env.nest.2.cpy[[letters[i]]]
  }
  alike(env.nest.1, env.nest.2, settings=vetr_settings(env.depth.max=16))

  # Global env test

  alike(.GlobalEnv, env.nest.1)
})
unitizer_sect("Calls / Formulas", {
  alike(quote(1 + 1), quote(x + y))
  alike(quote(fun(1 + 1)), quote(fun(x + y, 9)))
  alike(quote(fun(x + y, 9)), quote(fun(1 + 1)))

  # Need to add parens in error messages, which we will illustrate with an
  # operator

  "%plusq%" <- function(x, y) call("+", substitute(x), substitute(y))
  alike(quote(1 + 1), 1 %plusq% b)

  # With defined fun

  fun <- function(a, b, c) NULL
  # TRUE, since constants including NULL match any constants
  alike(quote(fun(b=fun2(x, y), 1, 3)), quote(fun(NULL, fun2(a, b), 1)))
  alike(  # FALSE, match.call disabled
    quote(fun(b=fun2(x, y), 1, 3)), quote(fun(NULL, fun2(a, b), 1)),
    settings=vetr_settings(lang.mode=1)
  )
  # FALSE, mismatch
  alike(quote(fun(b=fun2(x, y), 1, 3)), quote(fun(fun2(a, b), NULL, 1)))
  alike(quote(fun(a=1)), quote(fun(b=1)))  # FALSE, name mismatch

  alike(quote(fun(1, 2)), quote(fun(1)))   # FALSE
  alike(quote(fun(1)), quote(fun(1, 2)))   # FALSE

  alike(quote(fun(1, 2)), quote(fun2(1, 2)))            # FALSE, fun mismatch
  # FALSE, fun mismatch, nested
  alike(quote(fun(1, fun2(3))), quote(fun(1, fun(3))))

  # zero len matches anything

  alike(quote(fun()), quote(fun(a, b, c)))    # TRUE
  alike(quote(fun()), quote(fun2(a, b, c)))   # FALSE, still need match fun names
  alike(quote(fun(a, fun2())), quote(fun(b, fun2(a, b, c))))    # TRUE

  # Attributes on sub-components should not affect anything
  # actually, these tests need to be with alike since lang_alike doesn't check
  # attributes

  c0 <- quote(fun(a, b, a, 25))
  c0.1 <- c0.2 <- c0
  attr(c0.1, "blah") <- "hello"
  attr(c0.2, "blah") <- 1:3

  alike(c0, c0.1)     # TRUE
  alike(c0.1, c0)     # Missing attribute
  alike(c0.1, c0.2)   # Attribute mismatch

  # Formulas

  alike(x ~ y, z ~ w)
  alike(x ~ y, z ~ w + 1)
  alike(x ~ y + 2, z ~ w + 1)
  alike(x ~ y + z:y, w ~ v + u:v)
  alike(z ~ w + 1, x ~ y)
  alike(y ~ x ^ 2 + x * z + z + w:z, q ~ l ^ 2 + l * j + j + w:j)
  alike(y ~ x ^ 2 + x * z + z + w:z, q ~ l ^ 3 + l * j + j + w:j)

  # # Repeating parses to deal with potential parse issues in clean R runs

  exp.1 <- parse(text="x + y; fun2(fun(1, 2, 3), z)", keep.source=TRUE)
  exp.2 <- parse(text="z + 2; fun(fun2(1, 2, 3), q)", keep.source=TRUE)
  exp.3 <- parse(text="z + fun(3); fun(fun2(a, b, c), 3)", keep.source=TRUE)

  alike(exp.1, exp.2)
  alike(exp.2, exp.3)
  alike(exp.3, exp.2)

  exp.4 <- expression(1 + 1, 2 + x)
  exp.5 <- expression(1 + 1, 5 + y)
  exp.6 <- expression(1 + 1, 2 + 2)

  alike(exp.4, exp.5) # TRUE
  alike(exp.4, exp.6) # FALSE

  # Symbols

  alike(quote(x), quote(y))      # TRUE
  alike(NULL, quote(x))          # FALSE, overridden by type comparison
  alike(quote((NULL)), quote(y)) # TRUE, NULL matches anything as language object
  alike(quote(NULL), quote(x))   # FALSE, quoting NULL doesn't make it language
  alike(quote(x), c0)          # FALSE
  alike(c0, quote(x))          # FALSE
  alike(quote((x)), quote(y))    # TRUE, parens shouldn't matter
} )
# Most fun tests in internal/type, here to make sure interface working
unitizer_sect("Functions", {
  alike(print, print.data.frame)              # TRUE
  alike(print.data.frame, print)              # FALSE
  alike(`&&`, function() NULL)                # TRUE

  # check srcref issues

  fun <- fun2 <- function() NULL
  attributes(fun2) <- NULL

  alike(fun, fun2)   # TRUE
  alike(fun, fun2, settings=vetr_settings(attr.mode=2L))
  alike(fun2, fun, settings=vetr_settings(attr.mode=1L))
  alike(fun2, fun, settings=vetr_settings(attr.mode=2L))
})

# Subset of tests for version with settings

unitizer_sect("settings", {
  alike(1L, 1.0, settings=vetr_settings(type.mode=1L))
  alike(1.0, 1L, settings=vetr_settings(type.mode=1L))
  alike(1.0, 1L, settings=vetr_settings(type.mode=2L))   # FALSE
  alike(1:101, 1:101 + 0.0)  # FALSE
  # TRUE
  alike(1:101, 1:101 + 0.0, settings=vetr_settings(fuzzy.int.max.len=200))
  # TRUE
  alike(1:101, 1:101 + 0.0, settings=vetr_settings(fuzzy.int.max.len=-1))
  alike(list(a=1:10), data.frame(a=1:10))
  alike(list(a=1:10), data.frame(a=1:10), settings=vetr_settings(attr.mode=1L))
  # FALSE
  alike(list(a=1:10), data.frame(a=1:10), settings=vetr_settings(attr.mode=2L))

  fun <- function(a, b, c) NULL

  call.a <- quote(fun(b=fun2(x, y), 1, 3))
  call.b <- quote(fun(NULL, fun2(a, b), 1))

  # FALSE, function not defined in empty env

  alike(call.a, call.b, settings=vetr_settings(env=emptyenv()))

  # FALSE, also not defined this way

  alike(call.a, call.b, env=emptyenv())

  # TRUE, verify that settings takes precedence
  alike(
    call.a, call.b, env=emptyenv(), settings=vetr_settings(env=environment())
  )
  # TRUE
  alike(
    call.a, call.b
  )
  # FALSE

  alike(`&&`, function() NULL, settings=vetr_settings(type.mode=1))

  # Error

  alike(1, 2, settings=letters)
  alike(1, 2, settings=list())
  alike(1, 2, settings=setNames(vector("list", 16), letters[1:16]))
  alike(1, 2, settings=vector("list", 16))
} )
# These are also part of the examples, but here as well so that issues are
# detected during development and not the last minute package checks

unitizer_sect("Examples", {
  alike(1L, 1.0)         # TRUE, because 1.0 is integer-like
  alike(1L, 1.1)         # FALSE, 1.1 is not integer-like
  alike(1.1, 1L)         # TRUE, by default, integers are always considered real

  alike(1:100, 1:100 + 0.0)  # TRUE
  # FALSE, we do not check numerics for integerness if longer than 100
  alike(1:101, 1:101 + 0.0)

  # Scalarness can now be checked at same time as type

  alike(integer(1L), 1)            # integer-like and length 1?
  alike(logical(1L), TRUE)         # logical and length 1?
  alike(integer(1L), 1:3)
  alike(logical(1L), c(TRUE, TRUE))

  # Zero length match any length of same type

  alike(integer(), 1:10)
  alike(1:10, integer())   # but not the other way around

  # Recursive objects compared recursively

  alike(
    list(integer(), list(character(), logical(1L))),
    list(1:10, list(letters, TRUE))
  )
  alike(
    list(integer(), list(character(), logical(1L))),
    list(1:10, list(letters, c(TRUE, FALSE)))
  )
  # `NULL` is a wild card when nested within recursive objects

  alike(list(NULL, NULL), list(iris, mtcars))
  alike(NULL, mtcars)    # but not at top level

  # Since `data.frame` are lists, we can compare them recursively:

  iris.fake <- transform(iris, Species=as.character(Species))
  alike(iris, iris.fake)
  iris.fake2 <- transform(
    iris,
    Species=factor(Species, levels="[[<-"(levels(Species), 3, "americana"))
  )
  alike(iris, iris.fake2)  # we even check attributes (factor levels must match)!

  # We can use partially specified objects as templates

  iris.tpl <- abstract(iris)
  str(iris.tpl)
  alike(iris.tpl, iris)
  # any row sample of iris matches our iris template
  alike(iris.tpl, iris[sample(1:nrow(iris), 10), ])
  alike(iris.tpl, iris[c(2, 1, 3, 4, 5)])              # but column order matters

  # Also works with matrices / arrays

  alike(matrix(integer(), 3, 3), matrix(1:9, nrow=3))         # 3 x 3 integer
  # 3 x 3, but not integer!
  alike(matrix(integer(), 3, 3), matrix(runif(9), nrow=3))
  # partial spec, any 3 row integer matrix
  alike(matrix(integer(), 3), matrix(1:12, nrow=3))
  alike(matrix(integer(), 3), matrix(1:12, nrow=4))
  # Any logical matrix (but not arrays)
  alike(matrix(logical()), array(rep(TRUE, 8), rep(2, 3)))

  # In order for objects to be alike, they must share a family tree, not just
  # a common class

  obj.tpl <- structure(TRUE, class=letters[1:3])
  obj.cur.1 <-  structure(TRUE, class=c("x", letters[1:3]))
  obj.cur.2 <-  structure(TRUE, class=c(letters[1:3], "x"))

  alike(obj.tpl, obj.cur.1)
  alike(obj.tpl, obj.cur.2)

  # You can compare language objects; these are alike if they are self
  # consistent; we don't care what the symbols are, so long as they are used
  # consistently across target and current:

  # TRUE, symbols are consistent (adding two different symbols)
  alike(quote(x + y), quote(a + b))
  alike(quote(x + y), quote(a - b))   # FALSE, different function
  alike(quote(x + y), quote(a + a))   # FALSE, inconsistent symbols
} )
unitizer_sect("Raw", {
  # check for warning, in the future if we properly support RAW then this will
  # no longer produce a warning.  Really just looking for a valid STRSXP type.

  alike(as.raw(integer(3)), as.raw(integer(3)))
})
unitizer_sect("Errors", {
  alike(NULL, NULL, settings=vetr_settings(type.mode=3))
  alike(NULL, NULL, settings=vetr_settings(attr.mode=letters))
  alike(NULL, NULL, settings=vetr_settings(lang.mode=letters))
  alike(NULL, NULL, settings=vetr_settings(fuzzy.int.max.len=NA_integer_))
  alike(NULL, NULL, settings=vetr_settings(suppress.warnings=NA))
  alike(NULL, NULL, settings=vetr_settings(env=letters))
  alike(NULL, NULL, settings=vetr_settings(width=letters))
  alike(NULL, NULL, settings=vetr_settings(env.depth.max=-1L))
})
unitizer_sect("Attributes", {
  # There are implicit and explicit attribute checks elsewhere, this
  # is just for a corner case that showed up in coverage

  obj.tpl <- structure(integer(), a=integer())
  obj.obj <- structure(1:10, a=1:3)

  alike(obj.tpl, obj.obj)

  # issue93 More corner cases from the transition to sorted attribute checks

  obj.tpl.a <- structure(integer(), class='foo')
  obj.obj.a <- structure(matrix(1:3))

  alike(obj.tpl.a, obj.obj.a)

  obj.tpl.b <- structure(matrix(integer()), class='foo')
  obj.obj.b <- structure(matrix(1:3))

  alike(obj.tpl.b, obj.obj.b)

  obj.tpl.c <- structure(matrix(integer()), class='foo')
  obj.obj.c <- structure(matrix(1:3), class='foo')

  alike(obj.tpl.c, obj.obj.c)

  obj.tpl.d <- structure(integer(), class='foo')
  obj.obj.d <- structure(matrix(1:3), class='foo')

  alike(obj.tpl.d, obj.obj.d)

  obj.tpl.e <- structure(integer())
  obj.obj.e <- structure(matrix(1:3), class='foo')

  alike(obj.tpl.e, obj.obj.e)

  obj.tpl.f <- structure(integer(), a=integer())
  obj.obj.f <- structure(matrix(1:3), z=integer(), class='foo')

  alike(obj.tpl.f, obj.obj.f)

  obj.tpl.g <- structure(integer(), a=integer())
  obj.obj.g <- structure(matrix(1:3), z=integer(), class='foo')

  alike(obj.tpl.g, obj.obj.g)

  obj.tpl.h <- structure(matrix(integer()))
  obj.obj.h <- structure(1:3, a=integer())

  alike(obj.tpl.h, obj.obj.h)
  alike(obj.tpl.h, obj.obj.h, settings=vetr_settings(attr.mode=2))

  obj.tpl.i <- structure(character(), a=integer(), class='boo')
  obj.obj.i <- structure(matrix(1:3))

  alike(obj.tpl.i, obj.obj.i)

  obj.tpl.k <- integer()
  obj.obj.k <- matrix(1:3)

  alike(obj.tpl.k, obj.obj.k)
  alike(obj.tpl.k, obj.obj.k, settings=vetr_settings(attr.mode=2))
})

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

#' Compare Object Structure
#'
#' Similar to \code{\link{all.equal}}, but compares object structure rather than
#' value.  The \code{target} argument defines a template that the \code{current}
#' argument must match.
#'
#' @section alikeness:
#'
#' Generally speaking two objects are alike if they are of the same type (as
#' determined by \code{\link{type_alike}}) and length.  \code{\link{type_alike}}
#' has special treatment for integer-like numerics and function-like objects.
#'
#' Attributes on the objects are required to be recursively \code{alike}, though
#' the following attributes are treated specially: \code{class}, \code{dim},
#' \code{dimnames}, \code{names}, \code{row.names}, \code{levels}, \code{tsp},
#' and \code{srcref}.
#'
#' Exactly what makes two objects \code{alike} is complex, but should be
#' intuitive.  The best way to understand "alikeness" is to review the examples.
#' For a thorough exposition see \href{../doc/alike.html}{the vignette}.
#'
#' @note The semantics of alikeness for language objects, formulas, and
#' functions may change in the future.
#'
#' @export
#' @seealso \code{\link{type_alike}}, \code{\link{type_of}},
#'   \code{\link{abstract}}, \code{\link{vetr_settings}} for more control of
#'   settings
#' @param target the template to compare the object to
#' @param current the object to determine alikeness of to the template
#' @param settings a list of settings generated using \code{vetr_settings}, NULL
#'   for default
#' @param env environment used internally when evaluating expressions; currently
#'   used only when looking up functions to \code{\link{match.call}} when
#'   testing language objects, note that this will be overridden by the
#'   environment specified in \code{settings} if any, defaults to the parent
#'   frame.
#' @return TRUE if target and current are alike, character(1L) describing why
#'   they are not if they are not
#' @examples
#' ## Type comparison
#' alike(1L, 1.0)         # TRUE, because 1.0 is integer-like
#' alike(1L, 1.1)         # FALSE, 1.1 is not integer-like
#' alike(1.1, 1L)         # TRUE, by default, integers are always considered real
#'
#' alike(1:100, 1:100 + 0.0)  # TRUE
#'
#' ## We do not check numerics for integerness if longer than 100
#' alike(1:101, 1:101 + 0.0)
#'
#' ## Scalarness can now be checked at same time as type
#' alike(integer(1L), 1)            # integer-like and length 1?
#' alike(logical(1L), TRUE)         # logical and length 1?
#' alike(integer(1L), 1:3)
#' alike(logical(1L), c(TRUE, TRUE))
#'
#' ## Zero length match any length of same type
#' alike(integer(), 1:10)
#' alike(1:10, integer())   # but not the other way around
#'
#' ## Recursive objects compared recursively
#' alike(
#'   list(integer(), list(character(), logical(1L))),
#'   list(1:10, list(letters, TRUE))
#' )
#' alike(
#'   list(integer(), list(character(), logical(1L))),
#'   list(1:10, list(letters, c(TRUE, FALSE)))
#' )
#'
#' ## `NULL` is a wild card when nested within recursive objects
#' alike(list(NULL, NULL), list(iris, mtcars))
#' alike(NULL, mtcars)    # but not at top level
#'
#' ## Since `data.frame` are lists, we can compare them recursively:
#' iris.fake <- transform(iris, Species=as.character(Species))
#' alike(iris, iris.fake)
#'
#' ## we even check attributes (factor levels must match)!
#' iris.fake2 <- iris
#' levels(iris.fake2$Species) <- c("setosa", "versicolor", "africana")
#' alike(iris, iris.fake2)
#'
#' ## We can use partially specified objects as templates
#' iris.tpl <- abstract(iris)
#' str(iris.tpl)
#' alike(iris.tpl, iris)
#' ## any row sample of iris matches our iris template
#' alike(iris.tpl, iris[sample(1:nrow(iris), 10), ])
#' ## but column order matters
#' alike(iris.tpl, iris[c(2, 1, 3, 4, 5)])
#'
#' ## 3 x 3 integer
#' alike(matrix(integer(), 3, 3), matrix(1:9, nrow=3))
#' ## 3 x 3, but not integer!
#' alike(matrix(integer(), 3, 3), matrix(runif(9), nrow=3))
#' ## partial spec, any 3 row integer matrix
#' alike(matrix(integer(), 3), matrix(1:12, nrow=3))
#' alike(matrix(integer(), 3), matrix(1:12, nrow=4))
#' ## Any logical matrix (but not arrays)
#' alike(matrix(logical()), array(rep(TRUE, 8), rep(2, 3)))
#'
#' ## In order for objects to be alike, they must share a family
#' ## tree, not just a common class
#' obj.tpl <- structure(TRUE, class=letters[1:3])
#' obj.cur.1 <-  structure(TRUE, class=c("x", letters[1:3]))
#' obj.cur.2 <-  structure(TRUE, class=c(letters[1:3], "x"))
#'
#' alike(obj.tpl, obj.cur.1)
#' alike(obj.tpl, obj.cur.2)
#'
#' ## You can compare language objects; these are alike if they are self
#' ## consistent; we don't care what the symbols are, so long as they are used
#' ## consistently across target and current:
#'
#' ## TRUE, symbols are consistent (adding two different symbols)
#' alike(quote(x + y), quote(a + b))
#' ## FALSE, different function
#' alike(quote(x + y), quote(a - b))
#' ## FALSE, inconsistent symbols
#' alike(quote(x + y), quote(a + a))

alike <- function(target, current, env=parent.frame(), settings=NULL)
  .Call(VALC_alike_ext, target, current, substitute(current), env, settings)

# we go through the trouble of saving stuff to RDS to quash valgrind errors
# caused by heap alloc'ed strings that are strsplit with perl=TRUE

lorem <- "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
Encoding(lorem) <- "UTF-8"

# lorem.phrases <- unlist(strsplit(lorem, "[.,]\\K ", perl=TRUE))
# saveRDS(lorem.phrases, 'unitizer/_pre/lorem.objs/lorem.RDS')
lorem.phrases <- readRDS('unitizer/_pre/lorem.objs/lorem.RDS')

# From the Stalin russian wiki page:
#
# <https://ru.wikipedia.org/w/index.php?title=%D0%A1%D1%82%D0%B0%D0%BB%D0%B8%D0%BD,_%D0%98%D0%BE%D1%81%D0%B8%D1%84_%D0%92%D0%B8%D1%81%D1%81%D0%B0%D1%80%D0%B8%D0%BE%D0%BD%D0%BE%D0%B2%D0%B8%D1%87&oldid=12832996>
#
# Licensed under GNU Free Documentation License since it is from before 2009

# lorem.ru <- "Родился 6 (18) декабря 1878 года (по записи в метрической книге Горийской Успенской соборной церкви[6]Это подлинная дата рождения Сталина. Позже она подтверждалась в уведомлении Санкт-Петербургского жандармского управления и самим Сталиным в ответе на анкету шведской газеты «Folkets Dagblad Politiken» в 1920 г.[7]) в Грузии в городе Гори, хотя начиная с 1929 года[источник?] днём его рождения официально считалось 9 (21) декабря 1879. Был третьим сыном в семье, первые двое умерли в младенчестве. Его родным языком был грузинский, русский язык Сталин выучил позже, но всегда говорил с заметным грузинским акцентом [8]. Согласно утверждениям дочери Светланы, Сталин, однако, пел по-русски практически без акцента."
# Encoding(lorem.ru) <- "UTF-8"
# lorem.ru.phrases <- unlist(strsplit(lorem.ru, "[.,]\\K ", perl=TRUE))
# saveRDS(lorem.ru.phrases, 'unitizer/_pre/lorem.objs/lorem.ru.RDS')
lorem.ru.phrases <- readRDS('unitizer/_pre/lorem.objs/lorem.ru.RDS')

# From the Stalin turkish wiki page:
#
# <https://tr.wikipedia.org/w/index.php?title=Josef_Stalin&oldid=4719847>
#
# Licensed under GNU Free Documentation License since it is from before 2009

# lorem.tr <- "Bu tartışmalı tarihsel dönemle ilgili olarak, Stalin'e düşman veya Stalin'den yana olan her iki tarafın da farklı tezleri vardır. Stalin karşıtlarının tezlerine göre, Hitlerle aralarındaki açıklanmayan gizli protokole bağlı olarak Finlandiya, Estonya, Letonya, Litvanya, Romanya ve Polonya'nin Naziler veya Sovyetler tarafından işgalinin yolu açılmıştır. Stalin'in doğru yaptığını savunanlara göre ise, 1937'deki Münih görüşmelerinde açıkça ortaya çıktığı gibi, İngiliz ve Fransız emperyalistleri ve dolaylı olarak da Amerikalılar, Nazileri kışkırtıyorlardı ve onların Sovyetler Birliği'ne saldırısının önünü açmaya çalışıyorlardı. Bu amaçla Avusturya'nın Almanya'ya katılmasına (Anschluss) ve Çekoslovakya'nın işgaline göz yummuş ve onaylamışlardı.Ne var ki, özellikle Çekoslovakya'nın işgalinden sonra Sovyetler Birliği'nin İngiltere ve Fransa ile ilişki kurma çabalarına rağmen bu iki ülke Nazi tehdidini birlikte ortadan kaldırma girişimini reddetti. Böylece Sovyetler Birliği, kendi sınırlarını güvence altına almak için bu protokolü imzaladı. Stalin'in amaçlarına göre, Polonya ve Baltık ülkelerinde oluşturulacak tampon bölgeler, Nazilerin Sovyetler Birliği'ne ulaşmasını engelleyecekti. Böylece 1939 yılında Nazi işgalinden sonra Sovyetler Polonya'nın kalan yarısını işgal edip Estonya, Litvanya ve Letonya'yı sınırlarına kattı. Finlandiya'ya saldırdı ve büyük kayıplar vermesine rağmen Mart 1940'da \"kış savaşı' olarak bilinen bu savaşı da kazandı. 1941'de Hitler'in Sovyetlere saldırması üzerine Stalin bu sefer müttefiklerin yanında yer aldı. II. Dünya Savaşı'nın en ağır bedeli ödeyen güç olarak (24 milyon ölü) müttefiklerin yanında Nazi Almanyası'na karşı kazandığı zafer uluslararası alanda gücünü artırdı."
# Encoding(lorem.tr) <- "UTF-8"
# lorem.tr.phrases <- unlist(strsplit(lorem.tr, "[.,]\\K ", perl=TRUE))
# saveRDS(lorem.tr.phrases, 'unitizer/_pre/lorem.objs/lorem.tr.RDS')
lorem.tr.phrases <- readRDS('unitizer/_pre/lorem.objs/lorem.tr.RDS')

# From the PRC chinese wiki page:
#
# <https://zh.wikipedia.org/w/index.php?title=%E4%B8%AD%E5%8D%8E%E4%BA%BA%E6%B0%91%E5%85%B1%E5%92%8C%E5%9B%BD&oldid=8944743>
#
# Licensed under GNU Free Documentation License since it is from before 2009

# lorem.cn <- "中華人民共和國是單一制的多民族國家。全國劃分為23個省（其中台灣省并沒有實際管辖）、5個自治區、4個直轄市和2個根據一國兩制設立的特別行政區，均直屬於中央人民政府。中华人民共和国跨越五个地理时区，但全国均使用北京时间（UTC+8，东八区）作为标准时间。中華人民共和國官方認定的民族現有56個，其中最大民族汉族佔總人口的91.59%，其餘55族統稱為少数民族，所有民族統稱為中華民族；除回族外，其他54個少数民族如壮族、维吾尔族、滿族、蒙古族、藏族、朝鲜族等也多使用自己的語言與文字。主要宗教有佛教、道教、基督教（多指新教）、天主教和伊斯兰教等，但過半人口無特定宗教信仰。中华人民共和国的通用语言是汉语普通話，當中在中國大陸通行的漢字為簡體字，而在港澳台地區通行的漢字則為繁體字。"
# Encoding(lorem.cn) <- "UTF-8"
# cn.split <-  "[、。]\\K"
# Encoding(cn.split) <- "UTF-8"
# lorem.cn.phrases <- unlist(strsplit(lorem.cn, cn.split, perl=TRUE))
#
# saveRDS(lorem.cn.phrases, 'unitizer/_pre/lorem.objs/lorem.cn.RDS')
lorem.cn.phrases <- readRDS('unitizer/_pre/lorem.objs/lorem.cn.RDS')

# Emoji from Unicode Site <http://unicode.org/emoji/charts/full-emoji-list.html>

# emoji <- c(
#   "\U0001f600", "\U0001f619", "\U0001f61a", "\U0001f642", "\U0001f92f",
#   "\U0001f62c", "\U0001f630", "\U0001f631", "\U0001f633", "\U0001f92a",
#   "\U0001f635", "\U0001f637", "\U0001f912", "\U0001f915", "\U0001f922",
#   "\U0001f92e", "\U0001f927", "\U0001f607", "\U0001f920", "\U0001f921",
#   "\U0001f925", "\U0001f92b", "\U0001f92d", "\U0001f9d0", "\U0001f913",
#   "\U0001f608", "\U0001f4a9", "\U0001f63e",
#   "\U0001f469\U0001f3ff\U0000200d\U0001f3eb",
#   "\U0001f468\U0001f3fb\U0000200d\U00002696\U0000FE0F", "\U0001f46b",
#   "\U0001f469\U0000200D\U00002764\U0000200d\U0001f48b\U0000200d\U0001f468",
#   "\U0001f468\U0000200d\U0001f468\U0000200d\U0001f467",
#   "\U0001f468\U0000200d\U0001f468\U0000200d\U0001f467\U0000200d\U0001f466",
#   "\U0001f984", "\U0001f36b", "\U0001f1e6\U0001f1f7", "\U0001f1ed\U0001f1f0",
#   "\U0001f1ef\U0001f1f5", "\U0001f1f0\U0001f1f5", "\U0001f1fa\U0001f1f8",
#   "\U0001f3f4"
# )
# Encoding(emoji) <- "UTF-8"
# lorem.emo <- paste(
#   sample(
#     c(emoji, rep(", ", 3), rep(". ", 3), rep(" ", 40)), 450, replace=TRUE
#   ),
#   collapse=""
# )
# lorem.emo.phrases <- unlist(strsplit(lorem.emo, "[,.]\\K", perl=TRUE))

cat(getwd(), "\n")
if(suppressWarnings(require('unitizer'))) {
  local({
    suppressWarnings(RNGversion("3.5.2"));
    on.exit({
      RNGversion(as.character(getRversion()))
    })
    pattern <- 'alike'
    unitize_dir(
      'unitizer',
      # pattern=pattern,
      state='recommended'
    )
  })
} else {
  warning("Cannot run tests without package `unitizer`")
}

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

library(vetr)

set.seed(42)
x <- runif(100)
x[1] <- 1
x[2] <- 0

unitizer_sect('all_bw - real', {
  all_bw(x, 0, 1)
  all_bw(x, 0, 1, bounds="[)") # fail
  all_bw(x, 0, 1, bounds="(]") # fail
  all_bw(x, 0, 1, bounds="()") # fail

  all_bw(x, 0, 1 + 1e-6, bounds="[)") # pass
  all_bw(x, 0 - 1e-6, 1, bounds="(]") # pass
  all_bw(x, 0 - 1e-6, 1 + 1e-6, bounds="()") # pass

  y <- z <- x
  y[50] <- NA
  z[50] <- NaN

  all_bw(y, 0, 1)                           # fail
  all_bw(y, 0, 1, na.rm=TRUE)               # pass
  all_bw(y, 0.5, .75, na.rm=TRUE)           # fail

  all_bw(y, -1, 2, na.rm=TRUE, bounds="()") # pass
  all_bw(y, 0, 1, na.rm=TRUE, bounds="()")  # fail
  all_bw(y, 0 - 1e-6, 1, na.rm=TRUE, bounds="(]")  # pass
  all_bw(y, 0, 1, na.rm=TRUE, bounds="(]")  # fail
  all_bw(y, 0, 1 + 1e-6, na.rm=TRUE, bounds="[)")  # pass
  all_bw(y, 0, 1, na.rm=TRUE, bounds="[)")  # fail

  all_bw(z, 0, 1)              # fail
  all_bw(z, 0, 1, na.rm=TRUE)  # pass

  all_bw(z) # fail, NaN never inside
  all_bw(z, na.rm=TRUE) # pass

  w <- runif(1e3, -1e3, 1e3)
  all_bw(w, -1e3, 1e3)
  all_bw(w, -1.5e3, 0.5e3)
  all_bw(w, -0.5e3, 1.5e3)
})
unitizer_sect('corner cases - real', {
  all_bw(x, 0, 0)                        # fail
  all_bw(0, 0, 0)                        # pass
  all_bw(0, 0, 0, bounds="()")   # fail

  all_bw(NA_real_)               # fail
  all_bw(NA_real_, bounds="()")  # fail

  all_bw(numeric(), 0, 1)  # pass
  all_bw(numeric(), 0, 0, bounds="()")  # pass?
})
unitizer_sect('Infinity - real', {

  n1e100 <- 1e100
  n_1e100 <- -1e100
  n2e100 <- 2e100
  n_2e100 <- -2e100
  n11e100 <- 1.1e100
  n_11e100 <- -1.1e100
  n1e200 <- 1e200
  n_1e200 <- -1e200

  z <- runif(100, n_1e100, n1e100)
  z[1] <- n_1e100
  z[2] <- n1e100

  r <- w <- v <- u <- x <- z

  # Infinitiy in bounds

  all_bw(z, -Inf, n1e100) # pass
  all_bw(z, -Inf, n1e100, bounds="[)") # fail
  all_bw(z, -Inf, n1e100, bounds="()") # fail
  all_bw(z, -Inf, n1e100, bounds="(]") # pass
  all_bw(c(z, n2e100), -Inf, n1e100, bounds="(]") # fail

  all_bw(z, -Inf, n11e100, bounds="[)") # pass
  all_bw(z, -Inf, n11e100, bounds="()") # pass

  all_bw(z, n_1e100, Inf) # pass
  all_bw(z, n_1e100, Inf, bounds="(]") # fail
  all_bw(z, n_1e100, Inf, bounds="()") # fail
  all_bw(z, n_1e100, Inf, bounds="[)") # pass
  all_bw(c(z, n_2e100), n_1e100, Inf, bounds="[)") # fail

  all_bw(z, n_11e100, Inf, bounds="(]") # pass
  all_bw(z, n_11e100, Inf, bounds="()") # pass

  # Infinity + NA

  r[50] <- NA_real_

  all_bw(r, -Inf, n1e100)                # fail
  all_bw(r, -Inf, n1e100, bounds="[)")   # fail
  all_bw(r, -Inf, n1e100, bounds="()")   # fail
  all_bw(r, -Inf, n1e100, bounds="(]")   # fail

  all_bw(r, -Inf, n11e100, bounds="[)")  # fail
  all_bw(r, -Inf, n11e100, bounds="()")  # fail

  all_bw(r, n_1e100, Inf)  # fail
  all_bw(r, n_1e100, Inf, bounds="(]")  # fail
  all_bw(r, n_1e100, Inf, bounds="()")  # fail
  all_bw(r, n_1e100, Inf, bounds="[)")  # fail

  all_bw(r, n_11e100, Inf, bounds="(]")  # fail
  all_bw(r, n_11e100, Inf, bounds="()")  # fail

  all_bw(r, -Inf, n1e100, na.rm=TRUE)  # pass
  all_bw(r, -Inf, n1e100, bounds="[)", na.rm=TRUE)  # fail
  all_bw(r, -Inf, n1e100, bounds="()", na.rm=TRUE)  # fail
  all_bw(r, -Inf, n1e100, bounds="(]", na.rm=TRUE)  # pass

  all_bw(r, -Inf, n11e100, bounds="[)", na.rm=TRUE) # pass
  all_bw(r, -Inf, n11e100, bounds="()", na.rm=TRUE) # pass

  all_bw(r, n_1e100, Inf, na.rm=TRUE) # pass
  all_bw(r, n_1e100, Inf, bounds="(]", na.rm=TRUE)  # fail
  all_bw(r, n_1e100, Inf, bounds="()", na.rm=TRUE)  # fail
  all_bw(r, n_1e100, Inf, bounds="[)", na.rm=TRUE)  # pass

  all_bw(r, n_11e100, Inf, bounds="(]", na.rm=TRUE) # pass
  all_bw(r, n_11e100, Inf, bounds="()", na.rm=TRUE) # pass

  # special loop case

  all_bw(c(r, n2e100), -Inf, n1e100, na.rm=TRUE)  # fail
  all_bw(c(r, n_2e100), n_1e100, Inf, na.rm=TRUE) # fail

  # Infinity in values

  all_bw(z, -Inf, Inf)  # pass

  u[50] <- -Inf

  all_bw(u, n_1e200, n1e200)    # fail

  v[50] <- Inf

  all_bw(v, n_1e200, n1e200)    # fail

  w[50] <- -Inf
  w[51] <- Inf

  all_bw(w, -Inf, Inf)        # pass?
  all_bw(w, -Inf, Inf, bounds="[)")  # fail
  all_bw(w, -Inf, Inf, bounds="(]")  # fail
})

x.int <- sample(-50:50)

unitizer_sect('all_bw - int', {
  all_bw(x.int, -50, 50)
  all_bw(x.int, -50L, 50L)
  all_bw(x.int, -50, 50, bounds="[)") # fail
  all_bw(x.int, -50, 50, bounds="(]") # fail
  all_bw(x.int, -50, 50, bounds="()") # fail

  all_bw(x.int, -50, 50 + 1e-6, bounds="[)") # pass
  all_bw(x.int, -50 - 1e-6, 50, bounds="(]") # pass
  all_bw(x.int, -50 - 1e-6, 50 + 1e-6, bounds="()") # pass

  y.int <- z.int <- x.int
  y.int[50] <- NA

  all_bw(y.int, -50, 50)                           # fail
  all_bw(y.int, -50, 50, na.rm=TRUE)               # pass
  all_bw(y.int, -49.5, 49.5, na.rm=TRUE)           # fail

  all_bw(y.int, -51, 51, na.rm=TRUE, bounds="()") # pass
  all_bw(y.int, -50.5, 50.5, na.rm=TRUE, bounds="()") # pass
  all_bw(y.int, -50, 50, na.rm=TRUE, bounds="()")  # fail
  all_bw(y.int, -50 - 1e-6, 50, na.rm=TRUE, bounds="(]")  # pass
  all_bw(y.int, -50, 50, na.rm=TRUE, bounds="(]")  # fail
  all_bw(y.int, -50, 50 + 1e-6, na.rm=TRUE, bounds="[)")  # pass
  all_bw(y.int, -50, 50, na.rm=TRUE, bounds="[)")  # fail
})
unitizer_sect('corner cases - int', {
  all_bw(x.int, 0, 0)                        # fail
  all_bw(0L, 0, 0)                        # pass
  all_bw(0L, 0, 0, bounds="()")   # fail

  all_bw(NA_integer_)            # fail
  all_bw(NA)                     # fail
})
unitizer_sect('Infinity - int', {
  int.max <- (Reduce(`*`, rep(2L, 30L)) - 1L) * 2L + 1L
  int.min <- -int.max

  z.int <- x.int
  z.int[1] <- int.max
  z.int[2] <- int.min

  r.int <- w.int <- v.int <- u.int <- x.int <- z.int

  # Infinitiy in bounds

  all_bw(z.int, -Inf, int.max) # pass
  all_bw(z.int, -Inf, int.max, bounds="[)") # fail
  all_bw(z.int, -Inf, int.max, bounds="()") # fail
  all_bw(z.int, -Inf, int.max, bounds="(]") # pass
  all_bw(z.int, -Inf, int.max - 1L, bounds="(]") # fail

  all_bw(z.int, int.min - 1, int.max + 1) # pass
  all_bw(z.int, int.min - 1, int.max + 1, bounds="()") # pass?

  all_bw(int.max - 1L, -Inf, int.max, bounds="()") # pass

  all_bw(z.int, int.min, Inf) # pass
  all_bw(z.int, int.min, Inf, bounds="(]") # fail
  all_bw(z.int, int.min, Inf, bounds="()") # fail
  all_bw(z.int, int.min, Inf, bounds="[)") # pass
  all_bw(z.int, int.min + 1L, Inf, bounds="[)") # fail

  # Infinity + NA

  r.int[50] <- NA_integer_

  all_bw(r.int, -Inf, int.max)
  all_bw(r.int, -Inf, int.max, bounds="[)")
  all_bw(r.int, -Inf, int.max, bounds="()")
  all_bw(r.int, -Inf, int.max, bounds="(]")
  all_bw(r.int, -Inf, int.max - 10, bounds="(]")  # fail

  all_bw(z.int, -Inf, int.max - 1L, bounds="(]", na.rm=TRUE) # fail
  all_bw(c(int.max - 1L, NA), -Inf, int.max, bounds="()", na.rm=TRUE) # pass

  all_bw(r.int, int.min, Inf)
  all_bw(r.int, int.min, Inf, bounds="(]")
  all_bw(r.int, int.min, Inf, bounds="()")
  all_bw(r.int, int.min, Inf, bounds="[)")

  all_bw(r.int, -Inf, int.max, na.rm=TRUE)
  all_bw(r.int, -Inf, int.max, bounds="[)", na.rm=TRUE)
  all_bw(r.int, -Inf, int.max, bounds="()", na.rm=TRUE)
  all_bw(r.int, -Inf, int.max, bounds="(]", na.rm=TRUE)

  all_bw(r.int, int.min + 10, Inf, bounds="[)", na.rm=TRUE)  # fail

  all_bw(r.int, int.min, Inf, na.rm=TRUE)
  all_bw(r.int, int.min, Inf, bounds="(]", na.rm=TRUE)
  all_bw(r.int, int.min, Inf, bounds="()", na.rm=TRUE)
  all_bw(r.int, int.min, Inf, bounds="[)", na.rm=TRUE)
})
unitizer_sect('error', {
  all_bw(x, 0, -1)
  all_bw(x, -1, 1, na.rm=1)
  all_bw(x, -1, 1, na.rm=c(TRUE, FALSE))
  all_bw(x, -1, 1, na.rm=NA)

  all_bw(x, -1, 1, bounds=TRUE)
  all_bw(x, -1, 1, bounds=letters)
  all_bw(x, -1, 1, bounds="[[")
  all_bw(x, -1, 1, bounds="))")
  all_bw(x, -1, 1, bounds="[")
  all_bw(x, -1, 1, bounds="[.]")
  all_bw(x, -1, 1, bounds=NA_character_)

  all_bw(x, 1:3, 4)
  all_bw(x, 1, 4:5)

  all_bw(list(), 1, 2)
  all_bw(x, list(), 2)
  all_bw(x, 1, list())
  all_bw(x, list(1), 2)
  all_bw(x, 1, list(1))
  all_bw(x, "a", 1)
  all_bw(x, 1, "a")
})

unitizer_sect('all_bw - strings', {

  two.let <- two.let.na <- two.let.inf <- c(
    letters,
    do.call(paste0, expand.grid(letters, letters))
  )
  all_bw(letters, "a", "z")
  all_bw(letters, "z", "a") # error
  all_bw(letters, "a", "z", bounds="[)")
  all_bw(letters, "a", "z", bounds="(]")

  all_bw(two.let, "aa", "zz") # no, "a" is less

  all_bw(two.let, "a", "zz")
  all_bw(two.let, "a", "zz", bounds="()")
  all_bw("A", "a", "z", bounds="(]")

  # exclude ends, but still pass

  two.let.2 <- tail(head(two.let, -1), -1)
  all_bw(two.let.2, "a", "zz", bounds="()")
  all_bw(two.let.2, "a", "zz", bounds="[)")
  all_bw(two.let.2, "a", "zz", bounds="(]")

  two.let.2[50] <- NA_character_
  all_bw(two.let.2, "a", "zz", bounds="()", na.rm=TRUE)
  all_bw(two.let.2, "a", "zz", bounds="[)", na.rm=TRUE)
  all_bw(two.let.2, "a", "zz", bounds="(]", na.rm=TRUE)

  # inf bounds

  all_bw(two.let, -Inf, Inf, bounds="()")
  all_bw(two.let, -Inf, Inf, bounds="[)")
  all_bw(two.let, -Inf, Inf, bounds="(]")
  all_bw(two.let, -Inf, Inf, bounds="[]")

  all_bw(two.let, "a", Inf, bounds="()")
  all_bw(two.let, "a", Inf, bounds="[)")
  all_bw(two.let, "a", Inf, bounds="(]")
  all_bw(two.let, "a", Inf, bounds="[]")
  all_bw(two.let, "\t", Inf, bounds="(]")

  all_bw(two.let, -Inf, "zz", bounds="()")
  all_bw(two.let, -Inf, "zz", bounds="[)")
  all_bw(two.let, -Inf, "zz", bounds="(]")
  all_bw(two.let, -Inf, "zz", bounds="[]")

  two.let.inf[1] <- Inf
  two.let.inf[2] <- -Inf

  # All true b/c Inf values coerced to character

  all_bw(two.let.inf, -Inf, Inf, bounds="()")
  all_bw(two.let.inf, -Inf, Inf, bounds="[)")
  all_bw(two.let.inf, -Inf, Inf, bounds="(]")
  all_bw(two.let.inf, -Inf, Inf, bounds="[]")

  # All fail

  two.let.na[50] <- NA_character_

  all_bw(two.let.na, "a", "zz", bounds="()")
  all_bw(two.let.na, "a", "zz", bounds="[)")
  all_bw(two.let.na, "a", "zz", bounds="(]")
  all_bw(two.let.na, "a", "zz", bounds="[]")
  all_bw(two.let.na, -Inf, Inf, bounds="[]")

  # Some pass

  all_bw(two.let.na, "a", "zz", bounds="()", na.rm=TRUE)
  all_bw(two.let.na, "a", "zz", bounds="[)", na.rm=TRUE)
  all_bw(two.let.na, "a", "zz", bounds="(]", na.rm=TRUE)
  all_bw(two.let.na, "a", "zz", na.rm=TRUE)

  all_bw(two.let.na, "b", "zy", na.rm=TRUE)   # fail

  all_bw(two.let.na, -Inf, "zz", na.rm=TRUE)  # pass
  all_bw(two.let.na, -Inf, "zy", na.rm=TRUE)  # fail
  all_bw(two.let.na, -Inf, "zy") # fail

  all_bw(two.let.na, -Inf, "zzz", bounds="[)", na.rm=TRUE) # pass
  all_bw(two.let.na, -Inf, "zz", bounds="[)", na.rm=TRUE)  # fail

  all_bw(two.let.na, -Inf, "zz")  # fail

  all_bw(two.let.na, "a", Inf, na.rm=TRUE)  # pass
  all_bw(two.let.na, "a", Inf)              # fail
  all_bw(two.let.na, "b", Inf, na.rm=TRUE)  # fail

  all_bw(two.let.na, "\t",Inf,  bounds="(]", na.rm=TRUE) # pass
  all_bw(two.let.na, "a", Inf, bounds="(]", na.rm=TRUE)  # fail

  all_bw(two.let.na, "a", Inf)  # fail

  utf8 <- list(
    s4="\xF0\x90\x80\x80",  # four byte start
    e4="\xF4\x8F\xBF\xBD",  # last four byte before non-chars
    s3="\xE0\xA0\x80",      # three byte start
    e3="\xEF\xBF\xBD",      # last three byte before non-chars
    s2="\xC2\x80",          # two byte start
    e2="\xDF\xBF"           # two byte end
  )
  for(i in seq_along(utf8)) Encoding(utf8[[i]]) <- "UTF-8"

  # simple tests with utf8 bookends

  all_bw(lorem.ru.phrases, "\t", utf8$e2)
  all_bw(lorem.cn.phrases, "\t", utf8$e3)

  # # these two produce unsupressable warnings on Solaris
  # all_bw(lorem.ru.phrases, "\t", utf8$s2)
  # all_bw(lorem.cn.phrases, "\t", utf8$e2)

  # Unfortunately something is going wrong with how out-of-BMP unicode is read
  # in by windows so we have to comment out these tests; see #82
  # all_bw(lorem.emo.phrases, "\t", utf8$s4)
  # all_bw(lorem.emo.phrases, "\t", utf8$e4)
})

# Some latin-1 codes, on own file due to Solaris issues with escaping

lat.1.1 <- lat.1.2 <- c(
  "ni\xF1a",
  "hello",
  "\xB5 \xB6 \xBF \xC9 \xF4"
)
Encoding(lat.1.1) <- "latin1"
Encoding(lat.1.2) <- "bytes"

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

## Internal Funs
##
## R interface for an internal C functions used by \code{alike}.  Provided
## primarily for unit testing purposes
##
## @aliases name_compare class_compare dimname_compare dim_compare ts_compare
##   lang_alike fun_alike dep_alike match_call_alike env_track
## @keywords internal
## @param int_mode

attr_compare <- function(target, current, attr.mode=0L)
  .Call(VALC_compare_attributes, target, current, attr.mode)

name_compare <- function(target, current)
  .Call(VALC_compare_names, target, current)

class_compare <- function(target, current, rev=0)  # `rev` is unused; here for legacy
  .Call(VALC_compare_class, target, current)

dimname_compare <- function(target, current)
  .Call(VALC_compare_dimnames, target, current)

dim_compare <- function(
  target, current, tar_obj=integer(), cur_obj=integer(), rev=0L
)
  .Call(VALC_compare_dims, target, current, tar_obj, cur_obj, rev);

ts_compare <- function(target, current)
  .Call(VALC_compare_ts, target, current)

lang_alike <- function(target, current, match.call.env=parent.frame())
  .Call(VALC_lang_alike, target, current, match.call.env)

lang_alike_chr <- function(target, current, match.call.env=parent.frame())
  .Call(VALC_lang_alike_chr, target, current, match.call.env)

fun_alike <- function(target, current)
  .Call(VALC_fun_alike, target, current)

dep_alike <- function(obj, width.cutoff=60L)
  .Call(VALC_deparse, obj, width.cutoff)

dep_oneline <- function(obj, max.chars=20L, keep.at.end=0L)
  .Call(VALC_deparse_oneline, obj, max.chars, keep.at.end)

pad <- function(obj, lines=-1, pad=-1)
  .Call(VALC_pad, obj, lines, pad)

pad_or_quote <- function(obj, width=-1L, syntactic=-1L)
  .Call(VALC_pad_or_quote, obj, width, syntactic)

match_call_alike <- function(call, env)
  .Call(VALC_match_call, call, quote(match.call(NULL, quote(NULL))), env)

env_track <- function(env, size_init = 32, env_limit=65536L)
  .Call(VALC_env_track, env, size_init, env_limit)

is_valid_name <- function(name)
  .Call(VALC_is_valid_name_ext, name)

is_dfish <- function(obj)
  .Call(VALC_is_dfish, obj)

alike_mode <- function(obj)
  .Call(VALC_mode, obj)

syntactic_names <- function(lang) .Call(VALC_syntactic_names, lang)

msg_sort <- function(messages)
  .Call(VALC_msg_sort, messages)

msg_merge <- function(messages)
  .Call(VALC_msg_merge, messages)

msg_merge_2 <- function(messages)
  .Call(VALC_msg_merge_2, messages)

find_fun <- function(fun.name, env)
  .Call(VALC_find_fun, fun.name, env)

hash_test <- function(values, keys) .Call(VALC_hash_test, values, keys);

hash_test2 <- function(keys, add) .Call(VALC_hash_test2, keys, add);

track_hash <- function(keys, size) .Call(VALC_track_hash, keys, size);

hash_fun <- function(x) .Call(VALC_default_hash_fun, x)

check_assumptions <- function() .Call(VALC_check_assumptions)  # nocov

list_as_sorted_vec <- function(x) .Call(VALC_list_as_sorted_vec, x)

### Testing C stuff; should be deleted eventually
##
## @export
##test1 <- function(a) .Call(VALC_test1, a)
## @export
##test2 <- function(a, b) .Call(VALC_test2, a, b)
## @export
##test3 <- function(a, b, c) .Call(VALC_test3, a, b, c)

# Used for tests.  We can't really create them test time as `where` is not
# allowed, and even if it where, it only works becaue it seems topenv doesn't
# anticipate the possibility globalenv will not be on the search path.

#' Test Objects
#'
#' Objects used for testing purposes only.
#'
#' @rdname vetr-internal
#' @name vetr-interal
#' @keywords internal

setClass("vetr_foo", representation(a = "character", b = "numeric"))

#' @rdname vetr-internal

setClass("vetr_bar", representation(d = "numeric", c = "numeric"))

#' @rdname vetr-internal

setClass("vetr_baz", contains="vetr_foo", list(c="character"))

Foo <- setRefClass("Foo")
Bar <- setRefClass("Bar")

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

# redefine funs to give us flexibility if we change packages without having
# to export the internal functions

library(vetr)

unitizer_sect("Match Calls", {
  vetr:::match_call_alike(quote(var(y=1:10, runif(10))), baseenv())
  env0 <- new.env()
  env0$var <- function(yollo, zambia) NULL
  vetr:::match_call_alike(quote(var(y=1:10, runif(10))), env0)
})
unitizer_sect("Calls", {
  c0 <- quote(fun(a, b, a, 25))
  c1 <- quote(fun(x, y, x, "hello"))
  c2 <- quote(fun(x, y, z, "hello"))
  c3 <- quote(FUN(x, y, x, 1.01))
  c4 <- quote(fun(x, y, x, z))
  c5 <- quote(fun(a + b + a, FUN(z, a + 1)))
  c6 <- quote(fun(x + y + x, FUN(w, x + 2)))
  c7 <- quote(fun(x + y + x, FUN(w, y + 2)))
  c8 <- quote(fun(x + y + x, FUN(w, x - 2)))
  c9 <- quote(fun(x + y + x, FUN(w, x + "hello")))
  c10 <- quote(fun(1))
  c11 <- quote(fun(1, 2))

  c12 <- quote(a + b + c)
  c13 <- quote((a + b) + c)
  c14 <- quote(a + (b + c))

  vetr:::lang_alike(c0, c1, NULL)  # TRUE
  vetr:::lang_alike(c0, c2, NULL)  # no, inconsistent
  vetr:::lang_alike(c0, c3, NULL)  # no, wrong fun name
  vetr:::lang_alike(c0, c4, NULL)  # extra symbol
  vetr:::lang_alike(c5, c6, NULL)  # TRUE
  vetr:::lang_alike(c5, c7, NULL)  # inconsistent
  vetr:::lang_alike(c5, c8, NULL)  # wrong call `-`
  vetr:::lang_alike(c5, c9, NULL)  # TRUE
  vetr:::lang_alike(c11, c10, NULL)# Length mismatch

  # Parens

  vetr:::lang_alike(c12, c13)   # equivalent
  vetr:::lang_alike(c12, c14)   # not equivalent
  vetr:::lang_alike(c13, c14)   # not equivalent

  vetr:::lang_alike(c14, c13)   # not equivalent

  # with defined fun

  fun <- function(abc, bcd, efg) NULL

  ca <- quote(fun(a, b, a))
  cb <- quote(fun(x, e=x, y))

  vetr:::lang_alike(ca, cb, NULL)      # shouldn't match without match.call
  vetr:::lang_alike(cb, ca, NULL)      # false, different error
  vetr:::lang_alike(ca, cb)            # TRUE, should match

  # Actually use a function (and not just name of fun)

  ca.1 <- ca
  cb.1 <- cb

  ca.1[[1]] <- fun
  cb.1[[1]] <- fun

  vetr:::lang_alike(ca.1, cb.1)         # TRUE, should match

  # test nested match.call

  cc <- quote(fun(a, b, fun(b=1)))
  cd <- quote(fun(a, b, fun(c=1)))

  vetr:::lang_alike(cc, cd)

  # NULL in target matches anything

  ce <- quote(fun(a, b, NULL))

  vetr:::lang_alike(cc, ce)  # FALSE
  vetr:::lang_alike(ce, cc)  # TRUE

  # mismatched functions

  da <- quote(ff(a=1, b=2, c=3))
  db <- quote(ff(a=1, d=2, c=3))

  vetr:::lang_alike(da, db)

  # Errors

  vetr:::lang_alike(cc, 1:10)
  vetr:::lang_alike(ce, cc, match.call.env=1:10)
})
unitizer_sect("Calls as char", {
  vetr:::lang_alike_chr(c0, c1, NULL)  # TRUE
  vetr:::lang_alike_chr(c0, c2, NULL)  # no, inconsistent
  vetr:::lang_alike_chr(c0, c3, NULL)  # no, wrong fun name
  vetr:::lang_alike_chr(c0, c4, NULL)  # extra symbol
  vetr:::lang_alike_chr(c5, c6, NULL)  # TRUE
  vetr:::lang_alike_chr(c5, c7, NULL)  # inconsistent
  vetr:::lang_alike_chr(c5, c8, NULL)  # wrong call `-`
  vetr:::lang_alike_chr(c5, c9, NULL)  # TRUE

  vetr:::lang_alike_chr(ca, cb, NULL)      # shouldn't match without match.call
  vetr:::lang_alike_chr(cb, ca, NULL)      # false, different error
  vetr:::lang_alike_chr(ca, cb)            # TRUE, should match

  # test nested match.call

  vetr:::lang_alike_chr(cc, cd)

  # NULL in target matches anything

  vetr:::lang_alike_chr(cc, ce)  # FALSE
  vetr:::lang_alike_chr(ce, cc)  # TRUE
})
unitizer_sect("Formulas", {
  f0 <- y ~ x + 1
  f1 <- a ~ b + 1
  f2 <- a ~ b + 2
  f3 <- y ~ x + log(x) + z - 1
  f4 <- a ~ b + log(b) + c - 1
  f5 <- a ~ b + log(c) + b - 1
  f6 <- a ~ b + ln(b) + c - 1
  f7 <- a ~ b + log(b) + c + 1

  vetr:::lang_alike(f0, f1, NULL)        # TRUE
  vetr:::lang_alike(f0, f2, NULL)        # FALSE
  vetr:::lang_alike(f3, f4, NULL)        # TRUE
  vetr:::lang_alike(f3, f5, NULL)        # FALSE
  vetr:::lang_alike(f3, f6, NULL)        # FALSE
  vetr:::lang_alike(f3, f7, NULL)        # FALSE
})
unitizer_sect("Deparse", {
  l0 <- quote(
    a + b + fun(x + funz(
      matrix_over[25, 32]) + transform(iris, x = Sepal.Width * 3) /
      the_donkey_ate_a_carrot %in% {
        paste0(
          match(letter, LETTERS),
          c("hello there")
  ) } ) )
  # simple deparse

  (dep.txt <- vetr:::dep_alike(l0))
  vetr:::dep_alike(l0, 30)

  # manip the deparse

  vetr:::pad(dep.txt)
  old.opt <- options(prompt=">>", continue=" |")
  vetr:::pad(dep.txt)
  options(old.opt)
  vetr:::pad(dep.txt, pad=4)
  vetr:::pad(dep.txt, pad=4, lines=2)

  # oneline

  vetr:::dep_oneline(quote(1 + 1 + 3 + 944254235), 10)
  vetr:::dep_oneline(quote(1 + 1 + 3), 10)
  vetr:::dep_oneline(quote(1 + 1 + 3), "hello")
  vetr:::dep_oneline(quote(1 + 1 + 3 - (mean(1:10) + 3)), 15, 1L)
})

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

library(vetr)

# Redefine names; necessary because we renamed functions when moving to C
# versions

unitizer_sect("type_of", {
  type_of(1:100)
  type_of(1.1)
  type_of(1:100 + 1.0)
  type_of(1:100 + 1/1e9)
  type_of(NA_real_)
  type_of(Inf)
  type_of(-Inf)
} )
unitizer_sect("type_alike", {
  type_alike(1, 1.1)          # TRUE, 1 is numeric
  type_alike(1L, 1.1)         # FALSE
  type_alike(1L, 1.00000001)  # FALSE
  type_alike(1L, 1.0)         # TRUE

  type_alike(1, 1.1, vetr_settings(type.mode=1))   # TRUE, 1 is numeric
  type_alike(1L, 1.0, vetr_settings(type.mode=1))  # FALSE
  type_alike(1.0, 1L, vetr_settings(type.mode=1))  # TRUE
  type_alike(1.0, 1L, vetr_settings(type.mode=2))  # FALSE, must be num-num

  type_alike(1:100, 1:100 + 0.0)  # TRUE
  type_alike(1:101, 1:101 + 0.0)  # FALSE
  type_alike(1:101, 1:101 + 0.0, vetr_settings(fuzzy.int.max.len=200))  # TRUE

  type_alike(numeric(), c(1.1, 0.053, 41.8))  # TRUE
  type_alike(numeric(), list(1.1))  # FALSE
  type_alike(list(), integer())     # FALSE
  type_alike(1000000L, 1000000L + .1)    # FALSE
  type_alike(1000000L, 1000000L + .0)    # TRUE
  type_alike(data.frame(a=1:10), list()) # TRUE
  type_alike(NULL, NULL)
  type_alike(1/0, NA)

  # errors

  type_alike(1, 1.1, vetr_settings(type.mode=1:2))
  type_alike(1, 1.1, vetr_settings(fuzzy.int.max.len=1:2))
} )
unitizer_sect("functions", {
  type_alike(sd, var)     # clo-clo
  type_alike(`&&`, sd)    # spe-clo
  type_alike(`&&`, sum)   # spe-blt
  type_alike(sum, sd)     # blt-clo
  type_alike(sum, c)      # blt-blt
  type_alike(`&&`, `[`)   # spe-spe

  type_alike(sd, 1:3)

  type_alike(sd, var, vetr_settings(type.mode=1))     # clo-clo
  type_alike(`&&`, sd, vetr_settings(type.mode=1))    # spe-clo
  type_alike(`&&`, sum, vetr_settings(type.mode=1))   # spe-blt
  type_alike(sum, sd, vetr_settings(type.mode=1))     # blt-clo
  type_alike(sum, c, vetr_settings(type.mode=1))      # blt-blt
  type_alike(`&&`, `[`, vetr_settings(type.mode=1))   # spe-spe
} )

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

#' @export
#' @rdname type_alike

type_of <- function(object)
  .Call(VALC_typeof, object)

#' Fuzzily Compare Types of Objects
#'
#' Type evaluation and comparison is carried out with special treatment for
#' numerics, integers, and function types.  Whole number NA-free numeric vectors
#' of sufficiently short length (<100 by default) representable in the integer
#' type are considered to be type integer.  Closures, built-ins, and specials
#' are all treated as type closure.
#'
#' Specific behavior can be tuned with the `type.mode` parameter to the
#' [vetr_settings()] object passed as the `settings` parameter to this function.
#'
#' @seealso [alike()], [vetr_settings()], in particular the section about
#'   the `type.mode` parameter which affects how this function behaves.
#' @param target the object to test type alikeness against
#' @param current the object to test the type alikeness of
#' @param settings NULL, or a list as produced by [vetr_settings()]
#' @param object the object to check the type of
#' @return For `type_of` character(1L) the type of the object, for `type_alike`
#'   either TRUE, or a string describing why the types are not alike.
#' @export
#' @examples
#' type_of(1.0001)          # numeric
#' type_of(1.0)             # integer (`typeof` returns numeric)
#' type_of(1)               # integer (`typeof` returns numeric)
#' type_of(sum)             # closure (`typeof` returns builtin)
#' type_of(`$`)             # closure (`typeof` returns special)
#'
#' type_alike(1L, 1)
#' type_alike(1L, 1.1)
#' type_alike(integer(), numeric(100))
#' type_alike(integer(), numeric(101))  # too long

type_alike <- function(target, current, settings=NULL)
  .Call(VALC_type_alike, target, current, substitute(current), settings)

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

library(vetr)

unitizer_sect("Single template validation", {
  fun0 <- function(x, y, z)
    vetr(x=matrix(integer(), ncol=3), y=integer(2L), z=logical(1L))
  fun0(1, 2, 3)
  fun0(matrix(1), 2, 3)
  fun0(matrix(1:3, nrow=1), 2, 3)
  fun0(matrix(1:3, nrow=1), 2:3, 3)
  fun0(matrix(1:3, nrow=1), c(2.0, 3.0), 3)      # integer like
  fun0(matrix(1:3, nrow=1), c(2.0, 3.0), TRUE)
})
unitizer_sect("Multi-template validation", {
  fun1 <- function(x, y, z)
    vetr(
      x=matrix(integer(), ncol=3) || integer(3L),
      y=integer(2L) || NULL || logical(1L),
      z=logical(1L)
    )
  fun1(1:3, "fail", "fail")                   # x passes
  fun1(matrix(1:9, ncol=3), "fail", "fail")   # x passes
  fun1(letters[1:3], "fail", "fail")          # x fails

  fun1(1:3, 1:2, "fail")                      # x,y pass
  fun1(1:3, NULL, "fail")                     # x,y pass
  fun1(1:3, FALSE, "fail")                    # x,y pass

  fun1(1:3, FALSE, FALSE)                     # all pass
})
unitizer_sect("Template and Straight Eval", {
  fun2 <- function(x, y, z)
    vetr(
      x=(matrix(integer(), ncol=3) || integer(3L)) && .(!any(is.na(.))),
      y=integer(3L) && .(all(. > 0)),
      z=logical(1L) && .(!is.na(.))
    )
  fun2(matrix(c(1:8, NA), nrow=3), NULL, NULL)
  fun2(matrix(c(1:9), nrow=3), -1:1, NULL)
  fun2(matrix(c(1:9), nrow=3), 1:3, NA)
  fun2(matrix(c(1:9), nrow=3), 1:3, TRUE)
})
unitizer_sect("Complex OR outcomes", {
  fun2a <- function(x)
    vetr(
      x=setNames(character(3L), letters[1:3]) || matrix("", 3, 1) ||
        list(character(), x=integer())
    )
  fun2a(letters[1:3])
})
unitizer_sect("Errors in Arguments", {
  fun3 <- function(x, y)
    vetr(x=logical(1L), y=integer(3L))
  fun3(stop("boom"))
  fun3(TRUE, stop("boomBOOM"))
  fun3(1:3, stop("boomBOOM"))

  fun4 <- function(x, y)
    vetr(x=stop("BOOM"), y=integer(3L))
  fun4(NULL, 1:3)

  fun5 <- function(x, y)
    vetr(x=integer(3L), y=NULL || .(stop("hah")))
  fun5(1:3, NULL)
  fun5(1:2, NULL)

  fun6 <- function(x, y)
    vetr(x=integer(3L), y=NULL && .(stop("hah")))
  fun6(1:3, NULL)
})
unitizer_sect("Args evaled in correct env?", {
  fun7 <- function(x, y=z + 2) { z <- "boom"; vetr(x=TRUE, y=1L) }
  fun7a <- function(x, y=z + 2) { z <- 40; vetr(x=TRUE, y=1L) }
  z <- 1
  fun7(TRUE)     # fail because z in fun is character
  fun7a(TRUE)    # works
  fun8 <- function(x, y=z + 2) { a <- b <- TRUE; vetr(x=TRUE, y=1L) }
  fun8a <- function(x, y=z + 2) { a <- b <- NULL; vetr(x=TRUE, y=1L) }
  a <- NULL
  b <- TRUE
  fun8(a && b)   # fail because a in parent is NULL
  a <- TRUE
  fun8a(a && b)  # works despite NULLs in function

  # Make sure we can access defined templates in lexical parents

  fun_make <- function() {
    a <- matrix(1:9, 3)
    tpl <- matrix(numeric(), 3)

    function(x) {
      vetr(tpl)
      TRUE
    }
  }
  fun <- fun_make()
  a <- b <- 1:9
  local({
    NULL
    a <- character()
    fun(a)
  })
  local({
    b <- character()
    fun(b)
  })
  # make sure we can access variables that are not in fun lexical scope

  fun8b <- function(x) vetr(x=length(.) > 0 && integer())
  # should fail, using try b/c of new objectNotFoundError 
  inherits(try(get("zfqwefkj")), "try-error")
  local({
    zfqwefkj <- 200L
    fun8b(zfqwefkj)
  })
})
unitizer_sect("Compound Expression Scope Issues", {
  a <- quote(!anyNA(.))
  fun <- function(x) {
    a <- quote(all(. > 0))
    b <- quote(is.vector(.))
    vetr(a && b)
    TRUE
  }
  fun(-(1:3))
})
unitizer_sect("Non-equal args and validation exps", {
  fun8 <- function(x="hello", y=TRUE, z)
    vetr(x=integer(), z=integer(2L))

  fun8(1L, NULL, 1:2)
  fun8(1L, 1:2, NULL)

  fun8(1L, 1:2)
  fun8(1L)

  # default argument fails validation

  fun8(z=1:2)

})
unitizer_sect("Referencing argument in vet exp error", {
  fun1 <- function(x, y) vetr(x > 0, . < 3)
  fun1(1:10, 1:10)
  fun2 <- function(x, y) vetr(. > 0 && all(y > 0), y < 3)
  fun2(TRUE, 1:10)

  # also check with vet, although not as important

  x <- 1:10
  vet(x > 0, x)
  vet((x + 1) > 0, x + 1)  # this doesn't cause error, but maybe should?
})
unitizer_sect("Default arg mix-up", {
  fun10a <- function(x, y=TRUE, z=999) vetr(INT, LGL.1, INT.1)
  fun10a(1, z=1:3)

  fun10b <- function(x, y=TRUE, z=999) vetr(INT, z=INT.1)
  fun10b(1, z=1:3)
})
unitizer_sect("Dots", {
  f <- function(x, y=1L, z=1L, ...) vetr(1L, 1L, 1L)
  f(2L, w=3L)

  f <- function(x, y=1L, ...) vetr(1L, 1L, 1L)
  f(2L, z=3L)
})
unitizer_sect("Don't access promises in environments", {
  fenv <- function(env) vetr(environment())
  env <- FALSE
  fenv(environment())
})
unitizer_sect("Invocation via `do.call` (#109)", {
  f <- function(x) vetr(is.function(.))
  do.call(f, list(mean))
})

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

#' Generate Control Settings For vetr and alike
#'
#' Utility function to generate setting values.  We strongly recommend
#' that you generate the settings outside of function calls so that setting
#' generation does not become part of the \code{vet/vetr/alike} evaluation as
#' that could add noticeable overhead to the function evaluation.
#'
#' Settings after `fuzzy.int.max.len` are fairly low level and exposed mostly
#' for testing purposes.  You should generally not need to use them.
#'
#' Note that a successful evaluation of this function does not guarantee a
#' correct settings list.  Those checks are carried out internally by
#' \code{vet/vetr/alike}.
#'
#' @seealso \code{\link{type_alike}}, \code{\link{alike}}, \code{\link{vetr}}
#' @export
#' @param type.mode integer(1L) in 0:2, defaults to 0, determines how object
#'   types (as in `typeof`) are compared: \itemize{
#'     \item 0: integer like numerics (e.g. \code{1.0}) can match against
#'       integer templates, and integers always match real templates; all
#'       function types are considered of the same type
#'     \item 1: integers always match against numeric templates, but not vice
#'       versa, and integer-like numerics are treated only as numerics;
#'       functions only match same function type (i.e. closures only match
#'       closures, builtins builtins, and specials specials)
#'     \item 2: types must be equal for all objects types (for functions, this
#'       is unchanged from 1)
#'   }
#' @param attr.mode integer(1L) in 0:2, defaults to 0,  determines strictness of
#'   attribute comparison: \itemize{
#'     \item \code{0} only checks attributes that are present in target, and
#'       uses special comparisons for the special attributes (\code{class},
#'       \code{dim}, \code{dimnames}, \code{names}, \code{row.names},
#'       \code{levels}, \code{srcref}, and \code{tsp}) while requiring other
#'       attributes to be \code{alike}
#'     \item \code{1} is like \code{0}, except all atributes must be
#'       \code{alike}
#'     \item \code{2} requires all attributes to be present in \code{target} and
#'       \code{current} and to be alike
#'   }
#' @param lang.mode integer(1L) in 0:1, defaults to 0, controls language
#'   matching, set to `1` to turn off use of [match.call()]
#' @param fun.mode NOT IMPLEMENTED, controls how functions are compared
#' @param rec.mode integer(1L) `0` currently unused, intended to control how
#'   recursive structures (other than language objects) are compared
#' @param fuzzy.int.max.len max length of numeric vectors to consider for
#'   integer likeness (e.g. `c(1, 2)` can be considered "integer", even
#'   though it is numeric); currently we limit this check to vectors
#'   shorter than 100 to avoid a potentially expensive computation on large
#'   vectors, set to -1 to apply to all vectors irrespective of length
#' @param suppress.warnings logical(1L) suppress warnings if TRUE
#' @param width to use when deparsing expressions; default `-1`
#'   equivalent to \code{getOption("width")}
#' @param env.depth.max integer(1L) maximum number of nested environments to
#'   recurse through, defaults to 65535L; these are tracked to make sure we do
#'   not get into an infinite recursion loop, but because they are tracked we
#'   keep a limit on how many we will go through, set to -1 to allow unlimited
#'   recursion depth.  You should not need to change this unless you are running
#'   into the recursion limit.
#' @param symb.sub.depth.max integer(1L) maximum recursion depth when
#'   recursively substituting symbols in vetting expression, defaults to 65535L
#' @param symb.size.max integer(1L) maximum number of characters that a symbol
#'   is allowed to have in vetting expressions, defaults to 15000L.
#' @param track.hash.content.size integer(1L) (advanced) used to set the initial
#'   size of the symbol tracking vector used with the hash table that detects
#'   recursive symbol substitution.  If the tracking vector fills up it will be
#'   grown by 2x.  This parameter is exposed mostly for developer use.
#' @param nchar.max integer(1L) defaults to 65535L, threshold after which
#'   strings encountered in C code are truncated.  This is the read limit.  In
#'   theory `vetr` can produce strings longer than that by combining multiple
#'   shorter pieces.
#' @param env what environment to use to match calls and evaluate vetting
#'   expressions, although typically you would specify this with the `env`
#'   argument to `vet`; if NULL will use the calling frame to
#'   \code{vet/vetr/alike}.
#' @param result.list.size.init initial value for token tracking.   This will be
#'   grown by a factor of two each time it fills up until we reach
#'   `result.list.size.max`.
#' @param result.list.size.max maximum number of tokens we keep track of,
#'   intended mostly as a safeguard in case a logic error causes us to keep
#'   allocating memory.  Set to 1024 as a default value since it should be
#'   exceedingly rare to have vetting expressions with such a large number of
#'   tokens, enough so that if we reach that number it is more likely something
#'   went wrong.
#' @return list with all the setting values
#' @examples
#' type_alike(1L, 1.0, settings=vetr_settings(type.mode=2))
#' ## better if you are going to re-use settings to reduce overhead
#' set <- vetr_settings(type.mode=2)
#' type_alike(1L, 1.0, settings=set)

vetr_settings <- function(
  type.mode=0L, attr.mode=0L, lang.mode=0L, fun.mode=0L, rec.mode=0L,
  suppress.warnings=FALSE, fuzzy.int.max.len=100L,
  width=-1L, env.depth.max=65535L, symb.sub.depth.max=65535L,
  symb.size.max=15000L, nchar.max=65535L, track.hash.content.size=63L,
  env=NULL, result.list.size.init=64L, result.list.size.max=1024L
) {
  # we just use the function to match parameters
  as.list(environment())
}

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

library(vetr)

unitizer_sect("Standard Methods", {
  abstract(1:10)
  abstract(list(a=1:10, b=runif(10)))
  abstract(matrix(1:9, nrow=3))
  abstract(
    array(1:8, c(2, 2, 2), dimnames=list(letters[1:2], LETTERS[1:2], NULL))
  )
  # non atomic
  list.arr <- replicate(8, list(1), simplify=FALSE)
  dim(list.arr) <- rep(2, 3)
  abstract(list.arr)
  abstract(list(1, NULL))

  # df
  alike(abstract(iris), iris[1:10, ])
  alike(abstract(iris), iris[1:10, 1:3])
  alike(abstract(iris), transform(iris, Species=as.character(Species)))

  my.env <- new.env()
  identical(my.env, abstract(my.env))
})
unitizer_sect("Time Series", {
  y <- ts(runif(12), start=1970, frequency=12)
  attributes(abstract(y))
  attributes(abstract(y, "start"))
  attributes(abstract(y, "end"))
  attributes(abstract(y, "frequency"))
  attributes(abstract(y, c("start", "frequency")))

  y2 <- ts(numeric(12), start=1960, frequency=12)
  alike(abstract(y, c("start", "end")), y2)
  alike(abstract(y, c("start")), y2)

  y2 <- ts(numeric(12), start=1960, frequency=13)
  alike(abstract(y, c("start", "end")), y2)

  # Errors

  abstract(y, "boom")
  vetr:::abstract.ts(1:12)

  # Test what happens with both "tsp" and "tsp_vetr"
  y.abs <- abstract(y)
  alike(y.abs, y)
  alike(y, y.abs)
  # ... should have attribute "tsp", see ?abstract, str(y.abs)
  alike(y.abs, y.abs)
})
unitizer_sect("s4", {
  ## s4 objects are unaffected
  obj <- new("unitizerGlobalState")
  abstract(obj)
  nullify(obj, 1)
})
unitizer_sect("lm", {
  set.seed(1)
  df1 <- data.frame(x = runif(10), y=runif(10), z=runif(10))
  df2 <- data.frame(a = runif(5), b=runif(5), c=runif(5))
  mdl <- lm(y ~ x + poly(z, 2), df1)

  alike(abstract(mdl), mdl)

  mdl2 <- lm(x ~ y + poly(z, 2), df1)

  alike(abstract(mdl), mdl2)

  mdl3 <- lm(a ~ b + log(c), df2)

  alike(abstract(mdl), mdl3)

  mdl4 <- lm(a ~ b, df2)

  alike(abstract(mdl), mdl4)
})
unitizer_sect("nullify", {
  nullify(list(1, 2, 3), 2)
  nullify(list(1, 2, 3), -2)
  nullify(list(1, 2, 3, 4), c(TRUE, FALSE))
  nullify(list(1, 2, 3, 4), c(TRUE, FALSE, FALSE))
  nullify(list(1, 2, 3, 4), c(rep(FALSE, 4), TRUE))
  nullify(list(a=1, b=2, 3, 4), c("a", "b"))
  nullify(list(1, 2, 3, 4), "hello")

  nullify(list(1, 2, 3), 4)

  ## can't print the data frame because the concomitant warning changes in
  ## r75024 (see issue#96), and limitations of unitizer (unitizer:issue249)

  iris.null <- nullify(iris[1:10, ], 4)
  as.list(iris.null)
  class(iris.null)

  nullify(letters, 5)

  nullify(structure(letters[1:2], class='xqwer892jahaksdf'), 2)
})

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

#' Vetting Tokens With Custom Error Messages
#'
#' Utility function to generate vetting tokens with attached error messages.
#' You should only need to use this if the error message produced naturally by
#' `vetr` is unclear.  Several predefined tokens created by this function
#' are also documented here.
#'
#' Allows you to supply error messages for vetting to use for each error
#' token.  Your token should not contain top level `&&` or `||`.  If
#' it does your error message will not be reported because `vetr` looks for
#' error messages attached to atomic tokens.  If your token must involve
#' top level `&&` or `||`, use `I(x && y)` to ensure that
#' your error message is used by `vet`, but beware than in doing so you do
#' not use templates within the `I` call as everything therein will be
#' interpreted as a vetting expression rather than a template.
#'
#' Error messages are typically of the form "%s should be XXX".
#'
#' This package ships with many predefined tokens for common use cases. They
#' are listed in the `Usage` section of this documentation.  The tokens
#' are named in format `TYPE[.LENGTH][.OTHER]`.  For example
#' `INT` will vet an integer vector, `INT.1` will vet a scalar integer
#' vector, and `INT.1.POS.STR` will vet a strictly positive integer vector.
#' At this time tokens are predefined for the basic types as scalars or
#' any-length vectors.  Some additional checks are available (e.g. positive only
#' values).
#'
#' Every one of the predefined vetting tokens documented here implicitly
#' disallows NAs.  Numeric tokens also disallow infinite values. If you wish
#' to allow NAs or infinite values just use a template object (e.g.
#' `integer(1L)`).
#'
#' @note **This will only work with standard tokens containing `.`**.  Anything
#' else will be interpreted as a template token.
#'
#' @export
#' @seealso [vet()]
#' @param exp an expression which will be captured but not evaluated.
#' @param err.msg character(1L) a message that tells the user what the
#'   expected value should be, should contain a "%s" for `sprintf`
#'   to use (e.g. "%s should be greater than 2").
#' @return a quoted expressions with `err.msg` attribute set
#' @examples
#' ## Predefined tokens:
#' vet(INT.1, 1:2)
#' vet(INT.1 || LGL, 1:2)
#' vet(INT.1 || LGL, c(TRUE, FALSE))
#'
#' ## Check squareness
#' mx <- matrix(1:3)
#' SQR <- vet_token(nrow(.) == ncol(.), "%s should be square")
#' vet(SQR, mx)
#'
#' ## Let `vetr` make up error message; note `quote` vs `vet_token`
#' ## Often, `vetr` does fine without explictly specified err msg:
#' SQR.V2 <- quote(nrow(.) == ncol(.))
#' vet(SQR.V2, mx)
#'
#' ## Combine some tokens, notice how we use `quote` at the combining
#' ## step:
#' NUM.MX <- vet_token(matrix(numeric(), 0, 0), "%s should be numeric matrix")
#' SQR.NUM.MX <- quote(NUM.MX && SQR)
#' vet(SQR.NUM.MX, mx)
#'
#' ## If instead we used `vet_token` the overall error message
#' ## is not used; instead it falls back to the error message of
#' ## the specific sub-token that fails:
#' NUM.MX <- vet_token(matrix(numeric(), 0, 0), "%s should be numeric matrix")
#' SQR.NUM.MX.V2 <-
#'   vet_token(NUM.MX && SQR, "%s should be a square numeric matrix")
#' vet(SQR.NUM.MX.V2, mx)

vet_token <- function(exp, err.msg="%s") {
  if(
    !is.character(err.msg) || length(err.msg) != 1L || is.na(err.msg) ||
    inherits(
      suppressWarnings(try(sprintf(err.msg, "test"), silent=TRUE)),
      "try-error"
    ) ||
    identical(suppressWarnings(sprintf(err.msg, "test")), err.msg)
  ) {
    stop(
      "Argument `err.msg` must be character(1L) and contain a single '%s' ",
      "for use by `sprintf`."
    )
  }
  x <- substitute(exp)
  attr(x, "err.msg") <- err.msg
  x
}
#' @rdname vet_token
#' @export

NO.NA <- vet_token(!is.na(.), "%s should not contain NAs, but does")

#' @export
#' @name vet_token

NO.INF <- vet_token(
  is.finite(.), "%s should contain only finite values, but does not"
)
#' @export
#' @name vet_token

GTE.0 <- vet_token(
  . >= 0, "%s should contain only positive values, but has negatives"
)

#' @export
#' @name vet_token

LTE.0 <- vet_token(
  . <= 0, "%s should contain only negative values, but has positives"
)

#' @export
#' @name vet_token

GT.0 <- vet_token(
  . > 0,
  paste0(
    "%s should contain only \"strictly positive\" values, but has zeroes or ",
    "negatives"
  )
)

#' @export
#' @name vet_token

LT.0 <- vet_token(
  . < 0,
  paste0(
    "%s should contain only \"strictly negative\" values, but has zeroes ",
    "or positives"
  )
)

#' @export
#' @name vet_token

INT.1 <- quote(integer(1L) && NO.NA && NO.INF)

#' @export
#' @name vet_token

INT.1.POS <- quote(integer(1L) && NO.NA && NO.INF && GTE.0)

#' @export
#' @name vet_token

INT.1.NEG <- quote(integer(1L) && NO.NA && NO.INF && LTE.0)

#' @export
#' @name vet_token

INT.1.POS.STR <- quote(integer(1L) && NO.NA && NO.INF && GT.0)

#' @export
#' @name vet_token

INT.1.NEG.STR <- quote(integer(1L) && NO.NA && NO.INF && LT.0)

#' @export
#' @name vet_token

INT <- quote(integer() && NO.NA && NO.INF)
#' @export
#' @name vet_token

INT.POS <- quote(integer() && NO.NA && NO.INF && GTE.0)

#' @export
#' @name vet_token

INT.NEG <- quote(integer() && NO.NA && NO.INF && LTE.0)

#' @export
#' @name vet_token

INT.POS.STR <- quote(integer() && NO.NA && NO.INF && GT.0)

#' @export
#' @name vet_token

INT.NEG.STR <- quote(integer() && NO.NA && NO.INF && LT.0)

#' @export
#' @name vet_token

NUM.1 <- quote(numeric(1L) && NO.NA && NO.INF)

#' @export
#' @name vet_token

NUM.1.POS <- quote(numeric(1L) && NO.NA && NO.INF && GTE.0)

#' @export
#' @name vet_token

NUM.1.NEG <- quote(numeric(1L) && NO.NA && NO.INF && LTE.0)

#' @export
#' @name vet_token

NUM <- quote(numeric() && NO.NA && NO.INF)

#' @export
#' @name vet_token

NUM.POS <- quote(numeric() && NO.NA && NO.INF && GTE.0)

#' @export
#' @name vet_token

NUM.NEG <- quote(numeric() && NO.NA && NO.INF && LTE.0)

#' @export
#' @name vet_token

CHR.1 <- quote(character(1L) && NO.NA)

#' @export
#' @name vet_token

CHR <- quote(character() && NO.NA)

#' @export
#' @name vet_token

CPX <- quote(complex() && NO.NA && NO.INF)

#' @export
#' @name vet_token

CPX.1 <- quote(complex(1L) && NO.NA && NO.INF)

#' @export
#' @name vet_token

LGL <- quote(logical() && NO.NA)

#' @export
#' @name vet_token

LGL.1 <- quote(logical(1L) && NO.NA)

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.


## Number of Characters Taken By Numeric Representation
##
## e.g., for 100, would be 3, for 1324, would be 4.
##
## @param x integer(1L)
## @return integer(1L)

len_chr_len <- function(x) .Call(VALC_len_chr_len_ext, x)

## Number Converted To character
##
## e.g., \code{100} becomes \code{"100"}
##
## @param x integer(1L)
## @return character(1L)

len_as_chr <- function(x) .Call(VALC_len_as_chr_ext, x)

num_as_chr <- function(a, as.int=FALSE) .Call(VALC_num_as_chr, a, as.int)

## "Safe" String Manipulation Functions
##
## Implements variants of \code{strlcpy}, \code{strnlen}, etc.
##
## These functions are intended to be used directly from C but are exposed here
## for testing purposes.  They differ from similar existing versions in these
## respects:
##
## \itemize{
##   \item all these functions have a limit to how many characters they will
##     read so they should not read past an unterminated string if you know the
##     buffer size
##   \item Input strings are never modified; if changes are required to comply
##     with length limits a copy that is \code{R_alloc}ed is returned
##   \item Only C99 code is used
## }
## Mostly these functions are not actually necessary since the strings
## encountered will have be read in by R and as such should have been handled
## properly.  Additionally, for functions such as smprintf* the `maxlen`
## parameter doesn't really help if the strings to substitute themselves are not
## null terminated since we can only specify one maxlen and there may be many
## strings to deal with.  They were written as a learning exercise for myself.
##
## Function specific details follow.  Pay attention, the interfaces are not
## exactly the same as the \code{C} functions they intend to replace.  For
## example \code{smprintf} does not have a \code{str} parameter like
## \code{snprintf}, and \code{strmcpy} returns a new character string instead of
## modifying one passed as an argument.).
##
## \itemize{
##   \item \code{strmlen}: roughly equivalent to POSIX \code{strnlen}, but
##     should be fully portable
##   \item \code{strmcpy}: if \code{strmlen(str)} is greater than \code{maxlen},
##     returns a copy of \code{str} with \code{'\0'} as the \code{maxlen + 1}th
##     character (note return character array is not allocated beyond this NULL),
##     otherwise returns a copy of \code{str}.
##   \item \code{smprintf2}: returns a newly \code{R_alloc}ed string that is just
##     large enough to fit the combination of the format string and the tokens
##     to sub in (note that tokens must be character so only "%s" is meaningful
##     as a formatting element).  To achieve this without overflow, ensures that
##     \code{format} and each token is limited to \code{maxlen - 1} characters.
##     Fails if result string is longer than a \code{size_t} could index.  Note
##     that internally we define \code{smprintf1} - \code{smprintf6}, but we are
##     only exporting the one function directly to R for testing.
##   \item \code{strbullet} ads a bullet at front of string and padding after
##     each new line
##   \item \code{collapse} is essentially the same as \code{paste0(x,
##     collapse=sep}
## }
## @aliases strmcpy smprintf2 ucfirst lcfirst strbullet collapse
## @param str character string to measure or manipulate, should be scalar for
##   \code{strmlen} and \code{strmcpy}
## @param format character(1L) string to use as format template
## @param maxlen integer(1L) size limit to truncate to, or to limit tokens to
## @param a character(1L) another string
## @param b character(1L) another string
## @param bullet character(1L) a string to use as the bullet symbol
## @param ctd character(1L) a string preferably the same number of characters as
##   \code{bullet} to use after the first line wraps in a bullet
## @param sep character(1L) a separator to use when collapsing character vectors
##   with \code{collapse}
## @return integer(1L) for \code{strmlen}, character(1L) for \code{strmcpy}
##   and \code{smprintf2}

strmlen <- function(str, maxlen=10000L) .Call(VALC_strmlen_ext, str, maxlen)

## @rdname strmlen
## @export

strmcpy <- function(str, maxlen=10000L) .Call(VALC_strmcpy_ext, str, maxlen)

## @rdname strmlen
## @export

smprintf2 <- function(format, a, b, maxlen=10000L)
  .Call(VALC_smprintf2_ext, maxlen, format, a, b)

smprintf6 <- function(format, a, b, c, d, e, f, maxlen=10000L)
  .Call(VALC_smprintf6_ext, maxlen, format, a, b, c, d, e, f)

## @rdname strmlen
## @export

ucfirst <- function(str, maxlen=10000L) .Call(VALC_ucfirst_ext, str, maxlen)

## @rdname strmlen
## @export

lcfirst <- function(str, maxlen=10000L) .Call(VALC_lcfirst_ext, str, maxlen)

## @rdname strmlen
## @export

strbullet <- function(str, bullet="- ", ctd="  ", maxlen=10000L)
  .Call(VALC_bullet_ext, str, bullet, ctd, maxlen)

## @rdname strmlen
## @export

collapse <- function(str, sep="", maxlen=10000L)
  .Call(VALC_collapse_ext, str, sep, maxlen)

strsub <- function(string, chars=15L, mark=TRUE)
  .Call(VALC_strsub, string, chars, mark)

nchar_u <- function(string) .Call(VALC_nchar_u, string)

char_offsets <- function(string) .Call(VALC_char_offsets, string)

## Purely internal funs for testing

test_strmcpy <- function() .Call(VALC_test_strmcpy)
test_strappend <- function() .Call(VALC_test_strappend)
test_add_szt <- function() .Call(VALC_test_add_szt)
test_smprintfx <- function() .Call(VALC_test_smprintfx)
test_strappend2 <- function() .Call(VALC_test_strappend2)

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

# This file contains interfaces to internal C functions for use with R based
# unit testing frameworks

## Expand variable names to experessions if they contain language
##
## @keywords internal

symb_sub <- function(symb, env=parent.frame())
  .Call(VALC_symb_sub, symb, env)


## Parse Expressions For \code{`validate`} Use
##
## Takes expressions provided to \code{`\link{validate}`} and identifies which
## ones are templates, which ones are normal expressions, and substitutes the
## actual argument name for \code{`.`}.
##
## Internal function exposed for unit testing purposes
##
## @keywords internal
## @param symb an R symbol
## @param arg_name another R symbol
## @param an environment to look for language expressions to substitute
## @return list

parse_validator <- function(lang, arg_name, rho=parent.frame())
  .Call(VALC_parse, lang, arg_name, rho)

## Remove Parens and \code{`.(`} From Calls
##
## @keywords internal

remove_parens <- function(lang)
  .Call(VALC_remove_parens, lang)

## Evaluates a test
##
## For unit testing
##
## @keywords internal

eval_check <- function(lang, arg_name, arg_value, env=parent.frame())
  .Call(
    VALC_eval_check, lang, arg_name, arg_name, arg_value, sys.call(), env
  )

## Internal version of `all`
##
## @keywords internal

val_all <- function(x) .Call(VALC_all, x)

# overwrite unitizer_sect to just plain eval stuff

unitizer_sect <- function(
  title = NULL, expr=expression(), details=character(), compare=identical
) {
  expr.sub <- substitute(expr)
  for(i in as.list(tail(expr.sub, -1L))) try(eval(i, envir=parent.frame()))
}

source('unitizer/_pre/lorem.R')
source('unitizer/alike.R', echo=TRUE)
source('unitizer/cstringr.R', echo=TRUE)
source('unitizer/language.R', echo=TRUE)
source('unitizer/validate.R', echo=TRUE)

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

# Runs the tests without attempting to fully compose the result message

library(vetr)

unitizer_sect("evaluate", {
  vetr:::eval_check(quote(logical(2L)), quote(xyz), 1:2)
  vetr:::eval_check(quote(logical(2L)), quote(xyz), c(TRUE, FALSE))
  vetr:::eval_check(quote(logical(2L)), quote(xyz), c(TRUE, FALSE, TRUE))

  vetr:::eval_check(quote(logical(2L) || NULL), quote(xyz), 1:2)
  vetr:::eval_check(quote(logical(2L) || NULL), quote(xyz), NULL)
  vetr:::eval_check(quote(logical(2L) || NULL), quote(xyz), c(TRUE, TRUE))

  vetr:::eval_check(
    quote(matrix(integer(), nrow=3) || NULL), quote(xyz), matrix(1:21, ncol=7)
  )
  vetr:::eval_check(quote(matrix(integer(), nrow=3) || NULL), quote(xyz), 1:21)
  vetr:::eval_check(
    quote(matrix(integer(), nrow=3) || vector("list", 2L)),
    quote(xyz), list("hello")
  )
  vetr:::eval_check(
    quote(matrix(integer(), nrow=3) || vector("list", 2L)),
    quote(xyz), list("hello", "goodbye")
  )
  vetr:::eval_check(
    quote(matrix(integer(), nrow=3) || list(character(1L), 1L)),
    quote(xyz), list("hello", "goodbye"))
})
unitizer_sect("evaluate with sub", {
  xyz <- c(TRUE, TRUE)
  vetr:::eval_check(quote(logical(2L) && .(all(xyz))), quote(xyz), xyz)
  vetr:::eval_check(quote(logical(2L) && .(all(.))), quote(xyz), xyz)
  vetr:::eval_check(quote(logical(2L) && .(!any(is.na(.)))), quote(xyz), xyz)
  xyz <- c(TRUE, NA)
  vetr:::eval_check(quote(logical(2L) && .(!any(is.na(.)))), quote(xyz), xyz)
  xyz <- c(TRUE, FALSE, TRUE)
  vetr:::eval_check(quote(logical(2L) && .(!any(is.na(.)))), quote(xyz), xyz)

  abc1 <- letters[1:5]
  vetr:::eval_check(
    quote(character(5L) && .(all(. %in% letters[1:3]))), quote(abc1), abc1
  )
  abc2 <- rep("a", 5)
  vetr:::eval_check(
    quote(character(5L) && .(all(. %in% letters[1:3]))), quote(abc2), abc2
  )

  mat1 <- matrix(1:30, ncol=3)
  vetr:::eval_check(
    quote(
      (
        matrix(numeric(), ncol=3) || matrix(integer(), nrow=10) ||
        character(10L)
      ) && .(length(.) < 100)
    ),
    quote(mat1), mat1
  )
  mat2 <- matrix(1:120, ncol=3)
  vetr:::eval_check(
    quote(
      (
        matrix(numeric(), ncol=3) || matrix(integer(), nrow=10) ||
        character(10L)
      ) &&
      .(length(.) < 100)
    ),
    quote(mat2), mat2
  )
  mat3 <- LETTERS[1:9]
  vetr:::eval_check(   # Fail all
    quote(
      (
        matrix(numeric(), ncol=3) || matrix(integer(), nrow=10) ||
        character(10L)
      ) && .(length(.) < 100)
    ),
    quote(mat3), mat3
  )
  vetr:::eval_check(   # Fail all
    quote(
      matrix(numeric(), ncol=3) || matrix(integer(), nrow=10) ||
      character(10L) || .(length(.) > 20)
    ),
    quote(mat3), mat3
  )
})
unitizer_sect("custom expressions", {
  x <- -1:1
  y <- 1
  z <- -1
  w <- NA_integer_
  u <- integer()
  t <- 1:3
  vetr:::eval_check(quote(. > 0), quote(x), x)
  vetr:::eval_check(quote(. > 0), quote(y), y)
  vetr:::eval_check(quote(. > 0), quote(z), z)
  vetr:::eval_check(quote(. > 0), quote(t), t)
  vetr:::eval_check(quote(. > 0), quote(w), w)
  vetr:::eval_check(quote(. > 0), quote(u), u)
})
unitizer_sect("Errors", {
  vetr:::eval_check(1:3, 1:3, TRUE, env=list(1:3))
  vetr:::eval_check(quote(y), quote(x), TRUE, env=list(1:3))
})

# These examples cannot be in-line in the unitizer because Solaris fails to
# properly escape them, which then leads to failures.

# Examples from the Unicode 10.0 docs

unicode.10 <- c(
  "\xC2\x41\x41",
  "\x61\xF1\x80\x80",
  "\x61\xF1\x80\x80\xE1\x80",
  "\x61\xF1\x80\x80\xE1\x80\xC2\x62\x80\x63\x80\xBF\x64"
)
Encoding(unicode.10) <- "UTF-8"

#' Verify Values in Vector are Between Two Others
#'
#' Similar to \code{isTRUE(all(x >= lo & x <= hi))} with default settings,
#' except that it is substantially faster and returns a string describing the
#' first encountered violation rather than FALSE on failure.
#'
#' You can modify the comparison to be strictly greater/less than via the
#' `bounds` parameter, and the treatment of NAs with `na.rm`.  Note that NAs are
#' considered to be out of bounds by default.  While technically incorrect
#' since we cannot know whether an NA value is in or out of bounds, this
#' assumption is both conservative and convenient.  Zero length `x` will always
#' succeed.
#'
#' If `x` and `lo`/`hi` are different types, `lo`/`hi` will be coerced to the
#' type of `x`.  When `lo`/`hi` are numeric and `x` is integer, if `lo`/`hi`
#' values are outside of the integer range then that side will be treated as if
#' you had used `-Inf`/`Inf`.  `-Inf` and `Inf` mean `lo` and `hi` will be
#' unbounded for all data types.
#'
#' @export
#' @param x vector logical (treated as integer), integer, numeric, or character.
#'   Factors are treated as their underlying integer vectors.
#' @param lo scalar vector of type coercible to the type of `x`, cannot be NA,
#'   use `-Inf` to indicate unbounded (default).
#' @param hi scalar vector of type coercible to the type of `x`, cannot be NA,
#'   use `Inf` to indicate unbounded (default), must be greater than or equal to
#'   `lo`.
#' @param na.rm TRUE, or FALSE (default), whether NAs are considered to be
#'   in bounds.  Unlike with [all()], for `all_bw` `na.rm=FALSE` returns an
#'   error string if there are NAs instead of NA.  Arguably NA, but not NaN,
#'   should be considered to be in `[-Inf,Inf]`, but since `NA < Inf` is NA we
#'   treat them as always being out of bounds.
#' @param bounds `character(1L)` for values between `lo` and `hi`:
#'   * \dQuote{[]} include `lo` and `hi`
#'   * \dQuote{()} exclude `lo` and `hi`
#'   * \dQuote{(]} exclude `lo`, include `hi`
#'   * \dQuote{[)} include `lo`, exclude `hi`
#'
#' @return TRUE if all values in `x` conform to the specified bounds, a string
#'   describing the first position that fails otherwise
#' @examples
#' all_bw(runif(100), 0, 1)
#' all_bw(runif(100) * 2, 0, 1)
#' all_bw(NA, 0, 1)              # This is does not return NA
#' all_bw(NA, 0, 1, na.rm=TRUE)
#'
#' vec <- c(runif(100, 0, 1e12), Inf, 0)
#' all_bw(vec, 0)      # All +ve numbers
#' all_bw(vec, hi=0)   # All -ve numbers
#' all_bw(vec, 0, bounds="(]") # All strictly +ve nums
#' all_bw(vec, 0, bounds="[)") # All finite +ve nums

all_bw <- function(x, lo=-Inf, hi=Inf, na.rm=FALSE, bounds="[]")
  .Call(VALC_all_bw, x, lo, hi, na.rm, bounds)

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

library(vetr)

unitizer_sect("All", {
  vetr:::val_all(1:10)  # -2
  vetr:::val_all(rep(TRUE, 10)) # 1
  vetr:::val_all(c(rep(TRUE, 10), FALSE, TRUE)) # 0
  vetr:::val_all(c(rep(TRUE, 5), NA, rep(TRUE, 5))) # -4
  vetr:::val_all(FALSE) # -1
  vetr:::val_all(TRUE)  # 2
  vetr:::val_all(logical()) # 3, this used to be -5
  vetr:::val_all(NA)  # -3
  vetr:::val_all(c(TRUE, TRUE, NA, TRUE))  # -4
})

unitizer_sect("Hash", {
  # these should all equal 193
  vetr:::hash_fun(c("f b", "n b", "n d", "t m", "b r", "n w", "q w", "o x"))
})
unitizer_sect("bench_mark", {
  # three different time frames that should trigger all the code, need to remove
  # the time piece so that the tests don't fail due to variations

  capt_wo_time <- function(x) {
    txt <- capture.output(x)
    gsub("~ *-?[0-9.e+\\-]*", "~", txt)
  }
  capt_wo_time(bench_mark(Sys.sleep(1.2), times=1))
  capt_wo_time(bench_mark(Sys.sleep(.01), times=10))
  capt_wo_time(bench_mark(1 + 1, NULL, times=100))
})
unitizer_sect("sort lists", {
  vetr:::list_as_sorted_vec(list(c=1, a=list(), b=NULL))
  # # equal names not stable, but we should never hit this with attribute lists
  # vetr:::list_as_sorted_vec(pairlist(a=1, a=list(), a=NULL))
  vetr:::list_as_sorted_vec(list(b=1, 2, a=3))
  vetr:::list_as_sorted_vec(list())
  vetr:::list_as_sorted_vec(list(a=1))
  vetr:::list_as_sorted_vec(list(1))
})

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

#' Lightweight Benchmarking Function
#'
#' Evaluates provided expression in a loop and reports mean evaluation time.
#' This is inferior to `microbenchmark` and other benchmarking tools in many
#' ways except that it has zero dependencies or suggests which helps with
#' package build and test times.  Used in vignettes.
#'
#' Runs [gc()] before each expression is evaluated.  Expressions are evaluated
#' in the order provided.  Attempts to estimate the overhead of the loop by
#' running a loop that evaluates `NULL` the `times` times.
#'
#' Unfortunately because this computes the average of all iterations it is very
#' susceptible to outliers in small sample runs, particularly with fast running
#' code.  For that reason the default number of iterations is one thousand.
#'
#' @importFrom stats median
#' @export
#' @param ... expressions to benchmark, are captured unevaluated
#' @param times how many times to loop, defaults to 1000
#' @param deparse.width how many characters to deparse for labels
#' @return NULL, invisibly, reports timings as a side effect as screen output
#' @examples
#' bench_mark(runif(1000), Sys.sleep(0.001), times=10)

bench_mark <- function(..., times=1000L, deparse.width=40) {
  stopifnot(
    is.integer(times) || is.numeric(times), length(times) == 1, times > 0
  )
  times <- as.integer(times)
  dots <- as.list(match.call(expand.dots=FALSE)[["..."]])
  p.f <- parent.frame()

  timings <- vapply(
    dots, function(x) {
      call.q <- bquote({
        gc()
        start <- proc.time()
        for(i in 1:.(times)) .(x)
        stop <- proc.time()
        stop[['elapsed']] - start[['elapsed']]
      })
      eval(call.q, p.f)
    },
    numeric(1L)
  )
  # try to compute overhead

  o.h.times <- 10
  gc()
  overhead <- vapply(
    seq.int(o.h.times), function(x) {
      call.q.baseline <- bquote({
        start <- proc.time()
        for(j in 1:.(times)) NULL
        stop <- proc.time()
        stop[['elapsed']] - start[['elapsed']]
      })
      eval(call.q.baseline, p.f)
    },
    numeric(1L)
  )
  overhead.act <- median(overhead)
  timings.fin <- (timings - overhead.act) / times
  exps <- vapply(
    dots,
    function(x) dep_oneline(x, max.chars=deparse.width),
    character(1L)
  )
  timings.clean <- timings.fin[timings.fin >= 0]

  unit <- "seconds"
  mult <- 0

  if(length(timings.clean)) {
    min.time <- min(log(timings.clean, base=10))
    if(min.time <= -3) {
      unit <- "microseconds"
      mult <- 6
    } else if(min.time <= 0) {
      unit <- "milliseconds"
      mult <- 3
    }
  }
  timings.disp <- timings.fin * 10 ^ mult

  cat(
    sprintf(
      "Mean eval time from %d iteration%s, in %s:\n", times,
      if(times > 1) "s" else "", unit
  ) )
  cat(
    paste0(
      "  ",
      format(exps), "  ~  ",
      format(signif(timings.disp, 4), justify='right'), "\n"
    ),
    sep=""
  )
  invisible(data.frame(call=exps, mean.time=timings.fin))
}

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

library(vetr)

unitizer_sect("remove parens", {
  vetr:::remove_parens(quote((a)))
  vetr:::remove_parens(quote(.(a)))
  vetr:::remove_parens(quote((((a)))))
  vetr:::remove_parens(quote((.((.(a))))))
  vetr:::remove_parens(quote((a) && .(a)))  # Nothing should be removed
})
unitizer_sect("parse", {
  x <- quote(.(.) && ((a)))
  vetr:::parse_validator(x, quote(arg_to_validate))
  x # make sure unchanged from previous assignment

  vetr:::parse_validator(quote(FALSE), quote(arg_to_validate))
  vetr:::parse_validator(quote(((FALSE))), quote(arg_to_validate))
  vetr:::parse_validator(quote(((FALSE && ((TRUE))))), quote(arg_to_validate))
  vetr:::parse_validator(quote(.(FALSE)), quote(arg_to_validate))
  vetr:::parse_validator(quote(.), quote(arg_to_validate))
  vetr:::parse_validator(quote(. && a), quote(arg_to_validate))
  vetr:::parse_validator(quote(.(.)), quote(arg_to_validate))
  vetr:::parse_validator(quote(((a && b) || .(.))), quote(arg_to_validate))
  vetr:::parse_validator(quote(matrix(nrow=3)), quote(arg_to_validate))
  vetr:::parse_validator(quote(matrix(nrow=3) && .(.)), quote(arg_to_validate))
  vetr:::parse_validator(quote((a || ((b && c))) && .(a + .)), quote(arg_to_validate))
  vetr:::parse_validator(quote((a || ((b && .(c)))) && (a + .(.))), quote(arg_to_validate))

  vetr:::parse_validator(quote(a && (b + .(c))), quote(arg_to_validate))  # uninterpretable?
  vetr:::parse_validator(quote(a && .), "hello")                          # uninterpretable?
} )
unitizer_sect("token sub", {
  vetr:::symb_sub(INT.1)
  vetr:::symb_sub(NO.NA)

  # Dot unescaping
  `..` <- quote(yes)
  `.zzz` <- `zzz.` <- quote(yup)
  `.` <- quote(...)
  vetr:::symb_sub(quote(..))
  vetr:::symb_sub(quote(...))
  vetr:::symb_sub(quote(.zzz))
  vetr:::symb_sub(quote(zzz.))

  # Errors
  `.` <- quote(..)
  vetr:::symb_sub(quote(..))
  vetr:::symb_sub(quote(.))

  # Identity operations on non-symbols
  vetr:::symb_sub(quote(.(zzz)))
  vetr:::symb_sub("hello")
})

unitizer_sect("preset tokens", {
  x <- quote(integer(1L))
  y <- quote(integer(1L) || NULL)
  z <- quote(integer(1L) && .(!any(is.na(.))))
  vetr:::parse_validator(quote(x), quote(w))
  vetr:::parse_validator(quote(y), quote(w))
  vetr:::parse_validator(quote(z), quote(w))
  vetr:::parse_validator(quote(z || NULL), quote(w))
} )
unitizer_sect("validators", {
  vetr:::parse_validator(INT.1, quote(w))
  vetr:::parse_validator(INT, quote(w))
  vetr:::parse_validator(CHR.1, quote(w))
  vetr:::parse_validator(CHR, quote(w))
  vetr:::parse_validator(NUM.1, quote(w))
  vetr:::parse_validator(NUM, quote(w))
  vetr:::parse_validator(LGL.1, quote(w))
  vetr:::parse_validator(LGL, quote(w))
  vetr:::parse_validator(CPX.1, quote(w))
  vetr:::parse_validator(CPX, quote(w))
} )

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

# redefine funs to give us flexibility if we change packages without having
# to export the internal functions

library(vetr)

unitizer_sect("Name like attributes", {
  vetr:::name_compare(c("", "hello"), c("abc", "hello"))
  vetr:::name_compare(c("ab", "hello"), c("abc", "hello"))
  vetr:::name_compare(c(NA_character_, "hello"), c("abc", "hello"))
  vetr:::name_compare(c("ab", "hello"), c(NA_character_, "hello"))
  vetr:::name_compare(c(NA_character_, "hello"), c(NA_character_, "hello"))

  # mess with values

  vetr:::name_compare(1:3, 3:1)
} )
unitizer_sect("S3 Classes", {
  class1 <- letters[1:5]
  class2 <- letters[3:5]
  class3 <- letters[c(4, 3, 5)]
  class4 <- character()
  class5 <- NULL
  class6 <- list("a", "b", "c")

  # Note third argument is left in for legacy reasons but doesn't actually do
  # anything

  vetr:::class_compare(class2, class1, 0);
  vetr:::class_compare(class1, class2, 0);
  vetr:::class_compare(class1, class1[1:3], 0);
  vetr:::class_compare(class3, class2, 0);
  vetr:::class_compare(class3, class1, 0);
  vetr:::class_compare(class5, class2, 0);  # this should never happen in reality, but use alike check since not char char comparison
  vetr:::class_compare(class2, class5, 0);  # idem
  vetr:::class_compare(class5, class5, 0);
  vetr:::class_compare(class6, class2, 0);
  vetr:::class_compare(class2, class6, 0);

  class7 <- c("a", "data.frame")

  vetr:::class_compare(class7, class1, 0)
  vetr:::class_compare(class1, class7, 0)
})
unitizer_sect("Dimnames", {
  dimn1 <- list(NULL, NULL, NULL)
  dimn2 <- list(a=letters[1:3], b=letters[4:6], c=letters[7:9])
  dimn3 <- list(letters[1:3], b=letters[4:6], c=letters[7:9])
  dimn4 <- list(letters[1:3], B=letters[4:6], C=letters[7:9])
  dimn5 <- list(a=LETTERS[1:3], b=letters[4:6], c=letters[7:9])
  dimn6 <- list(a="", b=letters[4:6], c=letters[7:9])
  dimn7 <- list()
  dimn8 <- list(a=LETTERS[1:3], b=letters[4:6], c=letters[7:9], d=letters[24:26])
  dimn9 <- list(a=1:3, b=letters[4:6], c=letters[7:9])
  dimn10 <- list(a=list("a", "b", "c"), b=letters[4:6], c=letters[7:9])
  dimn11 <- NULL
  dimn12 <- matrix(letters[1:9], nrow=3)
  dimn13 <- `attr<-`(dimn2, "bar", "yowza")
  dimn14 <- `attr<-`(dimn2, "bar", "yowz")
  dimn15 <- list(a=letters[1:3], b=letters[1:3])
  dimn16 <- list(a=letters[1:3], b=letters[1:3])
  attr(dimn15, "a") <- 1:2
  attr(dimn16, "a") <- 1:3
  dimn17 <- list(a=letters[1:3])
  dimn18 <- list(a=letters[1:2], b=letters[1:3])

  # baseline cases

  vetr:::dimname_compare(dimn3, dimn2)
  vetr:::dimname_compare(dimn2, dimn3)
  vetr:::dimname_compare(dimn3, dimn4)
  vetr:::dimname_compare(dimn2, dimn5)
  vetr:::dimname_compare(dimn6, dimn5)
  vetr:::dimname_compare(dimn5, dimn6)

  # "empty-ish" cases

  vetr:::dimname_compare(dimn2, dimn1)
  vetr:::dimname_compare(dimn1, dimn2)
  vetr:::dimname_compare(dimn11, dimn2)
  vetr:::dimname_compare(dimn11, dimn11)
  vetr:::dimname_compare(dimn7, dimn2)
  vetr:::dimname_compare(dimn2, dimn7)
  vetr:::dimname_compare(dimn7, dimn7)

  # Breaking cases

  vetr:::dimname_compare(dimn5, dimn8)
  vetr:::dimname_compare(dimn8, dimn5)
  vetr:::dimname_compare(dimn9, dimn2)
  vetr:::dimname_compare(dimn2, dimn9)
  vetr:::dimname_compare(dimn2, dimn12)
  vetr:::dimname_compare(dimn12, dimn12)

  # Attr on attr

  vetr:::dimname_compare(dimn2, dimn13)
  vetr:::dimname_compare(dimn13, dimn2)
  vetr:::dimname_compare(dimn13, dimn14)
  vetr:::dimname_compare(dimn14, dimn13)

  # dimanames with attributes other than names that are not alike

  vetr:::dimname_compare(dimn15, dimn16)

  # Mismatched lengths

  vetr:::dimname_compare(dimn17, dimn18)
})
unitizer_sect("Dims", {
  dim1 <- rep(2L, 2)
  dim2 <- rep(2L, 3)
  dim3 <- rep(2L, 4)
  dim4 <- c(1L, 1L)
  dim5 <- 2L
  dim6 <- c(1L, 2L, 3L)
  dim7 <- rep(0L, 2)
  dim8 <- c(0L, 0L, 2L)
  dim9 <- NULL
  dim10 <- letters[1:2]
  dim11 <- list(2L, 2L)

  vetr:::dim_compare(dim1, dim2)  # fail
  vetr:::dim_compare(dim2, dim3)  # fail
  vetr:::dim_compare(dim1, dim4)  # fail
  vetr:::dim_compare(dim2, dim6)  # fail
  vetr:::dim_compare(dim7, dim1)  # works
  vetr:::dim_compare(dim7, dim4)  # works
  vetr:::dim_compare(dim1, dim7)  # fail
  vetr:::dim_compare(dim7, dim2)  # works
  vetr:::dim_compare(dim8, dim2)  # works
  vetr:::dim_compare(dim8, dim6)  # fail
  vetr:::dim_compare(dim6, dim9)  # works

  # really a corner case that shouldn't come about

  vetr:::dim_compare(9L, NULL)

  # With non atomic objects

  vetr:::dim_compare(dim1, dim2, list())          # fail
  vetr:::dim_compare(dim1, dim2, cur_obj=list())  # fail
  vetr:::dim_compare(dim1, dim2, list(), list())  # fail

  vetr:::dim_compare(dim1, dim2, integer(), list())  # fail

  # Errors

  vetr:::dim_compare(dim9, dim6)  # fail
  vetr:::dim_compare(dim10, dim1) # fail
})
unitizer_sect("Time Series", {
  ts.1 <- attr(ts(runif(24), 1970, frequency=12), "tsp")
  ts.2 <- attr(ts(runif(24), 1970, frequency=4), "tsp")
  ts.3 <- ts.4 <- ts.1
  ts.3[[2L]] <- 0
  ts.4[[3L]] <- 0

  vetr:::ts_compare(ts.1, ts.2)
  vetr:::ts_compare(ts.3, ts.2)  # zero is wildcard
  vetr:::ts_compare(ts.4, ts.2)
  vetr:::ts_compare(ts.4, ts.1)  # zero is wildcard
  vetr:::ts_compare(ts.1, ts.4)  # but not in reverse

  # non-ts comparisons

  vetr:::ts_compare(ts.4, "hello")
  vetr:::ts_compare("hello", 1:3)
  vetr:::ts_compare(ts.1, 1:3)   # TRUE, because second param is not REAL, kicks off to standard alike comparison
  vetr:::ts_compare(ts.4, 1:4)
})
unitizer_sect("All Attributes, default", {
  vetr:::attr_compare(1, 1)                                           # TRUE
  vetr:::attr_compare(matrix(integer(), 3), matrix(integer(), 3, 3))  # TRUE
  vetr:::attr_compare(matrix(integer(), 3), matrix(integer(), 3, 3), "hello")  # Error
  vetr:::attr_compare(matrix(integer(), 3), matrix(integer(), 3, 3), 1.1)      # Error
  vetr:::attr_compare(matrix(integer(), 4), matrix(integer(), 3, 3))           # Dim 1 error
  vetr:::attr_compare(matrix(integer(), ncol=4), matrix(integer(), 3, 3))      # Dim 2 error
  vetr:::attr_compare(                                                         # TRUE
    matrix(integer(), 3, 3, dimnames=list(NULL, letters[1:3])),
    matrix(integer(), 3, 3, dimnames=list(LETTERS[1:3], letters[1:3]))
  )
  vetr:::attr_compare(                                                         # dimnames error
    matrix(integer(), 3, 3, dimnames=list(NULL, letters[2:4])),
    matrix(integer(), 3, 3, dimnames=list(LETTERS[1:3], letters[1:3]))
  )
  vetr:::attr_compare(                                                         # dimnames error
    matrix(integer(), 3, 3, dimnames=list(letters[1:3], letters[1:3])),
    matrix(integer(), 3, 3, dimnames=list(LETTERS[1:3], letters[1:3]))
  )
  vetr:::attr_compare(                                                         # TRUE
    matrix(integer(), 3, 3, dimnames=list(LETTERS[1:3], letters[1:3])),
    matrix(integer(), 3, 3, dimnames=list(a=LETTERS[1:3], b=letters[1:3]))
  )
  vetr:::attr_compare(                                                         # dimnames error
    matrix(integer(), 3, 3, dimnames=list(A=LETTERS[1:3], letters[1:3])),
    matrix(integer(), 3, 3, dimnames=list(a=LETTERS[1:3], b=letters[1:3]))
  )
  vetr:::attr_compare(                                                         # TRUE
    structure(list(integer(), character())),
    data.frame(a=1:10, b=letters[1:10])
  )
  vetr:::attr_compare(                                                         # TRUE
    structure(list(integer(), character()), class="data.frame"),
    data.frame(a=1:10, b=letters[1:10])
  )
  vetr:::attr_compare(                                                         # TRUE
    structure(unname(data.frame(integer(), character())), class="data.frame"),
    data.frame(a=1:10, b=letters[1:10])
  )
  vetr:::attr_compare(                                                         # TRUE, zero length attr
    structure(list(), welp=list()),
    structure(list("hello"), welp=list(NULL, 1:3), belp=1:3)
  )
  vetr:::attr_compare(                                                         # Attr length mismatch
    structure(list(), welp=list(NULL)),
    structure(list("hello"), welp=list(NULL, 1:3), belp=1:3)
  )
  vetr:::attr_compare(                                                         # Missing attr
    structure(list(), welp=list(), belp=1:3),
    structure(list("hello"), welp=list(NULL, 1:3))
  )
  vetr:::attr_compare(                                                         # TRUE
    structure(list(), class=letters[1:3]),
    structure(list("hello"), class=letters[1:3])
  )
  vetr:::attr_compare(                                                         # class mismatch
    structure(list(), class=letters[1:3]),
    structure(list("hello"), class=letters[1:4])
  )
  vetr:::attr_compare(                                                         # TRUE
    structure(list(), class=letters[2:4]),
    structure(list("hello"), class=letters[1:4])
  )
} )
unitizer_sect("All attributes, strict", {
  # dim mismatch, but passes because comparison is `alike`
  vetr:::attr_compare(matrix(integer(), 3), matrix(integer(), 3, 3), 1)
  # TRUE
  vetr:::attr_compare(matrix(integer(), 3, 3), matrix(integer(), 3, 3), 1)
  # dimnames mismatch, but alike so passes
  vetr:::attr_compare(
    matrix(integer(), 3, 3, dimnames=list(NULL, letters[1:3])),
    matrix(integer(), 3, 3, dimnames=list(LETTERS[1:3], letters[1:3])),
    attr.mode=1
  )
  # dimnames mismatch, but passes because target has NULL names
  vetr:::attr_compare(
    matrix(integer(), 3, 3, dimnames=list(LETTERS[1:3], letters[1:3])),
    matrix(integer(), 3, 3, dimnames=list(a=LETTERS[1:3], b=letters[1:3])),
    attr.mode=1
  )
  # dimnames mismatch, but here fails because target has them but current doesnt
  vetr:::attr_compare(
    matrix(integer(), 3, 3, dimnames=list(a=LETTERS[1:3], b=letters[1:3])),
    matrix(integer(), 3, 3, dimnames=list(LETTERS[1:3], letters[1:3])),
    attr.mode=1
  )
  # actually passes because both have 2 length character name attrs, which are
  # alike
  vetr:::attr_compare(
    matrix(integer(), 3, 3, dimnames=list(A=LETTERS[1:3], letters[1:3])),
    matrix(integer(), 3, 3, dimnames=list(a=LETTERS[1:3], b=letters[1:3])),
    attr.mode=1
  )
  # TRUE
  vetr:::attr_compare(
    structure(list(integer(), character())),
    data.frame(a=1:10, b=letters[1:10]),
    attr.mode=1
  )
  # TRUE
  vetr:::attr_compare(
    structure(list(integer(), character()), class="data.frame"),
    data.frame(a=1:10, b=letters[1:10]),
    attr.mode=1
  )
  # Class mismatch
  vetr:::attr_compare(
    structure(list(), class=letters[2:4]),
    structure(list("hello"), class=letters[1:4]),
    attr.mode=1
  )
  # Too many attrs
  vetr:::attr_compare(
    structure(list(integer(), character())),
    data.frame(a=1:10, b=letters[1:10]),
    attr.mode=2
  )
  # Too many attrs
  vetr:::attr_compare(
    structure(list(integer(), character()), class="data.frame"),
    data.frame(a=1:10, b=letters[1:10]),
    attr.mode=2
  )
  # Missing attr
  vetr:::attr_compare(
    structure(list(), welp=list(NULL, 1:3), belp=1:3),
    structure(list("hello"), welp=list(NULL, 1:3)),
    attr.mode=2
  )
  # Missing attr, but attr count same
  vetr:::attr_compare(
    structure(list(), welp=list(NULL, 1:3), belp=1:3),
    structure(list("hello"), welp=list(NULL, 1:3), kelp=20),
    attr.mode=2
  )
} )
unitizer_sect("Closures", {
  # TRUE, methods should always match generics
  vetr:::fun_alike(print, print.data.frame)
  # FALSE, but generics won't match methods with more arguments
  vetr:::fun_alike(print.data.frame, print)
  vetr:::fun_alike(summary, summary.lm)
  vetr:::fun_alike(summary.lm, summary)

  fn0 <- function(x, y) NULL
  fn1 <- function(x, y, z) NULL
  fn2 <- function(y, x) NULL
  fn3 <- function(x=1, y=2) NULL
  fn4 <- function(x, ...) NULL
  fn5 <- function(x) NULL
  fn6 <- function(x, y, z, ...) NULL
  fn7 <- function(x, ..., y) NULL
  fn8 <- function(x, a, ..., g, y) NULL
  fn9 <- function(x, a, ..., g, y, w) NULL

  vetr:::fun_alike(fn0, fn1)  # FALSE
  vetr:::fun_alike(fn1, fn0)  # FALSE
  vetr:::fun_alike(fn4, fn1)  # FALSE, dots must be matched
  vetr:::fun_alike(fn0, fn2)  # FALSE
  vetr:::fun_alike(fn0, fn3)  # TRUE
  # FALSE - defaults in target must be specified in current as well
  vetr:::fun_alike(fn3, fn0)
  vetr:::fun_alike(fn4, fn5)  # FALSE dots in target must exit in current
  vetr:::fun_alike(fn4, fn6)  # TRUE
  vetr:::fun_alike(fn4, fn7)  # TRUE
  # FALSE - all arguments in target must be in current, even with dots
  vetr:::fun_alike(fn7, fn4)
  vetr:::fun_alike(fn7, fn8)  # TRUE
  # FALSE - extra arguments in current must be adjacent to dots
  vetr:::fun_alike(fn7, fn9)

  # Try some builtins / specials

  vetr:::fun_alike(`+`, `-`)  # TRUE, builtins
  vetr:::fun_alike(substitute, function(expr, env) NULL)  # TRUE, special
  vetr:::fun_alike(function(expr, env) NULL, substitute)  # TRUE, special
  vetr:::fun_alike(substitute, on.exit)  # FALSE, specials
  vetr:::fun_alike(on.exit, substitute)  # FALSE, specials
  vetr:::fun_alike(`[`, substitute)      # FALSE, argless specials
  vetr:::fun_alike(`[`, `&&`)          # TRUE, argless specials

  # Errors

  vetr:::fun_alike(identity, 10)
  vetr:::fun_alike(10, identity)
})
unitizer_sect("Env Track", {
  el.1 <- replicate(5, new.env())
  el.2 <- el.1[c(1, 1, 2, 3, 4, 1, 2, 3, 5, 1)]
  vetr:::env_track(el.1, 1L)  # first env a freebie, so should be 1
  vetr:::env_track(el.2, 1L)

  # Overwhelm env stack, though not that satifying a test

  vetr:::env_track(el.1, 1L, 3L)

  # Error

  vetr:::env_track(list(1, 2, 3), 1L, 3L)
} )
unitizer_sect("valid names", {
  vetr:::is_valid_name("hello")
  vetr:::is_valid_name(".hello")
  vetr:::is_valid_name("123")
  vetr:::is_valid_name("hello there")
  vetr:::is_valid_name("h1ello")
  vetr:::is_valid_name("_hello")
  vetr:::is_valid_name(".1fail")
  vetr:::is_valid_name("NULL")
  vetr:::is_valid_name("FALSE")

  vetr:::is_valid_name(letters)
} )
unitizer_sect("Is dfish", {
  df1 <- list(a=1:10, b=letters[1:10])
  df2 <- list(a=1:10, b=letters[1:9])
  vetr:::is_dfish(df1)
  vetr:::is_dfish(df2)
  vetr:::is_dfish(1:10)
} )
unitizer_sect("syntactic", {
  vetr:::syntactic_names(quote(hello))
  vetr:::syntactic_names(quote(`hello there`))
  vetr:::syntactic_names(quote(1 + 1))
  vetr:::syntactic_names(quote(1 %hello there% 1))
  vetr:::syntactic_names(quote(1 + `hello there`))
  vetr:::syntactic_names(quote(-(1:3)))
  vetr:::syntactic_names(quote(c(-1:1, NA_integer_)))
  vetr:::syntactic_names(quote(a == 25))
  vetr:::syntactic_names(quote(all(-1:1 > 0)))
})
unitizer_sect("Pad or Quote", {
  vetr:::pad_or_quote(quote(1 + 1))
  vetr:::pad_or_quote(quote(!anyNA(1 + 1)))

  vetr:::pad_or_quote(quote(1 + 1), syntactic=0L)
  vetr:::pad_or_quote(quote(1 + 1), syntactic=1L)
})
unitizer_sect("Merge messages", {
  vetr:::msg_sort(list(letters[5:1], letters[1:5]))
  # third element plays no role in sort
  vetr:::msg_sort(list(c("a", "a", "a", "z", "b"), c("a", "a", "z", "b", "b")))

  # corner cases

  vetr:::msg_sort(list(letters[5:1]))
  vetr:::msg_sort(as.list(letters[5:1]))
  vetr:::msg_sort(list(letters[1:5], NULL))
  vetr:::msg_sort(letters)

  msgs <- list(
    c("`my_var`", "be", "integer", "is", "character"),
    c("`my_var`", "have", "3 columns", "has", "1"),
    c("`length(names(my_var))`", "be", "2", "is", "4"),
    c("`my_var`", "be", "\"NULL\"", "is", "character"),
    c("`attr(my_var)`", "be", "\"NULL\"", "is", "list"),
    c("`my_var`", "be", "matrix", "is", "character"),
    c("`length(names(my_var))`", "be", "3", "is", "4")
  )
  vetr:::msg_merge(msgs)
  vetr:::msg_merge(msgs[1:3])  # no merging required here
  vetr:::msg_merge(msgs[1])    # no merging required here

  vetr:::msg_merge_2(msgs)
})
unitizer_sect("Hash", {
  keys <- vapply(
    1:26, function(x) paste0(letters[seq(x)], collapse=""), character(1L)
  )
  values <- vapply(
    1:26, function(x) paste0(LETTERS[seq(x)], collapse=""), character(1L)
  )
  vetr:::hash_test(keys, values)

  ## hash tracking, uses a hash to detect potential collisions, 1 means a value
  ## is added, >1 means a value was added and tracking array had to be resized
  ## to that size, 0 means it existed already, NA is a reset instruction, value
  ## following a reset instruction is what the reset was to

  vetr:::track_hash(letters[1:3], 2L)  # one resize
  vetr:::track_hash(letters[1:5], 2L)  # two resize

  vetr:::track_hash(c("a", "b", "b"), 2L)          # one repeat
  vetr:::track_hash(c("a", "b", NA, 1,  "b"), 2L)  # reset prior to repeat
  vetr:::track_hash(c("a", "b", NA, 1,  "a"), 2L)

  keys <- c(
    "a", "b", NA, 1,  "b", "hello", "goodbye", "a", NA, 3, "hello",
    "goodbye", "b"
  )
  vetr:::track_hash(keys, 8L)

  # all of these resolve to the same hash value, so should all be stored under
  # the same spot in the hash table.  We use this to test that additions and
  # removals from pairlists work

  collisions <- c("f b", "n b", "n d", "t m", "b r", "n w", "q w", "o x")
  keys.1 <- c(collisions, NA, 0, collisions)

  vetr:::track_hash(keys.1, 64L)

  keys.2 <- c(collisions, NA, 4, collisions)
  vetr:::track_hash(keys.2, 64L)

  # test additions and deletions under collisions

  vetr:::hash_test2(
    c(collisions[1:3], collisions[1:3], "hello"),
    c(T, T, T, F, F, F, F)
  )
})
unitizer_sect("Mode", {
  vetr:::alike_mode(NULL)
  vetr:::alike_mode(quote(a))
  vetr:::alike_mode(mean)
  vetr:::alike_mode(`+`)
  vetr:::alike_mode(log)
  vetr:::alike_mode(quote(1 + 1))
})
unitizer_sect("Find funs", {
  fun <- function(x, y) NULL
  vetr:::find_fun(quote(fun), environment())
  vetr:::find_fun(quote(asdhfqwerasdfasdf), environment())

  # corner case for missingArg, no longer supported since findVar became
  # inaccessible.  This should not be possible to trigger in regular use.
  # fun2 <- function(x) vetr:::find_fun(quote(x), environment())
  # fun2()
})

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

#' Verify Objects Meet Structural Requirements
#'
#' Use vetting expressions to enforce structural requirements and/or evaluate
#' test conditions for truth.  `tev` is identical to `vet` except with reversed
#' arguments for pipe based workflows.
#'
#' @section Vetting Expressions:
#'
#' Vetting expressions can be template tokens, standard tokens, or any
#' expression built with them, `||`, `&&`, and parentheses.  Template tokens
#' are R objects that define the required structure, much like the `FUN.VALUE`
#' argument to [vapply()].  Standard tokens are R expressions evaluated and
#' checked for being `all(TRUE)`.
#'
#' Standard tokens are distinguished from templates by whether they reference
#' the `.` symbol or not.  If you have a need to reference an object bound to
#' `.` in a vetting expression, you can escape the `.`  with an extra dot (i.e.
#' use `..`, and `...` for `..`,  and so forth for symbols comprising only
#' dots).  If you use standard tokens in your packages you will need to include
#' `utils::globalVariables(".")` as a top-level call to avoid the "no visible
#' binding for global variable '.'"' R CMD check NOTE.  Standard tokens that
#' return a string like e.g. `all.equal(x, .)` will result in that string being
#' incorporated into the error message.
#'
#' See `vignette('vetr', package='vetr')` and examples for details on how
#' to craft vetting expressions.
#'
#' @useDynLib vetr, .registration=TRUE, .fixes="VALC_"
#' @aliases tev
#' @export
#' @seealso [vetr()] for a version optimized to vet function arguments,
#'   [alike()] for how templates are used, [vet_token()] for how to specify
#'   custom error messages and also for predefined validation tokens for common
#'   use cases, [all_bw()] for fast bounds checks.
#' @param target a template, a vetting expression, or a compound expression
#' @param current an object to vet
#' @param env the environment to match calls and evaluate vetting expressions
#'   in; will be ignored if an environment is also specified via
#'   [vetr_settings()].  Defaults to calling frame.
#' @param format character(1L), controls the format of the return value for
#'   `vet`, in case of failure.  One of:\itemize{
#'     \item "text": (default) character(1L) message for use elsewhere in code
#'     \item "full": character(1L) the full error message used in "stop" mode,
#'       but actually returned instead of thrown as an error
#'     \item "raw": character(N) least processed version of the error message
#'       with none of the formatting or surrounding verbiage
#' }
#' @param stop TRUE or FALSE whether to call [stop()] on failure
#'   or not (default)
#' @param settings a settings list as produced by [vetr_settings()], or NULL to
#'   use the default settings
#' @return TRUE if validation succeeds, otherwise varies according to value
#'   chosen with parameter `stop`
#' @examples
#' ## Template token vetting
#' vet(numeric(2L), runif(2))
#' vet(numeric(2L), runif(3))
#' vet(numeric(2L), letters)
#' try(vet(numeric(2L), letters, stop=TRUE))
#'
#' ## Standard token vetting
#' vet(. > 0, runif(2))
#'
#' ## Expression made of standard and template tokens.
#' vet(numeric(1) && . > 0, 1)
#' try(vet(numeric(1) && . > 0, 1:2))
#' try(vet(numeric(1) && . > 0, -1))
#'
#' ## `tev` just reverses target and current
#' ## if(getRversion() >= "4.1.0") { # would be a parse error so commented
#' ##   runif(2) |> tev(numeric(2L))
#' ##   runif(3) |> tev(numeric(2L))
#' ## }
#'
#' ## Zero length templates are wild cards
#' vet(numeric(), runif(2))
#' vet(numeric(), runif(100))
#' vet(numeric(), letters)
#'
#' ## This extends to data.frames
#' iris.tpl <- iris[0,]   # zero row matches any # of rows
#' iris.1 <- iris[1:10,]
#' iris.2 <- iris[1:10, c(1,2,3,5,4)]  # change col order
#' vet(iris.tpl, iris.1)
#' vet(iris.tpl, iris.2)
#'
#' ## Short (<100 length) integer-like numerics will
#' ## pass for integer
#' vet(integer(), c(1, 2, 3))
#' vet(integer(), c(1, 2, 3) + 0.1)
#'
#' ## Nested templates; note, in packages you should consider
#' ## defining templates outside of `vet` or `vetr` so that
#' ## they are computed on load rather that at runtime
#' tpl <- list(numeric(1L), matrix(integer(), 3))
#' val.1 <- list(runif(1), rbind(1:10, 1:10, 1:10))
#' val.2 <- list(runif(1), cbind(1:10, 1:10, 1:10))
#' vet(tpl, val.1)
#' vet(tpl, val.2)
#'
#' ## See `example(alike)` for more template examples
#'
#' ## Standard tokens allow you to check values
#' vet(. > 0, runif(10))
#' vet(. > 0, -runif(10))
#'
#' ## Zero length token results are considered TRUE,
#' ## as is the case with `all(logical(0))`
#' vet(. > 0, numeric())
#'
#' ## `all_bw` is like `isTRUE(all(. >= x & . <= y))`, but
#' ## ~10x faster for long vectors:
#' vet(all_bw(., 0, 1), runif(1e6) + .1)
#'
#' ## You can combine templates and standard tokens with
#' ## `&&` and/or `||`
#' vet(numeric(2L) && . > 0, runif(2))
#' vet(numeric(2L) && . > 0, runif(10))
#' vet(numeric(2L) && . > 0, -runif(2))
#'
#' ## Using pre-defined tokens (see `?vet_token`)
#' vet(INT.1, 1)
#' vet(INT.1, 1:2)
#' vet(INT.1 && . %in% 0:1 || LGL.1, TRUE)
#' vet(INT.1 && . %in% 0:1 || LGL.1, 1)
#' vet(INT.1 && . %in% 0:1 || LGL.1, NA)
#'
#' ## Vetting expressions can be assembled from previously
#' ## defined tokens
#' scalar.num.pos <- quote(numeric(1L) && . > 0)
#' foo.or.bar <- quote(character(1L) && . %in% c('foo', 'bar'))
#' vet.exp <- quote(scalar.num.pos || foo.or.bar)
#'
#' vet(vet.exp, 42)
#' vet(scalar.num.pos || foo.or.bar, 42)  # equivalently
#' vet(vet.exp, "foo")
#' vet(vet.exp, "baz")
#'
#' ## Standard tokens that return strings see the string shown
#' ## in the error message:
#' vet(all.equal(., 2), 1)

vet <- function(
  target, current, env=parent.frame(), format="text", stop=FALSE, settings=NULL
)
  # note sys.call not matched, which is why we need substitute(current)
  .Call(
    VALC_validate, substitute(target), current, substitute(current),
    sys.call(), env, format, stop, settings
  )
#' @rdname vet
#' @export

tev <- function(
  current, target, env=parent.frame(), format="text", stop=FALSE, settings=NULL
)
  # note sys.call not matched, which is why we need substitute(current)
  .Call(
    VALC_validate, substitute(target), current, substitute(current),
    sys.call(), env, format, stop, settings
  )

#' Verify Function Arguments Meet Structural Requirements
#'
#' Use vetting expressions to enforce structural requirements for function
#' arguments.  Works just like [vet()], except that the formals of the
#' enclosing function automatically matched to the vetting expressions provided
#' in `...`.
#'
#' Only named arguments may be vetted; in other words it is not possible to vet
#' arguments passed via `...`.
#'
#' @inheritSection vet Vetting Expressions
#'
#' @note `vetr` will force evaluation of any arguments that are being
#'   checked (you may omit arguments that should not be evaluate from
#'   `vetr`)
#' @seealso [vet()], in particular `example(vet)`.
#' @param ... vetting expressions, each will be matched to the enclosing
#'   function formals as with [match.call()] and will be used to validate the
#'   value of the matching formal.
#' @param .VETR_SETTINGS a settings list as produced by [vetr_settings()], or
#'   NULL to use the default settings.  Note that this means you cannot use
#'   `vetr` with a function that takes a `.VETR_SETTINGS` argument
#' @return TRUE if validation succeeds, otherwise `stop` with error message
#'   detailing nature of failure.
#' @export
#' @examples
#' ## Look at `?vet` examples for more details on how to craft
#' ## vetting expressions.
#'
#' fun1 <- function(x, y) {
#'   vetr(integer(), LGL.1)
#'   TRUE   # do some work
#' }
#' fun1(1:10, TRUE)
#' try(fun1(1:10, 1:10))
#'
#' ## only vet the second argument
#' fun2 <- function(x, y) {
#'   vetr(y=LGL.1)
#'   TRUE   # do some work
#' }
#' try(fun2(letters, 1:10))
#'
#' ## Nested templates; note, in packages you should consider
#' ## defining templates outside of `vet` or `vetr` so that
#' ## they are computed on load rather that at runtime
#' tpl <- list(numeric(1L), matrix(integer(), 3))
#' val.1 <- list(runif(1), rbind(1:10, 1:10, 1:10))
#' val.2 <- list(runif(1), cbind(1:10, 1:10, 1:10))
#' fun3 <- function(x, y) {
#'   vetr(x=tpl, y=tpl && ncol(.[[2]]) == ncol(x[[2]]))
#'   TRUE   # do some work
#' }
#' fun3(val.1, val.1)
#' try(fun3(val.1, val.2))
#' val.1.a <- val.1
#' val.1.a[[2]] <- val.1.a[[2]][, 1:8]
#' try(fun3(val.1, val.1.a))

vetr <- function(..., .VETR_SETTINGS=NULL)
  .Call(
    VALC_validate_args,
    fun.match <- sys.function(sys.parent(1)),
    match.call(
      definition=fun.match,
      call=sys.call(sys.parent(1)),
      envir=parent.frame(2L),
      expand.dots=FALSE
    ),
    match.call(
      definition=fun.match,
      envir=(par.frame <- parent.frame()),
      expand.dots=FALSE
    ),
    par.frame,
    .VETR_SETTINGS
  )

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

library(vetr)

unitizer_sect("Basic Tests", {

  vetr:::len_chr_len(1000L)
  vetr:::len_chr_len(1L)
  vetr:::len_chr_len(1234567890L)
  len0 <- 1234567890000000000000000
  vetr:::len_chr_len(len0)

  vetr:::len_as_chr(1000L)
  vetr:::len_as_chr(1L)
  vetr:::len_as_chr(1234567890L)
  vetr:::len_as_chr(len0)

  identical(vetr:::strmlen(lorem), nchar(lorem))
  vetr:::strmlen(lorem, 100L)

  identical(vetr:::strmcpy(lorem), lorem)
  vetr:::strmcpy("")
  vetr:::strmcpy(lorem, 20L)
  identical(nchar(vetr:::strmcpy(lorem, 20L)), 20L)

  vetr:::smprintf2("%s %s", lorem, lorem, 10L)
  vetr:::smprintf2("%s %s hello world there", lorem, lorem, 10L)

  lorem # make sure lorem unchanged

  vetr:::ucfirst("hello WORLD")
  vetr:::lcfirst("HELLO world")

  vetr:::strbullet(
    c(
      "hello world\nhow are things today",
      "once upon a time\nlived a funny duck"
    )
  )
  vetr:::strbullet("hello\nblah\n", bullet="  - ", ctd="    ")

  vetr:::strbullet(1:10) # error

  vetr:::collapse(letters[1:5])
  vetr:::collapse(letters[1:5], sep="\n")
  vetr:::collapse(character())
})
unitizer_sect("numbers as character", {
  vetr:::num_as_chr(100)
  vetr:::num_as_chr(100.01)

  # switch ot scientific

  num0 <- 1e9 + 0.1
  num1 <- -1e9 - 0.1

  # need to sub leading zeros before 'e' due to different display in
  # windows vs nix
  #
  sub("e[+-]?\\K0*", "", vetr:::num_as_chr(num0), perl=TRUE)
  sub("e[+-]?\\K0*", "", vetr:::num_as_chr(num1), perl=TRUE)

  vetr:::num_as_chr(num0, as.int=TRUE)
  vetr:::num_as_chr(num1, as.int=TRUE)

  num2 <- 1e9 - 0.1
  num3 <- -(1e9 - 0.1)

  vetr:::num_as_chr(num2)
  vetr:::num_as_chr(num3)

  # corner cases

  vetr:::num_as_chr(NA)
  vetr:::num_as_chr(NaN)
  vetr:::num_as_chr(Inf)
  vetr:::num_as_chr(-Inf)
})
unitizer_sect("smprintf6", {
  vetr:::smprintf6(
    "%s %s %s %s %s %s", "a", "bb", "ccc", "dddd", "eeeee", "ffffff"
  )
  vetr:::smprintf6(
    "%s %s %s %s %s %s", "a", "bb", "ccc", "dddd", "eeeee", "ffffff", 10L
  )
  vetr:::smprintf6(
    "%s %s %s %s %s %s", "a", "bb", "ccc", "dddd", "eeeee", "ffffff", 18L
  )
  # bad format strings
  vetr:::smprintf6(
    "%s %s %s %s", "a", "bb", "ccc", "dddd", "eeeee", "ffffff"
  )
  # bad format strings2 - this one reads memeory it shouldn't
  # vetr:::smprintf6(
  #   "%s %s %s %s %s %s %s %s", "a", "bb", "ccc", "dddd", "eeeee", "ffffff"
  # )
})
unitizer_sect("Corner Cases", {
  # test maxlen overflows

  vetr:::strbullet(c("hello world"), maxlen=5L)  # early fail
  vetr:::strbullet(c("hello world"), maxlen=12L) # fails when adding bullets
  vetr:::strbullet(c("hello world"), maxlen=14L) # works

  # these are all supposed to fail

  vetr:::test_strmcpy()
  vetr:::test_strappend()
  vetr:::test_add_szt()

  vetr:::strmlen(list(), 100L)

  # quickly confirm the other smprintfs work correctly

  vetr:::test_smprintfx()

  vetr:::test_strappend2()   # warning
})

unitizer_sect("substr", {
  vetr:::strsub(lorem.phrases, 25L, TRUE)
  vetr:::strsub(lorem.phrases, 25L, FALSE)

  # UTF8

  vetr:::strsub(lorem.tr.phrases, 25L, TRUE)
  vetr:::strsub(lorem.tr.phrases, 25L, FALSE)

  vetr:::strsub(lorem.ru.phrases, 25L, TRUE)
  vetr:::strsub(lorem.ru.phrases, 25L, FALSE)

  vetr:::strsub(lorem.cn.phrases, 25L, TRUE)
  vetr:::strsub(lorem.cn.phrases, 25L, FALSE)

  # Unfortunately something is going wrong with how out-of-BMP unicode is read
  # in by windows so we have to comment out these tests; see #82
  # vetr:::strsub(lorem.emo.phrases, 25L, TRUE)
  # vetr:::strsub(lorem.emo.phrases, 25L, FALSE)

  # Errors

  vetr:::strsub(lorem.phrases, 1:2, TRUE)
  vetr:::strsub(lorem.phrases, 25L, 1:2)
  vetr:::strsub(1:2, 25L, TRUE)

  vetr:::strsub(lorem.phrases, 2L, TRUE)
  vetr:::strsub(lorem.phrases, 3L, TRUE)  # works
})
unitizer_sect("nchar_u", {
  vetr:::nchar_u(1:10)
  vetr:::nchar_u(c("a", "ab", "abc"))
})
unitizer_sect("char_offsets", {
  vetr:::char_offsets(1:10)
  vetr:::char_offsets(c("a", "ab", "abc"))
})
unitizer_sect("UTF8 corner cases, in UTF-8", {
  # Originally we tried using `Sys.setlocale` but that isn't guaranteed to work
  # e.g. failed on windows

  utf8.kuhn <- readLines('unitizer/helper/UTF-8-test.txt', encoding='UTF-8');
  test.start <- grep("^Here come the tests:", utf8.kuhn, useBytes=TRUE)
  test.start

  utf8.test <- tail(utf8.kuhn, -test.start)
  # suppressWarnings(utf8.test)  # Solaris problems

  nchar.base <- nchar(utf8.test, allowNA=TRUE)
  untranslatable <- is.na(nchar.base)
  nchar.vetr <- vetr:::nchar_u(utf8.test)

  # Not all lines match between nchar and vetr, differences seem to be that the
  # invalid encoding starting \xf4\x90 is resolved to 1 character by base (test
  # 2.3.5), and that base resolves things in the U+D800 - U+DFFF range as 1
  # character (5.1-5.2); perhaps base implements the more forgiving versions of
  # UTF8?

  base.vetr.diff <- !is.na(nchar.base) & nchar.vetr != nchar.base
  # # this actually causes problems with `str`, in particular with `strtrim` so
  # # we can't use it in the tests
  # sprintf(
  #   "%d %d %s",
  #   nchar.base[base.vetr.diff],
  #   nchar.vetr[base.vetr.diff],
  #   utf8.test[base.vetr.diff]
  # )

  # For visual verification, according to docs all test lines should render at
  # 79 characters, but this appears to be in contravention to the UTF8
  # documentation.  This is the body of the e-mail I sent Dr. Kuhn inquiring
  # about the seeming discrepancy (which could possibly be explained by unicode
  # version differences):
  #
  # In section 3.3.3, 4-byte sequence with last byte missing, the sequence in
  # question appears to be f0 80 80 22 where 22 is the double quote ending the
  # sequence.  Furthermore, you state that I should expect to "... see only a
  # single replacement character".
  #
  # What I'm a little confused by is that the sequence "f0 80 80" becomes
  # illegal after the first byte as per table 3.7 from the Unicode Standard
  # v10.0 since the second byte is less than 90.  Then, by the definition of
  # "Maximal subpart of an ill-formed subsequence", it seems that the maximal
  # subpart starting at "f0" is actually "f0" since "f0 80" is not legal and
  # thus not a subpart.  There is a close example in the documentation (p129 of
  # the document in question):
  #
  #    Another example illustrates the application of the concept of maximal
  #    subpart for UTF-8 continuation bytes outside the allowable ranges defined
  #    in Table 3-7.  The UTF-8 sequence <41 E0 9F 80 41> is ill-formed, because
  #    <9F> is not an allowed second byte of a UTF-8 sequence commencing with
  #    <E0>. In this case, there is an unconvertible offset at <E0> and the
  #    maximal subpart at that offset is also <E0>. The subsequence <E0 9F>
  #    cannot be a maximal subpart, because it is not an initial subsequence of
  #    any well-formed UTF-8 code unit sequence.
  #
  # This is basically the same issue with "e0" substituted for "f0" in our case
  # (I think). This would suggest we should see three characters, one for the
  # maximal subpart "f0", and then also one each for the two "80" since they
  # themselves are illegal.  I think that aligns with the "recommended" policy.
  # I'm pretty new at this so I suspect I'm just misunderstanding something.  If
  # you see an obvious mistake in my reasoning I would appreciate you pointing
  # me to it.

  # # Same issue with strtrim
  # paste(
  #   ifelse(nchar.vetr > 79, sprintf("<%d>", nchar.vetr), "  "), utf8.test
  # )

  # Other examples from the Unicode 10.0 docs

  source('unitizer/helper/UTF-8-unicode-10-ex.R', local=TRUE)

  vetr:::nchar_u(unicode.10[1])
  vetr:::nchar_u(unicode.10[2])
  vetr:::nchar_u(unicode.10[3])
  vetr:::nchar_u(unicode.10[4])

  # Confirm offsets are what they should be

  vetr:::char_offsets(unicode.10[4])

  # Check all the critical cases where we transition from legal to illegal
  # sequences

  source('unitizer/helper/UTF-8-critical.R', local=TRUE)

  Map(vetr:::char_offsets, crit.1)
  Map(vetr:::char_offsets, crit.2)
  Map(vetr:::char_offsets, crit.3)
  Map(vetr:::char_offsets, crit.4)
})
unitizer_sect("UTF-8 corner cases - other encodings", {

  source('unitizer/helper/latin-1.R', local=TRUE)

  lapply(lat.1.1, vetr:::char_offsets)
  lapply(lat.1.2, vetr:::char_offsets)

  vetr:::strsub(lat.1.1, 3L, mark=FALSE)
  vetr:::strsub(lat.1.2, 3L, mark=FALSE)
})

# Copyright (C) Brodie Gaslam
#
# This file is part of "vetr - Trust, but Verify"
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.

library(vetr)

set.seed(1)
unitizer_sect("Tokens Pass", {
  vet(INT.1, 1)
  vet(INT.1.POS, 1)
  vet(INT.1.NEG, -1)
  vet(INT.1.POS.STR, 1)
  vet(INT.1.NEG.STR, -1)
  vet(INT, -1:1)
  vet(INT.POS, 0:3)
  vet(INT.NEG, 0:-3)
  vet(INT.POS.STR, 1:3)
  vet(INT.NEG.STR, -(1:3))
  vet(NUM.1, 1.44)
  vet(NUM.1.POS, 1.44)
  vet(NUM.1.NEG, -1.44)
  vet(NUM, runif(5))
  vet(NUM.POS, runif(5))
  vet(NUM.NEG, -runif(5))
  vet(CHR, character())
  vet(CHR.1, "hello")
  vet(CHR, letters)
  vet(CPX, 1:10 + .5i)
  vet(CPX.1, 1 + .5i)
  vet(LGL, c(TRUE, FALSE))
  vet(LGL.1, TRUE)
} )
unitizer_sect("Tokens Fail", {
  vet(INT.1, 1.2)
  vet(INT.1, 1:2)
  vet(INT.1, NA_integer_)
  vet(INT.1, Inf)
  vet(INT.1.POS, -1)
  vet(INT.1.POS, 1:2)
  vet(INT.1.NEG, 1)
  vet(INT.1.NEG, -(1:2))
  vet(INT.1.POS.STR, 0)
  vet(INT.1.NEG.STR, 0)
  vet(INT, c(-1:1, NA_integer_))
  vet(INT, letters)
  vet(INT.POS, -(1:3))
  vet(INT.NEG, 1:3)
  vet(INT.POS.STR, 0:3)
  vet(INT.NEG.STR, -(0:3))
  vet(NUM.1, 1.44 + 1:2)
  vet(NUM.1.POS, -runif(1) - 1)
  vet(NUM.1.NEG, runif(1) + 1)
  vet(NUM, c(NA_real_, 1))
  vet(NUM, NULL)
  vet(NUM.POS, -runif(5) - 1)
  vet(NUM.NEG, runif(5) + 1)
  vet(CHR.1, letters)
  vet(CHR, list(1, 2, 3))
  vet(CPX, list(1, 2, 3))
  vet(CPX.1, list(1, 2, 3))
  vet(LGL, NA)
  vet(LGL, letters)
  vet(LGL.1, 1:2 == 1:2)
})
unitizer_sect("Custom expressions", {
  vet(. > 5, 1:10)
  vet(. > 5, 6:10)

  # corner cases

  vet(.(c(TRUE, NA, TRUE)), 1:5)
  vet(.(1:5), 1:5)
  vet(.(1:5, 1:5), 1:5)      # error
  vet(.(list(1, 2, 3)), 1:3) # error

  vet(.(c('hello world', 'goodbye moon')), 1:3)
})
unitizer_sect("Compound Expressions", {
  vet(INT.1 || NULL, 1)    # Pass
  vet(INT.1 || NULL, NULL) # Pass
  vet(INT.1 || NULL, 1.4)  # Fail
  vet(INT.1 || NULL || character(3L), 1)    # Pass
  vet(INT.1 || NULL || character(3L), 1.2)  # Fail
  vet(INT.1 || NULL || character(3L), letters)  # Fail

  # Pass

  vet(
    (matrix(integer(), 0) && nrow(.) == ncol(.)) || NULL,
    matrix(1:16, nrow=4)
  )
  # Fail

  vet(
    (matrix(integer(), 0) && nrow(.) == ncol(.)) || NULL,
    matrix(1:16, nrow=2)
  )
  # Fail

  vet(
    (matrix(integer(), 0) && nrow(.) == ncol(.)) || NULL,
    matrix(runif(16), nrow=4)
  )
  # custom expression partially defined in parent env

  exp.a <- quote(all(. > 0))
  exp.b <- quote(is.vector(.))

  vet(exp.a && exp.b, -(1:3))

  # some testing of nesting, this could conflict with prior exp.a
  # if not done properly

  local({
    exp.a <- quote(all(. < 0))
    vet(exp.a, -(1:3))
  })
  # Duplicate expressions should get collapsed in error message

  vet(1 || "a" || 1 || "a" || 1 || letters, 1:3)
})

unitizer_sect("Other Return Modes", {
  vet(INT.1 || NULL || LGL, "hello", format="text")
  vet(INT.1 || NULL || LGL, "hello", format="raw")
  vet(INT.1 || NULL || LGL, "hello", format="full")
  vet(INT.1 || NULL || LGL, "hello", format="halloween")
  vet(INT.1 || NULL || LGL, "hello", format=1:10)

  vet(INT.1 || NULL || LGL, "hello", format="text", stop=TRUE)
  vet(INT.1 || NULL || LGL, "hello", format="text", stop=1:3)
})

unitizer_sect("Multi-line Stuff", {
  # with a validator with message attached
  vet(
    NO.NA,
    c(234234131431, 123413413413, 1341341341, 12341234134, 562456234, 24624624,
      2452345234, 2345234524, 23452452, 2243524352, 254254234, 2452452435, NA)
  )
  vet(
    NO.NA || !anyNA(.),
    c(234234131431, 123413413413, 1341341341, 12341234134, 562456234, 24624624,
      2452345234, 2345234524, 23452452, 2243524352, 254254234, 2452452435, NA)
  )
  # No message
  vet(
    !anyNA(.),
    c(234234131431, 123413413413, 1341341341, 12341234134, 562456234, 24624624,
      2452345234, 2345234524, 23452452, 2243524352, 254254234, 2452452435, NA)
  )
  vet(!anyNA(.), c(234234131431, 123413413413, NA))

  # stored validation

  val.exp <- quote(!anyNA(.))
  vet(val.exp, c(234234131431, 123413413413, NA))
})
unitizer_sect("Embedded String Errors", {
  vet(all_bw(., 0, 1), 0:5)
  vet(all.equal(., 1:5), 1:6)
})

unitizer_sect("Language", {
  # Note issue #18; not 100% sure this is correct, actually it should be, the
  # validator expression is always substituted, and any symbols pointing to
  # language are used as language.
  vet(quote(quote(a + b)), quote(x2 + x3))
  x <- quote(quote(a + b))
  vet(x, quote(x2 + x3))
  vet(quote(a + b), quote(2 + x3))
  vet(quote(a + b), quote(x1 + x2 + x3))

  # Test recursive substitution across environments, first check that all the
  # symbols we use don't actually exist currently

  unlist(lapply(c('aaA', 'bbB', 'ccC', 'ddD', 'eeE'), find))# should be length 0

  x <- quote(aaA + bbB)
  my.env <- new.env()
  my.env$y <- quote(ccC - ddD)

  # FALSE because `x` is expanded

  evalq(vet(quote(x * y), quote(A * (B - C))), envir=my.env)

  # TRUE because `eeE` is not expanded (but `y` is)

  evalq(vet(quote(eeE * y), quote(A * (B - C))), envir=my.env)

  # TRUE because expansion matches

  evalq(vet(quote(x * y), quote((A + D) * (B - C))), envir=my.env)

  # potentialy infinite recursion

  expA <- expB <- expC <- expD <- expE <- 0
  expA <- quote(expB && expC)
  expB <- quote(expD * expE)
  expE <- quote(expA || expD)

  vet(expA, TRUE)

  # check that symbols (i.e. not call) are resolved recursively too

  expE <- quote(expA)
  vet(expA, TRUE)

  # Check that `..` is expanded properly

  . <- quote(. > 0)
  vet(.., 1.4)
  . <- quote(numeric(1L))
  vet(.., 1.5)
})
unitizer_sect("Errors", {
  vet(1, 1, env="hello")

  vet(`&&`(1, 2, 3), 1)
  vet(`||`(1, 2, 3), 1)
})

unitizer_sect("Custom tokens", {
  cust.tok.1 <- vet_token(quote(TRUE), "%sshould be logical(1L)")

  vet(cust.tok.1, TRUE)
  vet(cust.tok.1, 1:2)

  # impossible tokens

  vet_token(quote(TRUE), "should be logical(1L)")
  vet_token(quote(TRUE), letters)

  # hack impossile token (`vet_token` itself wont allow it)

  cust.tok.2 <- quote(. > 2)
  attr(cust.tok.2, "err.msg") <- letters

  vet(cust.tok.2, TRUE)
})

unitizer_sect("Result Buffer", {
  # testing that result buffer expands correctly

  set1 <- vetr_settings(result.list.size.init=1)

  vet.exp <- quote(1 || 1:2 || 1:3 || 1:4 || 1:5 || 1:6 || 1:7 || 1:8)

  vet(vet.exp, 1:8, settings=set1)
  vet(vet.exp, 1:9, settings=set1)

  set2 <- vetr_settings(result.list.size.init=1, result.list.size.max=7)

  vet(vet.exp, 1:8, settings=set2)
  vet(vet.exp, 1:9, settings=set2)

  set3 <- vetr_settings(result.list.size.init=1, result.list.size.max=8)

  vet(vet.exp, 1:8, settings=set3)
  vet(vet.exp, 1:9, settings=set3)

  # impossible settings

  set4 <- vetr_settings(result.list.size.init="hello", result.list.size.max=8)
  set5 <- vetr_settings(result.list.size.init=1, result.list.size.max="hello")

  vet(1, 1, settings=set4)
  vet(1, 1, settings=set5)
})

unitizer_sect("pkg::fun calls (issue #100)", {
  vet(base::sum(.), 1:10)
  vet((base::.)(identity), is.function)
  vet((base::.)(identity), is.integer)
})
unitizer_sect("promises (issue #106)", {
  env1 <- new.env()
  delayedAssign("x", stop('error 1'), assign.env=env1)
  env2 <- new.env()
  delayedAssign("x", stop('error 2'), assign.env=env2)
  env0 <- list2env(list(x=TRUE), new.env())

  vet(env1, env0)   # error 1
  vet(env0, env2)   # error 2
})

unitizer_sect("pkg::fun calls v2 (issue #112)", {
  vetr::vet(base::sum(), 1)
  vetr::vet(base::sum, 1)
  # Recognize as is expression (see also #100 above)
  vetr::vet((.)(isTRUE), identity)  
  vetr::vet(base:::sum(), 1)
})

---
title: "alike"
author: "Brodie Gaslam"
output:
    rmarkdown::html_vignette:
        toc: true
        css: styles.css

vignette: >
  %\VignetteIndexEntry{alike}
  %\VignetteEngine{knitr::rmarkdown}
  %\usepackage[utf8]{inputenc}
---

```{r global_options, echo=FALSE}
knitr::opts_chunk$set(error=TRUE, comment=NA)
library(vetr)
```

## What is Alikeness?

`alike` is similar to `all.equal` from base R except it only compares object
structure.  As with `all.equal`, the first argument (`target`) must be matched
by the second (`current`).

```{r}
library(vetr)
alike(integer(5), 1:5)      # different values, but same structure
alike(integer(5), 1:4)      # wrong size
alike(integer(26), letters) # same size, but different types
```

`alike` only compares structural elements that are defined in `target` (a.k.a.
the template).  This allows "wildcard" templates.  For example, we consider
length zero vectors to have undefined length so those match vectors of any
length:

```{r}
alike(integer(), 1:5)
alike(integer(), 1:4)
alike(integer(), letters)  # type is still defined and must match
```

Similarly, if a template does not specify an attribute, objects with any value
for that attribute will match:

```{r}
alike(list(), data.frame())  # a data frame is a list with a attributes
alike(data.frame(), list())  # but a list does not have the data.frame attributes
```

As an extension to the wildcard concept, we interpret partially specified [core
R attributes](#Special Attributes).  Here we allow any three column integer
matrix to match:

```{r}
mx.tpl <- matrix(integer(), ncol=3)          # partially specified matrix
alike(mx.tpl, matrix(sample(1:12), nrow=4))  # any number of rows match
alike(mx.tpl, matrix(sample(1:12), nrow=3))  # but column count must match
```

or a data frame of arbitrary number of rows, but same column structure as `iris`:

```{r}
iris.tpl <- iris[0, ]                        # no rows, but structure is defined
alike(iris.tpl, iris[1:10, ])                # any number of rows match
alike(iris.tpl, CO2)                         # but column structure must match
```

"alikeness" is complex to describe, but should be intuitive to grasp. We
recommend you look `example(alike)` to get a sense of "alikeness".  If you want
to understand the specifics, read on.

## Declarative Comparison

`alike`'s template based comparison is declarative.  You declare what structure
an object is expected to implement, and `vetr` infers all the computations
required to verify that is so.  This makes is particularly well suited for
enforcing structural requirements for S3 objects.  The S4 system does this and
more, but S3 objects are still used extensively in R code, and sometimes S4
classes are not appropriate.

There are several advantages to template based comparisons:

* Often times it is simpler to define a template than to write out all the
  checks to confirm an object conforms to a particular structure.
* We can generate the template from a known correct instance of an object and
  [abstract away](#Abstracting-Existing-Objects) the elements that are not
  specific to the prototype (this is particularly valuable for otherwise complex
  objects).
* We can produce plainish-english interpretations of structural mismatches since
  we are dealing with a known limited set of comparisons.

The template concept was inspired by `vapply`.

## Object Comparison

### Overview

`alike` compares objects on [type](#type-comparison),
[length](#length-comparison), and attributes. Recursive structures are compared
element by element.  [Language objects](#language-objects) and
[functions](#functions) are compared specially because the concept of a value
within those is more complex (e.g., is the `+` in `x + y` just a value?).

We will defer discussion of attribute comparison to the [attributes
section](#attribute-comparison).

### Length Comparison

Objects must be the same length to be `alike`, unless the template (`target`) is
zero length, in which case the object may be any length.
[Environments](#environments) are an exception: we only require that all the
elements present in `target` be present in `current`.  Also, note that calls to
`(` are ignored in [language objects](#language-objects), which may affect
length computation.

### Type Comparison

Type comparison is done on type (i.e. the `typeof`) with some adjustments to
better align comparisons to "percieved" types as opposed to internal storage
types.

#### Numerics and Integers

We allow integer vectors to be considered numeric, and [short](#fuzzylen)
integer-like numerics to be treated as integers:

```{r}
alike(1L, 1)     # `1` is not technically integer, but we treat it as such
alike(1L, 1.1)   # 1.1 is not integer-like
alike(1.1, 1L)   # integers can match numerics
```

This feature is designed to simplify checks for integer-like numbers.  The
following two expressions are roughly equivalent:

```{r, eval=FALSE}
stopifnot(length(x) == 1L && (is.integer(x) || is.numeric(x) && floor(x) == x))
stopifnot(alike(integer(1L), x))
```

See `?type_alike` for additional details.

<a name="fuzzylen"></a>Note that we only check numerics of length <= 100 for
integerness to avoid full scans on large vectors.  We  expect that the primary
source of these integer-like numerics is hand input vectors (e.g. `c(1, 2, 3)`),
so hopefully this compromise is not too limiting.  You can modify the threshold
length for this treatment via the `fuzzy.int.max.len` parameter to the
`settings` objects (see `?vetr_settings`).

#### Functions

Closures, builtins, and specials are all treated as a single type, even though
internally they are stored as different types.

### Recursive Objects

`alike` will recurse through lists (and by extension data frames), pairlists,
expressions, and environments and will check pairwise alikeness between the
corresponding elements of the `target` and `current` objects.

<a name="environments"></a>Environments have slightly different comparison rules
in two respects:

* only the elements present in the template are checked, so `current` may have
  additional items
* if the template is the global environment, then `current` must be too (this is
  because the global environment is often littered with many objects, and
  explicitly comparing it to another environment could be computationally
  expensive)

`NULL` elements within templates in recursive objects are considered undefined
and as such act like wildcards:

```{r}
## two NULLs match two length list
alike(list(NULL, NULL), list(1:10, letters))
## but not three length list
alike(list(NULL, NULL), list(1:10, letters, iris))
```

Note that top level `NULL`s do not act as wildcards:

```{r}
alike(NULL, 1:10)                   # NULL only matches NULL
```

Treating `NULL` inconsistently depending on whether it is nested or not is a
compromise designed to make `alike` a better fit for argument validation because
arguments that are `NULL` by default are fairly common.

`alike` will check for self-referential loops in nested environments and prevent
infinite recursion.  If you somehow introduce a self-referential structure in a
template without using environments then `alike` will get stuck in an infinite
recursion loop.

We are currently considering adding new comparison modes for lists that would
allow for checks more similar to environments (see
[#29](https://github.com/brodieG/vetr/issues/29)).

### Language Objects, Formulas, and Functions

Alikeness for these types of objects is a little harder to define.  We have
settled on somewhat arbitrary semantics, though hopefully they are intuitive.
These may change in the future as we gain experience using `alike` with these
types of objects.  This is particularly true of functions.

Language objects are also compared recursively, but alikeness has a slightly
different meaning for them:

#### Language Objects

```{r}
alike(quote(sum(a, b)), quote(sum(x, y)))   # calls are consistent
alike(quote(sum(a, b)), quote(sum(x, x)))   # calls are inconsistent
alike(quote(mean(a, b)), quote(sum(x, y)))  # functions are different
```

Since variables can contain anything we do not require them to match directly
across calls.  In the examples above the second call fails because the template
defines different variables for each argument, but the `current` object uses the
same variable twice.  The third call fails because the functions are different
and as such the calls are fundamentally different.

If a function is defined in the calling frame, `alike` will `match.call` it
prior to testing alikeness:

```{r}
fun <- function(a, b, c) NULL
alike(quote(fun(p, q, p)), quote(fun(y, x, x)))
# `match.call` re-orders arguments
alike(quote(fun(p, q, p)), quote(fun(b=y, x, x)))
```

Constants match any constants, but keep in mind that expressions like `1:10` or
`c(1, 2, 3)` are calls to `:` and `c` respectively, not constants in the context
of language objects.

`NULL` is a wild card in calls as well:

```{r}
str(one.arg.tpl <- as.call(list(NULL, NULL)))
alike(one.arg.tpl, quote(log(10)))
alike(one.arg.tpl, quote(sd(runif(20))))
alike(one.arg.tpl, quote(log(10, 10)))
```

Calls to `(` are ignored when comparing calls since parentheses are redundant in
call trees because the tree structure encodes operation precedence independent
of operator precedence.

We concede that the rules for "alikeness" of language objects are arbitrary, but
hope the outcomes of those rules is generally intuitive.  Unfortunately value
and structure are somewhat intertwined for language objects so we must impose
our own view of what is value and what is structure.

#### Formulas

Formulas are treated like calls, except that constants must match:

```{r}
alike(y ~ x ^ 2, a ~ b ^ 2)
alike(y ~ x ^ 2, a ~ b ^ 3)
```

#### Functions

Functions are `alike` if the signature of the `current` function can reasonably
be interpreted as a valid method for the `target` function.

```{r}
alike(print, print.default)   # print can be the generic for print.default
alike(print.default, print)   # but not vice versa
```

A method of a generic must have all arguments present in the generic, with the
same default values if those are defined.  If the generic contains `...` then
the method may have additional arguments, but must also contain `...`.

Potential changes / improvements for function comparison are being considered in
[#35](https://github.com/brodieG/vetr/issues/35).

### S4 and R5 (RC Objects)

S4 and RC objects are considered alike if `current` inherits from
`class(target)`.  Since these objects embed structural information in their
definitions `alike` relies on class alone to establish alikeness.

### Pointer Objects

Objects of the following types are actually references to specific memory
locations:

* External Pointers
* Weak References
* Byte codes

These are typically attached as attributes to other objects that contain the
information required to establish alikeness (e.g. `data.table`, byte-compiled
functions), so we only check their type.

## Attribute Comparison

### Normal Attributes

Much of the structure of an object is determined by attributes.  `alike`
recursively compares object attributes and requires them to be `alike`, unless
the attribute is a [special attribute](#special-attributes) or an environment.
Environments within attributes in the template must be matched by an
environment, but nothing is checked about the environments to avoid expensive
computations on objects that commonly include environments in their attributes
(e.g. formulas); note this is different than the treatment of environments as
actual objects.

Only attributes present in the template object are checked:

```{r}
alike(structure(logical(1L), a=integer(3L)), structure(TRUE, a=1:3, b=letters))
alike(structure(TRUE, a=1:3, b=letters), structure(logical(1L), a=integer(3L)))
```

Attributes present in `current` but missing in `target` may be anything at all.

### Special Attributes

#### Overview

The special attributes are `names`, `row.names`, `dim`, `dimnames`, `class`,
`tsp`, and `levels`.  These attributes are discussed in sections [2.2 and 2.3 of
the R Language
Definition](https://cran.r-project.org/doc/manuals/r-release/R-lang.html#Attributes),
and have well defined and consistently applied semantics in R.  Since the
semantics of these attributes are well known, we are able to define "alikeness"
for them in a more granular way than we can for arbitrary attributes.

We also consider `srcref` to be a special attribute.  This attribute is not
checked.

#### row.names and names

If present in `target`, then must be matched exactly by the corresponding
attribute in `current`, except that:

* zero length `target` `names`/`row.names` (i.e. `character(0L)`) will match any
  character `names`/`row.names`
* a zero character _element_ (i.e. `""`) in a `target` `names`/`row.names`
  character vector will allow any value to match at the corresponding position
  of the `current` `names`/`row.names` vector

```{r}
alike(setNames(integer(), character()), 1:3)
alike(setNames(integer(), character()), c(a=1, b=2, c=3))
alike(setNames(integer(3), c("", "", "Z")), c(a=1, b=2, c=3))
alike(setNames(integer(3), c("", "", "Z")), c(a=1, b=2, Z=3))
```

#### dim

`dim` attributes must be identical between `target` and `current`, except that
if a value of the `dim` _vector_ is zero in `target` then the corresponding
value in `current` can be any value.  This is how comparisons like the following
succeed:

```{r}
mx.tpl <- matrix(integer(), ncol=3)                # partially specified matrix
alike(mx.tpl, matrix(sample(1:12), nrow=4))
alike(mx.tpl, matrix(sample(1:12), nrow=3))        # wrong number of columns
str(mx.tpl)    # notice 0 for 1st dimension
```

#### dimnames

Must also be identical, except that if the `target` value of the `dimnames` list
for a particular dimension is `NULL`, then the corresponding `dimnames` value in
`current` may be anything.  As with `names`, zero character `dimname` element
elements match any name.

```{r}
mx.tpl <- matrix(integer(), ncol=3, dimnames=list(row.id=NULL, c("R", "G", "")))
mx.cur <- matrix(sample(0:255, 12), ncol=3, dimnames=list(row.id=1:4, rgb=c("R", "G", "Blue")))
mx.cur2 <- matrix(sample(0:255, 12), ncol=3, dimnames=list(1:4, c("R", "G", "b")))

alike(mx.tpl, mx.cur)
alike(mx.tpl, mx.cur2)
```

Note that `dimnames` can have a `names` attribute.  This `names` attributed is treated as described in [row.names and names](#row.names-and-names).

```{r}
names(dimnames(mx.tpl))
```

#### class

S3 objects are considered alike if the `current` class inherits from the `target` class.  Note that "inheritance" here is used in a stricter context than in the typical S3 application:

* Every class present in `target` must be present in `current`
* The overlapping classes must be in the same order
* The last class in `current` must be the same as the last class in `target`

To illustrate:

```{r}
tpl <- structure(TRUE, class=c("a", "b", "c"))
cur <- structure(TRUE, class=c("x", "a", "b", "c"))
cur2 <- structure(TRUE, class=c("a", "b", "c", "x"))

alike(tpl, cur)
alike(tpl, cur2)
```

#### tsp

The `tsp` attribute of `ts` objects behaves similarly to the [`dim` attribute](#dim).  Any component (i.e. start, end, frequency) that is set to zero will act as a wild card.  Other components must be identical.  It is illegal to set `tsp` components to zero throught the standard R interface, but you may use `abstract` as a work-around.

#### levels

Levels are compared like [row.names and names](#row.names-and-names).

#### srcref

This attribute is completely ignored.

#### Normal Attributes that Happen To Have Special Names

If an object contains one of the special attributes, but the attribute value is inconsistent with the standard definition of the attribute, `alike` will silently treat that attribute as any other normal attribute.

## Modifying Comparison Behavior

You can use the `settings` parameter to `alike` to modify comparison behavior.
See `?vetr_settings` for details.

## Creating Templates

### From The Ground Up

You can always create your own templates by manually building R structures:

```{r}
int.scalar <- integer(1L)
int.mat.2.by.4 <- matrix(integer(), 2, 4)
# A df without column names
df.chr.num.num <- structure(
  list(character(), numeric(), numeric()), class="data.frame"
)
```

### Abstracting Existing Structures

Alternatively, you can start with a known structure, and abstract away the instance-specific details.  For example, suppose we are sending sample collectors out on the field to record information about iris flowers:

```{r, eval=FALSE}
iris.tpl <- iris[0, ]
alike(iris.tpl, iris.sample.1)  # make sure they submit data correctly
```

Or equivalently:

```{r, eval=FALSE}
iris.tpl <- abstract(iris)
```

`abstract` is an S3 generic defined by `alike` along with methods for common objects.  `abstract` primarily sets the `length` of atomic vectors to zero:

```{r}
abstract(list(c(a=1, b=2, c=3), letters))
```

and also abstracts the `dim`, `dimnames`, and `tsp` attributes if present.  Other attributes are left untouched unless a specific `abstract` method exists for a particular object that also modifies attributes.  One example of such a method is `abstract.lm`, and it does some minor tweaking to the base abstractions to allow us to match models produced by `lm`:

```{r}
df.dummy <- data.frame(x=runif(3), y=runif(3), z=runif(3))
mdl.tpl <- abstract(lm(y ~ x + z, df.dummy))
# TRUE, expecting bi-variate model
alike(mdl.tpl, lm(Sepal.Length ~ Sepal.Width + Petal.Width, iris))
alike(mdl.tpl, lm(Sepal.Length ~ Sepal.Width, iris))
```

The error message is telling us that at index `"terms"` (i.e. `lm(Sepal.Length ~
Sepal.Width, iris)$terms`) `alike` was expecting a call to `+` instead of a
symbol (i.e `Sepal.Width + <somevar>` instead of `Sepal.Width`).  The message
could certainly be more eloquent, but with a little context it should provide
enough information to figure out the problem.

## Performance Considerations

### Sample Timings

We have gone to great lengths to make `alike` fast so that it can be included in
other functions without concerns for what overhead:

```{r}
type_and_len <- function(a, b)
  typeof(a) == typeof(b) && length(a) == length(b)  # for reference

bench_mark(times=1e4,
  identical(rivers, rivers),
  alike(rivers, rivers),
  type_and_len(rivers, rivers)
)
```

While `alike` is slower than `identical` and the comparable bare bones R
function, it is competitive with a bare bones R function that checks types and
length.  As objects grow more complex, `identical` will obviously pull ahead,
though `alike` should be sufficiently fast for most applications:

```{r}
bench_mark(times=1e4,
  identical(mtcars, mtcars),
  alike(mtcars, mtcars)
)
```
In the above example, we are comparing the data frames, their attributes, and
the 11 columns individually.

Keep in mind that the complexity of the `alike` comparison is driven by the
complexity of the template, not the object we are checking, so we can always
manage the expense of the `alike` evaluation.

Comparisons that succeed will be substantially faster than comparisons that fail
as the construction of error messages is non-trivial and we have prioritized
optimization in the success case.

Language object comparison is relatively slow.  We intend to optimize this some day.

Templates with large numbers of attributes (e.g. > 25) may scale non-linearly.
We intend to optimize this some day, though in our experience objects with that
many attributes are rare (note having multiple objects each with a handful
attributes nested in recursive structures is not a problem).

Large objects will be slower to evaluate.  Let us revisit the `lm` example,
though this time we compare our template to itself to ensure that the
comparisons succeed for `alike`, `all.equal`, and `identical`:

```{r}
mdl.tpl <- abstract(lm(y ~ x + z, data.frame(x=runif(3), y=runif(3), z=runif(3))))
# compare mdl.tpl to itself to ensure success in all three scenarios
bench_mark(
  alike(mdl.tpl, mdl.tpl),
  all.equal(mdl.tpl, mdl.tpl),   # for reference
  identical(mdl.tpl, mdl.tpl)
)
```

Even with template as large as `lm` results (check `str(mdl.tpl)`) we can evaluate `alike` thousands of times before the overhead becomes noticeable.

### Pre-defining Templates

Some fairly innocuous R expressions carry substantial overhead.  Consider:

```{r}
df.tpl <- data.frame(a=integer(), b=numeric())
df.cur <- data.frame(a=1:10, b=1:10 + .1)

bench_mark(
  alike(df.tpl, df.cur),
  alike(data.frame(integer(), numeric()), df.cur)
)
```

`data.frame` is a particularly slow constructor, but in general you are best
served by defining your templates (including calls to `abstract`) outside of
your function so they are created on package load rather than every time your
function is called.

## Miscellaneous

### `alike` as an S3 generic

`alike` is not currently an S3 generic, but will likely one in the future
provided we can create an implementation with and acceptable performance
profile.

---
title: "vetR - Trust, but Verify"
author: "Brodie Gaslam"
output:
    rmarkdown::html_vignette:
        toc: true
        css: styles.css

vignette: >
  %\VignetteIndexEntry{vetr}
  %\VignetteEngine{knitr::rmarkdown}
  %\usepackage[utf8]{inputenc}
---

```{r global_options, echo=FALSE}
knitr::opts_chunk$set(error=TRUE, comment=NA)
library(vetr)
```

```{r child='./rmdhunks/trust-but-verify.Rmd'}
```

```{r child='./rmdhunks/declarative-checks.Rmd'}
```

## Vetting Expressions

### Introduction

```{r child='./rmdhunks/vetting-expressions.Rmd'}
```

### Non Standard Evaluation

#### Vetting Expressions are Language Objects

`vet` captures the first argument unevaluated.  For example in:

```{r, eval=FALSE}
vet(. > 0, 1:3)
```

`. > 0` is captured, processed, and evaluated in a special manner.  This is a
common pattern in R (e.g. as in `with`, `subset`, etc.) called Non Standard
Evaluation (NSE).  One additional wrinkle with `vet` is that symbols in the
captured expression are recursively substituted:

```{r, eval=FALSE}
a <- quote(integer() && . > 0)
b <- quote(logical(1L) && !is.na(.))
c <- quote(a || b)

vet(c, 1:3)
```

The above is thus equivalent to:

```{r, eval=FALSE}
vet((integer() && . > 0) || (logical(1L) && !is.na(.)), 1:3)
```

The recursive substitution removes the typical limitation on "programming" with
NSE, although there are a few things to know:

* Symbols in vetting expressions that evaluate to language objects (calls or
  symbols) in the parent frame are substituted with the corresponding language
  object.
* The result of this substitution is implicitly wrapped in parentheses to avoid
  operator precedence problems.
* The function part of a call is never substituted (e.g. the `fun` in
  `fun(a, b)`); this extends to operators.
* `.` is never substituted, though you can work around that by escaping it with
  an additional `.` (i.e. `..`).
* You must take particular care when constructing vetting expressions for
  language objects.

To illustrate the last point, suppose we want to check that an object is a call
in the form `x + y`, then we could use:

```{r, eval=FALSE}
vet(quote(x + y), my.call)       # notice `quote`
```

Or:

```{r, eval=FALSE}
tpl.call <- quote(quote(x + y))  # notice `quote(quote(...))`
vet(tpl.call, my.call)
```

Additionally, you will need to ensure that `x` and `y` themselves do not
evaluate to language objects in the parent frame.

#### Parsing and Evaluation Rules

Once a vetting expression has been recursively substituted, it is parsed into
tokens.  Tokens are the parts of the vetting expression bounded by the `&&` and
`||` operators and optionally enclosed in parentheses.  For example, there are
three tokens in the following vetting expression:

```{r, eval=FALSE}
logical(1) || (numeric(1) && (. > 0 & . < 1))
```

They are `logical(1)`, `numeric(1)`, and `. > 0 & . < 1`.  The last
token is just one token not because of the parentheses around it but because it
is a call to `&` as opposed to `&&`.  Here we use the parentheses to remove
parsing ambiguity caused by `&` and `&&` having the same operator precedence.

After the tokens have been identified they are classified as standard tokens or
template tokens.  Standard tokens are those that contain the `.` symbol.  Every
other token is considered a template token.

Standard tokens are further processed by substituting any `.` with the value
of the object being vetted.  These tokens are then evaluated and if
`all(<result-of-evaluation>)` is `TRUE` then the tokens pass, otherwise they
fail. Note `all(logical(0L))` is TRUE.  With:

```{r}
vet(. > 0, 1:3)
```

`. > 0` becomes `1:3 > 0`, which evaluates to `c(TRUE, TRUE, TRUE)` and the
token passes.

Template tokens, i.e. tokens without a `.` symbol, are evaluated and the
resulting R object is sent along with the object to vet to `alike` for
structural comparison.  If `alike` returns `TRUE` then the token passes,
otherwise it fails.

Finally, the result of evaluating each token is plugged back into the original
expression.  So<sup>1</sup>:

```{r, eval=FALSE}
vet(logical(1) || (numeric(1) && (. > 0 & . < 1)), 42)
# becomes:
alike(logical(1L), 42) || (alike(numeric(1L), 42) && all(42 > 0 & 42 < 1))
# becomes:
FALSE || (TRUE && FALSE)
# becomes:
FALSE
```

And the vetting fails:

```{r}
vet(logical(1) || (numeric(1) && (. > 0 & . < 1)), 42)
```

### Special Cases

If you need to reference a literal dot (`.`) in a token, you can escape it by
adding another dot so that `.` becomes `..`.  If you want to reference `...`
you'll need to use `....`.  If you have a standard token that does not
reference the vetting object (i.e. does not use `.`) you can mark it as a
standard token by wrapping it in `.()` (if you want to use a literal `.()` you
can use `..()`).

If you need `&&` or `||` to be interpreted literally you can wrap the call in
`I` to tell `vet` to treat the entire call as a single token:

```{r, eval=FALSE}
I(length(a) == length(b) && . %in% 0:1)
```

`vet` will stop searching for tokens at the first call to a function other than
`(`, `&&`, and `||`.  The use of `I` here is just an example of this behavior
and convenient since `I` does not change the meaning of the vetting token.  An
implication of this is you should not nest template tokens inside functions as
`vet` will not identify them as template tokens and you may get unexpected
results.  For example:

```{r, eval=FALSE}
I(logical(1L) && my_special_fun(.))
```

will always fail because `logical(1L)` is part of a standard token and is
evaluated as `FALSE` rather than used a template token for a scalar logical.

## In Functions

The `vetr` function streamlines parameter checks in functions.  It behaves just
like `vet`, except that you need only specify the vetting expressions.  The
objects to vet are captured from the function environment:

```{r}
fun <- function(x, y, z) {
  vetr(
    matrix(numeric(), ncol=3),
    logical(1L),
    character(1L) && . %in% c("foo", "bar")
  )
  TRUE  # do work...
}
fun(matrix(1:12, 3), TRUE, "baz")
fun(matrix(1:12, 4), TRUE, "baz")
fun(matrix(1:12, 4), TRUE, "foo")
```

The arguments to `vetr` are matched to the arguments of the enclosing function
in the same way as with `match.call`.  For example, if we wished to vet just
the third argument:

```{r}
fun <- function(x, y, z) {
  vetr(z=character(1L) && . %in% c("foo", "bar"))
  TRUE  # do work...
}
fun(matrix(1:12, 3), TRUE, "baz")
fun(matrix(1:12, 4), TRUE, "bar")
```

Vetting expressions work the same way with `vetr` as they do with `vet`.

## Performance Considerations

### Benchmarks

`vetr` is written primarily in C to minimize the performance impact of adding
validation checks to your functions.  Performance should be faster than using
`stopifnot` except for the most trivial of checks.  The `vetr` function itself
carries some additional overhead from matching arguments, but it should still be
faster than `stopifnot` except in the simplest of cases.  Here we run our checks
on valid iris objects we used to illustrate [declarative
checks](#declarative-checks-with-templates):

```{r}
vetr_iris <- function(x) vetr(tpl.iris)

bench_mark(times=1e4,
  vet(tpl.iris, iris),
  vetr_iris(iris),
  stopifnot_iris(iris)   # defined in "Templates" section
)
```

Performance is optimized for the success case.  Failure cases should still
perform reasonably well, but will be slower than most success cases.

### Templates and Performance

Complex templates will be slower to evaluate than simple ones, particularly for
lists with lots of nested elements.  Note however that the cost of the vetting
expression is a function of the complexity of the template, not that of the
value being vetted.

We recommend that you predefine templates in your package and not in the
validation expression since some seemingly innocuous template creation
expressions carry substantial overhead:

```{r}
bench_mark(data.frame(a=numeric()))
```

In this case the `data.frame` call alone take over 100us.  In your package code
you could use:

```{r}
df.tpl <- data.frame(a=numeric())

my_fun <- function(x) {
  vetr(x=df.tpl)
  TRUE    # do work
}
```

This way the template is created once on package load and re-used each time your
function is called.

## Alternatives

There are many alternatives available to `vetr`.  We do a survey of the
following in our [parameter validation functions][4] review:

```{r child='./rmdhunks/related-packages.Rmd'}
```

----
<sup>1</sup>We take some liberties in this example for clarity.  For instance, `alike` returns a character vector on failure, not `FALSE`, so really what `vet` is doing is `isTRUE(alike(...))`.

[1]: ./vetr.html
[2]: ./alike.html
[3]: #non-standard-evaluation
[4]: http://htmlpreview.github.io/?https://github.com/brodieG/vetr/blob/master/extra/compare.html

## Declarative Checks with Templates

### Templates

Declare a template that an object should conform to, and let `vetr` take care of
the rest:

```{r}
library(vetr)
tpl <- numeric(1L)
vet(tpl, 1:3)
vet(tpl, "hello")
vet(tpl, 42)
```

The template concept is based on `vapply`, but generalizes to all S3 objects and
adds some special features to facilitate comparison.  For example, zero length
templates match any length:

```{r}
tpl <- integer()
vet(tpl, 1L:3L)
vet(tpl, 1L)
```

And for convenience short (<= 100 length) integer-like numerics are considered
integer:

```{r}
tpl <- integer(1L)
vet(tpl, 1)       # this is a numeric, not an integer
vet(tpl, 1.0001)
```

`vetr` can compare recursive objects such as lists, or data.frames:

```{r}
tpl.iris <- iris[0, ]      # 0 row DF matches any number of rows in object
iris.fake <- iris
levels(iris.fake$Species)[3] <- "sibirica"   # tweak levels

vet(tpl.iris, iris)
vet(tpl.iris, iris.fake)
```

From our declared template `iris[0, ]`, `vetr` infers all the required checks.
In this case, `vet(iris[0, ], iris.fake, stop=TRUE)` is equivalent to:

```{r}
stopifnot_iris <- function(x) {
  stopifnot(
    is.data.frame(x),
    is.list(x),
    length(x) == length(iris),
    identical(lapply(x, class), lapply(iris, class)),
    is.integer(attr(x, 'row.names')),
    identical(names(x), names(iris)),
    identical(typeof(x$Species), "integer"),
    identical(levels(x$Species), levels(iris$Species))
  )
}
stopifnot_iris(iris.fake)
```

`vetr` saved us typing, and the time and thought needed to come up with what
needs to be compared.

You could just as easily have created templates for nested lists, or data frames
in lists.  Templates are compared to objects with the `alike` function.  For a
thorough description of templates and how they work see the [`alike`
vignette][2].  For template examples see `example(alike)`.

### Auto-Generated Error Messages

Let's revisit the error message:

```{r}
vet(tpl.iris, iris.fake)
```

It tells us:

* The reason for the failure
* What structure would be acceptable instead
* The location of failure `levels(iris.fake$Species)[3]`

`vetr` does what it can to reduce the time from error to resolution.  The
location of failure is generated such that you can easily copy it in part or
full to the R prompt for further examination.

# Valaddin

Snippet used to compare `valaddin` to `vetr` performance.  This is not actually
generated as part of vignettes because it will then require `valaddin` as part
of the build package, which brings in unwanted dependencies (via suggests) that
make the build process slow on travis.

```{r}
secant <- function(f, x, dx) (f(x + dx) - f(x)) / dx

secant_valaddin <- valaddin::firmly(secant, list(~x, ~dx) ~ is.numeric)

secant_stopifnot <- function(f, x, dx) {
  stopifnot(is.numeric(x), is.numeric(dx))
  secant(f, x, dx)
}

secant_vetr <- function(f, x, dx) {
  vetr(x=numeric(), dx=numeric())
  secant(f, x, dx)
}

library(microbenchmark)
microbenchmark(
  secant_valaddin(log, 1, .1),
  secant_stopifnot(log, 1, .1),
  secant_vetr(log, 1, .1)
)
```

* `stopifnot` by R Core
* [`vetr`](https://github.com/brodieG/vetr) by Yours Truly
* [`asserthat`](https://github.com/hadley/assertthat) by Hadley Wickham
* [`assertive`](https://www.r-pkg.org/pkg/assertive) by Richie Cotton
* [`checkmate`](https://github.com/mllg/checkmate) by Michel Lang

The following packages also perform related tasks, although we have not used
them and do not review them:

* [`valaddin`](https://github.com/egnha/valaddin) v0.1.0 by Eugene Ha, a
  framework for augmenting existing functions with validation contracts.
  Currently the package is undergoing a major overhaul so we will add it to the
  comparison once the new release (v0.3.0) is out.
* [`ensurer`](https://github.com/smbache/ensurer) v1.1 by Stefan M. Bache, a
  framework for flexibly creating and combining validation contracts.  The
  development version adds an experimental method for creating type safe
  functions, but it is not published to CRAN so we do not test it here.
* [`validate`](https://github.com/data-cleaning/validate) by Mark van
  der Loo and Edwin de Jonge, with a primary focus on validating data in data
  frames and similar data structures.
* [`assertr`](https://github.com/tonyfischetti/assertr) by Tony Fischetti, also
  focused on data validation in data frames and similar structures.
* [`types`](https://github.com/jimhester/types) by Jim Hester, which implements
  but does not enforce type hinting.
* [`argufy`](https://github.com/gaborcsardi/argufy) by Gábor Csárdi, which
  implements parameter validation via roxygen tags (not released to CRAN).
* [`typed`](https://moodymudskipper.github.io/typed/) by Antoine Fabri, which
  enforces types of symbols, function parameters, and return values.
* [`erify`](https://github.com/flujoo/erify/) by Renfei Mao, with a focus on
  readable error messages.

You can combine templates with `&&` / `||`:

```{r}
vet(numeric(1L) || NULL, NULL)
vet(numeric(1L) || NULL, 42)
vet(numeric(1L) || NULL, "foo")
```

Templates only check structure.  When you need to check values use `.` to
refer to the object:

```{r}
vet(numeric(1L) && . > 0, -42)  # strictly positive scalar numeric
vet(numeric(1L) && . > 0, 42)
```

If you do use the `.` symbol in your vetting expressions in your packages, you
will need to include `utils::globalVariables(".")` as a top-level call to avoid
the "no visible binding for global variable '.'" R CMD check NOTE.

You can compose vetting expressions as language objects and combine them:

```{r}
scalar.num.pos <- quote(numeric(1L) && . > 0)
foo.or.bar <- quote(character(1L) && . %in% c('foo', 'bar'))
vet.exp <- quote(scalar.num.pos || foo.or.bar)

vet(vet.exp, 42)
vet(vet.exp, "foo")
vet(vet.exp, "baz")
```

`all_bw` is available for value range checks (~10x faster than
`isTRUE(all(. >= x & . <= y))` for large vectors):

```{r}
vet(all_bw(., 0, 1), runif(5) + 1)
```

There are a number of predefined vetting tokens you can use in your
vetting expressions such as:

```{r}
vet(NUM.POS, -runif(5))    # positive numeric; see `?vet_token` for others
```

Vetting expressions are designed to be intuitive to use, but their
implementation is complex.  We recommend you look at `example(vet)` for usage
ideas, or at the ["Non Standard Evaluation" section of the vignette][3] for the
gory details.

## Trust, but Verify

### Easily

When you write functions that operate on  S3 or unclassed objects you can either
trust that your inputs will be structured as expected, or tediously check that
they are.

`vetr` takes the tedium out of structure verification so that you can trust,
but verify.  It lets you express structural requirements declaratively with
templates, and it auto-generates human-friendly error messages as needed.

### Quickly

`vetr` is written in C to minimize overhead from parameter checks in your
functions.  It has no dependencies.

<!--- No longer in use since we removed microbenchmark suggests -->

```{r echo=FALSE}
mb <- function(..., times=25) {
  if(require(microbenchmark, quietly=TRUE)) {
    mb.c <- match.call()
    mb.c[[1]] <- quote(microbenchmark::microbenchmark)
    res <- eval(mb.c, parent.frame())
    res.sum <- summary(res)
    cat(attr(res.sum, "unit"), "\n")
    print(res.sum[1:5])
  } else {
    warning("Package microbenchmark not available.")
  }
}
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/abstract.R
\name{abstract}
\alias{abstract}
\alias{abstract.data.frame}
\alias{abstract.default}
\alias{abstract.array}
\alias{abstract.matrix}
\alias{abstract.list}
\alias{abstract.lm}
\alias{abstract.environment}
\alias{abstract.ts}
\title{Turn S3 Objects Into Templates}
\usage{
abstract(x, ...)

\method{abstract}{data.frame}(x, ...)

\method{abstract}{default}(x, ...)

\method{abstract}{array}(x, ...)

\method{abstract}{matrix}(x, ...)

\method{abstract}{list}(x, ...)

\method{abstract}{lm}(x, ...)

\method{abstract}{environment}(x, ...)

\method{abstract}{ts}(x, what = c("start", "end", "frequency"), ...)
}
\arguments{
\item{x}{the object to abstract}

\item{...}{arguments for methods that require further arguments}

\item{what, }{for time series which portion of the \code{ts} attribute to
abstract, by default all three are abstracted, but you can select, any one,
two, or all}
}
\value{
abstracted object
}
\description{
Create templates for use by \code{\link{alike}}. Currently somewhat
experimental; behavior may change in future.
}
\details{
\code{abstract} is intended to create templates for use by
\code{\link{alike}}.  The result of abstraction is often a partially
specified object.  This type of object may not be suited for use in typical
R computations and may cause errors (or worse) if you try to use them as
normal R objects.

There is no guarantee that the \code{abstract}ed object is suitable for use
as a template to \code{alike} as is.  You may need to modify it further so
that it suits your purposes.

\code{abstract} is an S3 generic.  The default method will
dispatch on implicit classes, so if you attempt to \code{abstract} an object
without an explicit \code{abstract} method, it will get abstracted based on
its implicit class.  If you define your own \code{abstract} method and do not
wish further abstraction based on implicit classes do not use
\code{\link{NextMethod}}.

S4 and RC objects are returned unchanged.
}
\section{Time Series}{


\code{abstract} replaces the "tsp" attribute with a "tsp_vetr" attribute with the
components specified in \code{what} set to zero.  \code{\link{alike}} will
treat it as a "tsp" attribute except that the zero components become
wildcards.  If you manually create a template object with both
"tsp" and "tsp_vetr" attributes, "tsp_vetr" is treated as a regular
attribute.  \code{vetr} does not consider whether the "ts" class is also set
when attributing special semantics to the "tsp".  The "tsp_vetr" attribute is
required because R does not permit 0 values in "tsp".  Prior to R4.6.0 and
\code{vetr} 2.20, \code{vetr} constructed a "tsp" attribute with zeroes, but it no
longer does that.
}

\examples{
iris.tpl <- abstract(iris)
alike(iris.tpl, iris[1:10, ])
alike(iris.tpl, transform(iris, Species=as.character(Species)))

abstract(1:10)
abstract(matrix(1:9, nrow=3))
abstract(list(1:9, runif(10)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{vetr}
\alias{vetr}
\title{Verify Function Arguments Meet Structural Requirements}
\usage{
vetr(..., .VETR_SETTINGS = NULL)
}
\arguments{
\item{...}{vetting expressions, each will be matched to the enclosing
function formals as with \code{\link[=match.call]{match.call()}} and will be used to validate the
value of the matching formal.}

\item{.VETR_SETTINGS}{a settings list as produced by \code{\link[=vetr_settings]{vetr_settings()}}, or
NULL to use the default settings.  Note that this means you cannot use
\code{vetr} with a function that takes a \code{.VETR_SETTINGS} argument}
}
\value{
TRUE if validation succeeds, otherwise \code{stop} with error message
detailing nature of failure.
}
\description{
Use vetting expressions to enforce structural requirements for function
arguments.  Works just like \code{\link[=vet]{vet()}}, except that the formals of the
enclosing function automatically matched to the vetting expressions provided
in \code{...}.
}
\details{
Only named arguments may be vetted; in other words it is not possible to vet
arguments passed via \code{...}.
}
\note{
\code{vetr} will force evaluation of any arguments that are being
checked (you may omit arguments that should not be evaluate from
\code{vetr})
}
\section{Vetting Expressions}{


Vetting expressions can be template tokens, standard tokens, or any
expression built with them, \code{||}, \code{&&}, and parentheses.  Template tokens
are R objects that define the required structure, much like the \code{FUN.VALUE}
argument to \code{\link[=vapply]{vapply()}}.  Standard tokens are R expressions evaluated and
checked for being \code{all(TRUE)}.

Standard tokens are distinguished from templates by whether they reference
the \code{.} symbol or not.  If you have a need to reference an object bound to
\code{.} in a vetting expression, you can escape the \code{.}  with an extra dot (i.e.
use \code{..}, and \code{...} for \code{..},  and so forth for symbols comprising only
dots).  If you use standard tokens in your packages you will need to include
\code{utils::globalVariables(".")} as a top-level call to avoid the "no visible
binding for global variable '.'"' R CMD check NOTE.  Standard tokens that
return a string like e.g. \code{all.equal(x, .)} will result in that string being
incorporated into the error message.

See \code{vignette('vetr', package='vetr')} and examples for details on how
to craft vetting expressions.
}

\examples{
## Look at `?vet` examples for more details on how to craft
## vetting expressions.

fun1 <- function(x, y) {
  vetr(integer(), LGL.1)
  TRUE   # do some work
}
fun1(1:10, TRUE)
try(fun1(1:10, 1:10))

## only vet the second argument
fun2 <- function(x, y) {
  vetr(y=LGL.1)
  TRUE   # do some work
}
try(fun2(letters, 1:10))

## Nested templates; note, in packages you should consider
## defining templates outside of `vet` or `vetr` so that
## they are computed on load rather that at runtime
tpl <- list(numeric(1L), matrix(integer(), 3))
val.1 <- list(runif(1), rbind(1:10, 1:10, 1:10))
val.2 <- list(runif(1), cbind(1:10, 1:10, 1:10))
fun3 <- function(x, y) {
  vetr(x=tpl, y=tpl && ncol(.[[2]]) == ncol(x[[2]]))
  TRUE   # do some work
}
fun3(val.1, val.1)
try(fun3(val.1, val.2))
val.1.a <- val.1
val.1.a[[2]] <- val.1.a[[2]][, 1:8]
try(fun3(val.1, val.1.a))
}
\seealso{
\code{\link[=vet]{vet()}}, in particular \code{example(vet)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\docType{class}
\name{vetr-interal}
\alias{vetr-interal}
\alias{vetr_bar-class}
\alias{vetr_baz-class}
\title{Test Objects}
\description{
Objects used for testing purposes only.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alike.R
\name{alike}
\alias{alike}
\title{Compare Object Structure}
\usage{
alike(target, current, env = parent.frame(), settings = NULL)
}
\arguments{
\item{target}{the template to compare the object to}

\item{current}{the object to determine alikeness of to the template}

\item{env}{environment used internally when evaluating expressions; currently
used only when looking up functions to \code{\link{match.call}} when
testing language objects, note that this will be overridden by the
environment specified in \code{settings} if any, defaults to the parent
frame.}

\item{settings}{a list of settings generated using \code{vetr_settings}, NULL
for default}
}
\value{
TRUE if target and current are alike, character(1L) describing why
they are not if they are not
}
\description{
Similar to \code{\link{all.equal}}, but compares object structure rather than
value.  The \code{target} argument defines a template that the \code{current}
argument must match.
}
\note{
The semantics of alikeness for language objects, formulas, and
functions may change in the future.
}
\section{alikeness}{


Generally speaking two objects are alike if they are of the same type (as
determined by \code{\link{type_alike}}) and length.  \code{\link{type_alike}}
has special treatment for integer-like numerics and function-like objects.

Attributes on the objects are required to be recursively \code{alike}, though
the following attributes are treated specially: \code{class}, \code{dim},
\code{dimnames}, \code{names}, \code{row.names}, \code{levels}, \code{tsp},
and \code{srcref}.

Exactly what makes two objects \code{alike} is complex, but should be
intuitive.  The best way to understand "alikeness" is to review the examples.
For a thorough exposition see \href{../doc/alike.html}{the vignette}.
}

\examples{
## Type comparison
alike(1L, 1.0)         # TRUE, because 1.0 is integer-like
alike(1L, 1.1)         # FALSE, 1.1 is not integer-like
alike(1.1, 1L)         # TRUE, by default, integers are always considered real

alike(1:100, 1:100 + 0.0)  # TRUE

## We do not check numerics for integerness if longer than 100
alike(1:101, 1:101 + 0.0)

## Scalarness can now be checked at same time as type
alike(integer(1L), 1)            # integer-like and length 1?
alike(logical(1L), TRUE)         # logical and length 1?
alike(integer(1L), 1:3)
alike(logical(1L), c(TRUE, TRUE))

## Zero length match any length of same type
alike(integer(), 1:10)
alike(1:10, integer())   # but not the other way around

## Recursive objects compared recursively
alike(
  list(integer(), list(character(), logical(1L))),
  list(1:10, list(letters, TRUE))
)
alike(
  list(integer(), list(character(), logical(1L))),
  list(1:10, list(letters, c(TRUE, FALSE)))
)

## `NULL` is a wild card when nested within recursive objects
alike(list(NULL, NULL), list(iris, mtcars))
alike(NULL, mtcars)    # but not at top level

## Since `data.frame` are lists, we can compare them recursively:
iris.fake <- transform(iris, Species=as.character(Species))
alike(iris, iris.fake)

## we even check attributes (factor levels must match)!
iris.fake2 <- iris
levels(iris.fake2$Species) <- c("setosa", "versicolor", "africana")
alike(iris, iris.fake2)

## We can use partially specified objects as templates
iris.tpl <- abstract(iris)
str(iris.tpl)
alike(iris.tpl, iris)
## any row sample of iris matches our iris template
alike(iris.tpl, iris[sample(1:nrow(iris), 10), ])
## but column order matters
alike(iris.tpl, iris[c(2, 1, 3, 4, 5)])

## 3 x 3 integer
alike(matrix(integer(), 3, 3), matrix(1:9, nrow=3))
## 3 x 3, but not integer!
alike(matrix(integer(), 3, 3), matrix(runif(9), nrow=3))
## partial spec, any 3 row integer matrix
alike(matrix(integer(), 3), matrix(1:12, nrow=3))
alike(matrix(integer(), 3), matrix(1:12, nrow=4))
## Any logical matrix (but not arrays)
alike(matrix(logical()), array(rep(TRUE, 8), rep(2, 3)))

## In order for objects to be alike, they must share a family
## tree, not just a common class
obj.tpl <- structure(TRUE, class=letters[1:3])
obj.cur.1 <-  structure(TRUE, class=c("x", letters[1:3]))
obj.cur.2 <-  structure(TRUE, class=c(letters[1:3], "x"))

alike(obj.tpl, obj.cur.1)
alike(obj.tpl, obj.cur.2)

## You can compare language objects; these are alike if they are self
## consistent; we don't care what the symbols are, so long as they are used
## consistently across target and current:

## TRUE, symbols are consistent (adding two different symbols)
alike(quote(x + y), quote(a + b))
## FALSE, different function
alike(quote(x + y), quote(a - b))
## FALSE, inconsistent symbols
alike(quote(x + y), quote(a + a))
}
\seealso{
\code{\link{type_alike}}, \code{\link{type_of}},
\code{\link{abstract}}, \code{\link{vetr_settings}} for more control of
settings
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type.R
\name{type_of}
\alias{type_of}
\alias{type_alike}
\title{Fuzzily Compare Types of Objects}
\usage{
type_of(object)

type_alike(target, current, settings = NULL)
}
\arguments{
\item{object}{the object to check the type of}

\item{target}{the object to test type alikeness against}

\item{current}{the object to test the type alikeness of}

\item{settings}{NULL, or a list as produced by \code{\link[=vetr_settings]{vetr_settings()}}}
}
\value{
For \code{type_of} character(1L) the type of the object, for \code{type_alike}
either TRUE, or a string describing why the types are not alike.
}
\description{
Type evaluation and comparison is carried out with special treatment for
numerics, integers, and function types.  Whole number NA-free numeric vectors
of sufficiently short length (<100 by default) representable in the integer
type are considered to be type integer.  Closures, built-ins, and specials
are all treated as type closure.
}
\details{
Specific behavior can be tuned with the \code{type.mode} parameter to the
\code{\link[=vetr_settings]{vetr_settings()}} object passed as the \code{settings} parameter to this function.
}
\examples{
type_of(1.0001)          # numeric
type_of(1.0)             # integer (`typeof` returns numeric)
type_of(1)               # integer (`typeof` returns numeric)
type_of(sum)             # closure (`typeof` returns builtin)
type_of(`$`)             # closure (`typeof` returns special)

type_alike(1L, 1)
type_alike(1L, 1.1)
type_alike(integer(), numeric(100))
type_alike(integer(), numeric(101))  # too long
}
\seealso{
\code{\link[=alike]{alike()}}, \code{\link[=vetr_settings]{vetr_settings()}}, in particular the section about
the \code{type.mode} parameter which affects how this function behaves.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/abstract.R
\name{nullify}
\alias{nullify}
\alias{nullify.default}
\title{Set Element to NULL Without Removing It}
\usage{
nullify(obj, index)

\method{nullify}{default}(obj, index)
}
\arguments{
\item{obj}{the R object to NULL a value in}

\item{index}{an indexing vectors of values to NULL}
}
\value{
object with selected values NULLified
}
\description{
This function is required because there is no straightforward way to
over-write a value in a list with NULL without completely removing the entry
from the list as well.
}
\details{
This returns a copy of the object modified with null slots; it does
not modify the input argument.

Default method will attempt to convert non-list objects to lists
with \code{\link{as.list}}, and then back to whatever they were by using a
function with name \code{paste0("as.", class(obj)[[1L]])}
if it exists and works.  If the object cannot be coerced back
to its original type the corresponding list will be returned.

If this is not appropriate for your object type you can write an S3 method
for it.
}
\note{
attributes are copied from original object and re-applied to final
object before return, which may
not make sense in some circumstances.
}
\examples{
nullify(list(1, 2, 3), 2)
nullify(call("fun", 1, 2, 3), 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/settings.R
\name{vetr_settings}
\alias{vetr_settings}
\title{Generate Control Settings For vetr and alike}
\usage{
vetr_settings(
  type.mode = 0L,
  attr.mode = 0L,
  lang.mode = 0L,
  fun.mode = 0L,
  rec.mode = 0L,
  suppress.warnings = FALSE,
  fuzzy.int.max.len = 100L,
  width = -1L,
  env.depth.max = 65535L,
  symb.sub.depth.max = 65535L,
  symb.size.max = 15000L,
  nchar.max = 65535L,
  track.hash.content.size = 63L,
  env = NULL,
  result.list.size.init = 64L,
  result.list.size.max = 1024L
)
}
\arguments{
\item{type.mode}{integer(1L) in 0:2, defaults to 0, determines how object
types (as in \code{typeof}) are compared: \itemize{
\item 0: integer like numerics (e.g. \code{1.0}) can match against
integer templates, and integers always match real templates; all
function types are considered of the same type
\item 1: integers always match against numeric templates, but not vice
versa, and integer-like numerics are treated only as numerics;
functions only match same function type (i.e. closures only match
closures, builtins builtins, and specials specials)
\item 2: types must be equal for all objects types (for functions, this
is unchanged from 1)
}}

\item{attr.mode}{integer(1L) in 0:2, defaults to 0,  determines strictness of
attribute comparison: \itemize{
\item \code{0} only checks attributes that are present in target, and
uses special comparisons for the special attributes (\code{class},
\code{dim}, \code{dimnames}, \code{names}, \code{row.names},
\code{levels}, \code{srcref}, and \code{tsp}) while requiring other
attributes to be \code{alike}
\item \code{1} is like \code{0}, except all atributes must be
\code{alike}
\item \code{2} requires all attributes to be present in \code{target} and
\code{current} and to be alike
}}

\item{lang.mode}{integer(1L) in 0:1, defaults to 0, controls language
matching, set to \code{1} to turn off use of \code{\link[=match.call]{match.call()}}}

\item{fun.mode}{NOT IMPLEMENTED, controls how functions are compared}

\item{rec.mode}{integer(1L) \code{0} currently unused, intended to control how
recursive structures (other than language objects) are compared}

\item{suppress.warnings}{logical(1L) suppress warnings if TRUE}

\item{fuzzy.int.max.len}{max length of numeric vectors to consider for
integer likeness (e.g. \code{c(1, 2)} can be considered "integer", even
though it is numeric); currently we limit this check to vectors
shorter than 100 to avoid a potentially expensive computation on large
vectors, set to -1 to apply to all vectors irrespective of length}

\item{width}{to use when deparsing expressions; default \code{-1}
equivalent to \code{getOption("width")}}

\item{env.depth.max}{integer(1L) maximum number of nested environments to
recurse through, defaults to 65535L; these are tracked to make sure we do
not get into an infinite recursion loop, but because they are tracked we
keep a limit on how many we will go through, set to -1 to allow unlimited
recursion depth.  You should not need to change this unless you are running
into the recursion limit.}

\item{symb.sub.depth.max}{integer(1L) maximum recursion depth when
recursively substituting symbols in vetting expression, defaults to 65535L}

\item{symb.size.max}{integer(1L) maximum number of characters that a symbol
is allowed to have in vetting expressions, defaults to 15000L.}

\item{nchar.max}{integer(1L) defaults to 65535L, threshold after which
strings encountered in C code are truncated.  This is the read limit.  In
theory \code{vetr} can produce strings longer than that by combining multiple
shorter pieces.}

\item{track.hash.content.size}{integer(1L) (advanced) used to set the initial
size of the symbol tracking vector used with the hash table that detects
recursive symbol substitution.  If the tracking vector fills up it will be
grown by 2x.  This parameter is exposed mostly for developer use.}

\item{env}{what environment to use to match calls and evaluate vetting
expressions, although typically you would specify this with the \code{env}
argument to \code{vet}; if NULL will use the calling frame to
\code{vet/vetr/alike}.}

\item{result.list.size.init}{initial value for token tracking.   This will be
grown by a factor of two each time it fills up until we reach
\code{result.list.size.max}.}

\item{result.list.size.max}{maximum number of tokens we keep track of,
intended mostly as a safeguard in case a logic error causes us to keep
allocating memory.  Set to 1024 as a default value since it should be
exceedingly rare to have vetting expressions with such a large number of
tokens, enough so that if we reach that number it is more likely something
went wrong.}
}
\value{
list with all the setting values
}
\description{
Utility function to generate setting values.  We strongly recommend
that you generate the settings outside of function calls so that setting
generation does not become part of the \code{vet/vetr/alike} evaluation as
that could add noticeable overhead to the function evaluation.
}
\details{
Settings after \code{fuzzy.int.max.len} are fairly low level and exposed mostly
for testing purposes.  You should generally not need to use them.

Note that a successful evaluation of this function does not guarantee a
correct settings list.  Those checks are carried out internally by
\code{vet/vetr/alike}.
}
\examples{
type_alike(1L, 1.0, settings=vetr_settings(type.mode=2))
## better if you are going to re-use settings to reduce overhead
set <- vetr_settings(type.mode=2)
type_alike(1L, 1.0, settings=set)
}
\seealso{
\code{\link{type_alike}}, \code{\link{alike}}, \code{\link{vetr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/all-bw.R
\name{all_bw}
\alias{all_bw}
\title{Verify Values in Vector are Between Two Others}
\usage{
all_bw(x, lo = -Inf, hi = Inf, na.rm = FALSE, bounds = "[]")
}
\arguments{
\item{x}{vector logical (treated as integer), integer, numeric, or character.
Factors are treated as their underlying integer vectors.}

\item{lo}{scalar vector of type coercible to the type of \code{x}, cannot be NA,
use \code{-Inf} to indicate unbounded (default).}

\item{hi}{scalar vector of type coercible to the type of \code{x}, cannot be NA,
use \code{Inf} to indicate unbounded (default), must be greater than or equal to
\code{lo}.}

\item{na.rm}{TRUE, or FALSE (default), whether NAs are considered to be
in bounds.  Unlike with \code{\link[=all]{all()}}, for \code{all_bw} \code{na.rm=FALSE} returns an
error string if there are NAs instead of NA.  Arguably NA, but not NaN,
should be considered to be in \verb{[-Inf,Inf]}, but since \code{NA < Inf} is NA we
treat them as always being out of bounds.}

\item{bounds}{\code{character(1L)} for values between \code{lo} and \code{hi}:
\itemize{
\item \dQuote{[]} include \code{lo} and \code{hi}
\item \dQuote{()} exclude \code{lo} and \code{hi}
\item \dQuote{(]} exclude \code{lo}, include \code{hi}
\item \dQuote{[)} include \code{lo}, exclude \code{hi}
}}
}
\value{
TRUE if all values in \code{x} conform to the specified bounds, a string
describing the first position that fails otherwise
}
\description{
Similar to \code{isTRUE(all(x >= lo & x <= hi))} with default settings,
except that it is substantially faster and returns a string describing the
first encountered violation rather than FALSE on failure.
}
\details{
You can modify the comparison to be strictly greater/less than via the
\code{bounds} parameter, and the treatment of NAs with \code{na.rm}.  Note that NAs are
considered to be out of bounds by default.  While technically incorrect
since we cannot know whether an NA value is in or out of bounds, this
assumption is both conservative and convenient.  Zero length \code{x} will always
succeed.

If \code{x} and \code{lo}/\code{hi} are different types, \code{lo}/\code{hi} will be coerced to the
type of \code{x}.  When \code{lo}/\code{hi} are numeric and \code{x} is integer, if \code{lo}/\code{hi}
values are outside of the integer range then that side will be treated as if
you had used \code{-Inf}/\code{Inf}.  \code{-Inf} and \code{Inf} mean \code{lo} and \code{hi} will be
unbounded for all data types.
}
\examples{
all_bw(runif(100), 0, 1)
all_bw(runif(100) * 2, 0, 1)
all_bw(NA, 0, 1)              # This is does not return NA
all_bw(NA, 0, 1, na.rm=TRUE)

vec <- c(runif(100, 0, 1e12), Inf, 0)
all_bw(vec, 0)      # All +ve numbers
all_bw(vec, hi=0)   # All -ve numbers
all_bw(vec, 0, bounds="(]") # All strictly +ve nums
all_bw(vec, 0, bounds="[)") # All finite +ve nums
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vetr-package.R
\docType{package}
\name{vetr-package}
\alias{vetr-package}
\title{Trust, but Verify}
\description{
Declarative template-based framework for verifying that objects meet
structural requirements, and auto-composing error messages when they do not.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/brodieG/vetr}
  \item Report bugs at \url{https://github.com/brodieG/vetr/issues}
}

}
\author{
\strong{Maintainer}: Brodie Gaslam \email{brodie.gaslam@yahoo.com}

Other contributors:
\itemize{
  \item Paxdiablo (Hash table implementation in src/pfhash.h) [copyright holder]
  \item R Core Team \email{R-core@r-project.org} (Used/adapted several code snippets from R sources, see src/r-copied.c) [copyright holder]
  \item Michael Chirico \email{michaelchirico4@gmail.com} (\href{https://orcid.org/0000-0003-0787-087X}{ORCID}) [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/abstract.R
\name{abstract.ggplot}
\alias{abstract.ggplot}
\title{Experimental Abstraction Method for GGPlot}
\usage{
\method{abstract}{ggplot}(x, ...)
}
\description{
Not entirely sure this can ever work well since so much of \code{ggplot} is
done with \code{proto} objects and those do not really use meta data, which
makes \code{alike} rather useless.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\docType{data}
\name{vet_token}
\alias{vet_token}
\alias{NO.NA}
\alias{NO.INF}
\alias{GTE.0}
\alias{LTE.0}
\alias{GT.0}
\alias{LT.0}
\alias{INT.1}
\alias{INT.1.POS}
\alias{INT.1.NEG}
\alias{INT.1.POS.STR}
\alias{INT.1.NEG.STR}
\alias{INT}
\alias{INT.POS}
\alias{INT.NEG}
\alias{INT.POS.STR}
\alias{INT.NEG.STR}
\alias{NUM.1}
\alias{NUM.1.POS}
\alias{NUM.1.NEG}
\alias{NUM}
\alias{NUM.POS}
\alias{NUM.NEG}
\alias{CHR.1}
\alias{CHR}
\alias{CPX}
\alias{CPX.1}
\alias{LGL}
\alias{LGL.1}
\title{Vetting Tokens With Custom Error Messages}
\format{
An object of class \code{call} of length 2.

An object of class \code{call} of length 2.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.

An object of class \code{call} of length 3.
}
\usage{
vet_token(exp, err.msg = "\%s")

NO.NA

NO.INF

GTE.0

LTE.0

GT.0

LT.0

INT.1

INT.1.POS

INT.1.NEG

INT.1.POS.STR

INT.1.NEG.STR

INT

INT.POS

INT.NEG

INT.POS.STR

INT.NEG.STR

NUM.1

NUM.1.POS

NUM.1.NEG

NUM

NUM.POS

NUM.NEG

CHR.1

CHR

CPX

CPX.1

LGL

LGL.1
}
\arguments{
\item{exp}{an expression which will be captured but not evaluated.}

\item{err.msg}{character(1L) a message that tells the user what the
expected value should be, should contain a "\%s" for \code{sprintf}
to use (e.g. "\%s should be greater than 2").}
}
\value{
a quoted expressions with \code{err.msg} attribute set
}
\description{
Utility function to generate vetting tokens with attached error messages.
You should only need to use this if the error message produced naturally by
\code{vetr} is unclear.  Several predefined tokens created by this function
are also documented here.
}
\details{
Allows you to supply error messages for vetting to use for each error
token.  Your token should not contain top level \code{&&} or \code{||}.  If
it does your error message will not be reported because \code{vetr} looks for
error messages attached to atomic tokens.  If your token must involve
top level \code{&&} or \code{||}, use \code{I(x && y)} to ensure that
your error message is used by \code{vet}, but beware than in doing so you do
not use templates within the \code{I} call as everything therein will be
interpreted as a vetting expression rather than a template.

Error messages are typically of the form "\%s should be XXX".

This package ships with many predefined tokens for common use cases. They
are listed in the \code{Usage} section of this documentation.  The tokens
are named in format \code{TYPE[.LENGTH][.OTHER]}.  For example
\code{INT} will vet an integer vector, \code{INT.1} will vet a scalar integer
vector, and \code{INT.1.POS.STR} will vet a strictly positive integer vector.
At this time tokens are predefined for the basic types as scalars or
any-length vectors.  Some additional checks are available (e.g. positive only
values).

Every one of the predefined vetting tokens documented here implicitly
disallows NAs.  Numeric tokens also disallow infinite values. If you wish
to allow NAs or infinite values just use a template object (e.g.
\code{integer(1L)}).
}
\note{
\strong{This will only work with standard tokens containing \code{.}}.  Anything
else will be interpreted as a template token.
}
\examples{
## Predefined tokens:
vet(INT.1, 1:2)
vet(INT.1 || LGL, 1:2)
vet(INT.1 || LGL, c(TRUE, FALSE))

## Check squareness
mx <- matrix(1:3)
SQR <- vet_token(nrow(.) == ncol(.), "\%s should be square")
vet(SQR, mx)

## Let `vetr` make up error message; note `quote` vs `vet_token`
## Often, `vetr` does fine without explictly specified err msg:
SQR.V2 <- quote(nrow(.) == ncol(.))
vet(SQR.V2, mx)

## Combine some tokens, notice how we use `quote` at the combining
## step:
NUM.MX <- vet_token(matrix(numeric(), 0, 0), "\%s should be numeric matrix")
SQR.NUM.MX <- quote(NUM.MX && SQR)
vet(SQR.NUM.MX, mx)

## If instead we used `vet_token` the overall error message
## is not used; instead it falls back to the error message of
## the specific sub-token that fails:
NUM.MX <- vet_token(matrix(numeric(), 0, 0), "\%s should be numeric matrix")
SQR.NUM.MX.V2 <-
  vet_token(NUM.MX && SQR, "\%s should be a square numeric matrix")
vet(SQR.NUM.MX.V2, mx)
}
\seealso{
\code{\link[=vet]{vet()}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/benchmark.R
\name{bench_mark}
\alias{bench_mark}
\title{Lightweight Benchmarking Function}
\usage{
bench_mark(..., times = 1000L, deparse.width = 40)
}
\arguments{
\item{...}{expressions to benchmark, are captured unevaluated}

\item{times}{how many times to loop, defaults to 1000}

\item{deparse.width}{how many characters to deparse for labels}
}
\value{
NULL, invisibly, reports timings as a side effect as screen output
}
\description{
Evaluates provided expression in a loop and reports mean evaluation time.
This is inferior to \code{microbenchmark} and other benchmarking tools in many
ways except that it has zero dependencies or suggests which helps with
package build and test times.  Used in vignettes.
}
\details{
Runs \code{\link[=gc]{gc()}} before each expression is evaluated.  Expressions are evaluated
in the order provided.  Attempts to estimate the overhead of the loop by
running a loop that evaluates \code{NULL} the \code{times} times.

Unfortunately because this computes the average of all iterations it is very
susceptible to outliers in small sample runs, particularly with fast running
code.  For that reason the default number of iterations is one thousand.
}
\examples{
bench_mark(runif(1000), Sys.sleep(0.001), times=10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate.R
\name{vet}
\alias{vet}
\alias{tev}
\title{Verify Objects Meet Structural Requirements}
\usage{
vet(
  target,
  current,
  env = parent.frame(),
  format = "text",
  stop = FALSE,
  settings = NULL
)

tev(
  current,
  target,
  env = parent.frame(),
  format = "text",
  stop = FALSE,
  settings = NULL
)
}
\arguments{
\item{target}{a template, a vetting expression, or a compound expression}

\item{current}{an object to vet}

\item{env}{the environment to match calls and evaluate vetting expressions
in; will be ignored if an environment is also specified via
\code{\link[=vetr_settings]{vetr_settings()}}.  Defaults to calling frame.}

\item{format}{character(1L), controls the format of the return value for
\code{vet}, in case of failure.  One of:\itemize{
\item "text": (default) character(1L) message for use elsewhere in code
\item "full": character(1L) the full error message used in "stop" mode,
but actually returned instead of thrown as an error
\item "raw": character(N) least processed version of the error message
with none of the formatting or surrounding verbiage
}}

\item{stop}{TRUE or FALSE whether to call \code{\link[=stop]{stop()}} on failure
or not (default)}

\item{settings}{a settings list as produced by \code{\link[=vetr_settings]{vetr_settings()}}, or NULL to
use the default settings}
}
\value{
TRUE if validation succeeds, otherwise varies according to value
chosen with parameter \code{stop}
}
\description{
Use vetting expressions to enforce structural requirements and/or evaluate
test conditions for truth.  \code{tev} is identical to \code{vet} except with reversed
arguments for pipe based workflows.
}
\section{Vetting Expressions}{


Vetting expressions can be template tokens, standard tokens, or any
expression built with them, \code{||}, \code{&&}, and parentheses.  Template tokens
are R objects that define the required structure, much like the \code{FUN.VALUE}
argument to \code{\link[=vapply]{vapply()}}.  Standard tokens are R expressions evaluated and
checked for being \code{all(TRUE)}.

Standard tokens are distinguished from templates by whether they reference
the \code{.} symbol or not.  If you have a need to reference an object bound to
\code{.} in a vetting expression, you can escape the \code{.}  with an extra dot (i.e.
use \code{..}, and \code{...} for \code{..},  and so forth for symbols comprising only
dots).  If you use standard tokens in your packages you will need to include
\code{utils::globalVariables(".")} as a top-level call to avoid the "no visible
binding for global variable '.'"' R CMD check NOTE.  Standard tokens that
return a string like e.g. \code{all.equal(x, .)} will result in that string being
incorporated into the error message.

See \code{vignette('vetr', package='vetr')} and examples for details on how
to craft vetting expressions.
}

\examples{
## Template token vetting
vet(numeric(2L), runif(2))
vet(numeric(2L), runif(3))
vet(numeric(2L), letters)
try(vet(numeric(2L), letters, stop=TRUE))

## Standard token vetting
vet(. > 0, runif(2))

## Expression made of standard and template tokens.
vet(numeric(1) && . > 0, 1)
try(vet(numeric(1) && . > 0, 1:2))
try(vet(numeric(1) && . > 0, -1))

## `tev` just reverses target and current
## if(getRversion() >= "4.1.0") { # would be a parse error so commented
##   runif(2) |> tev(numeric(2L))
##   runif(3) |> tev(numeric(2L))
## }

## Zero length templates are wild cards
vet(numeric(), runif(2))
vet(numeric(), runif(100))
vet(numeric(), letters)

## This extends to data.frames
iris.tpl <- iris[0,]   # zero row matches any # of rows
iris.1 <- iris[1:10,]
iris.2 <- iris[1:10, c(1,2,3,5,4)]  # change col order
vet(iris.tpl, iris.1)
vet(iris.tpl, iris.2)

## Short (<100 length) integer-like numerics will
## pass for integer
vet(integer(), c(1, 2, 3))
vet(integer(), c(1, 2, 3) + 0.1)

## Nested templates; note, in packages you should consider
## defining templates outside of `vet` or `vetr` so that
## they are computed on load rather that at runtime
tpl <- list(numeric(1L), matrix(integer(), 3))
val.1 <- list(runif(1), rbind(1:10, 1:10, 1:10))
val.2 <- list(runif(1), cbind(1:10, 1:10, 1:10))
vet(tpl, val.1)
vet(tpl, val.2)

## See `example(alike)` for more template examples

## Standard tokens allow you to check values
vet(. > 0, runif(10))
vet(. > 0, -runif(10))

## Zero length token results are considered TRUE,
## as is the case with `all(logical(0))`
vet(. > 0, numeric())

## `all_bw` is like `isTRUE(all(. >= x & . <= y))`, but
## ~10x faster for long vectors:
vet(all_bw(., 0, 1), runif(1e6) + .1)

## You can combine templates and standard tokens with
## `&&` and/or `||`
vet(numeric(2L) && . > 0, runif(2))
vet(numeric(2L) && . > 0, runif(10))
vet(numeric(2L) && . > 0, -runif(2))

## Using pre-defined tokens (see `?vet_token`)
vet(INT.1, 1)
vet(INT.1, 1:2)
vet(INT.1 && . \%in\% 0:1 || LGL.1, TRUE)
vet(INT.1 && . \%in\% 0:1 || LGL.1, 1)
vet(INT.1 && . \%in\% 0:1 || LGL.1, NA)

## Vetting expressions can be assembled from previously
## defined tokens
scalar.num.pos <- quote(numeric(1L) && . > 0)
foo.or.bar <- quote(character(1L) && . \%in\% c('foo', 'bar'))
vet.exp <- quote(scalar.num.pos || foo.or.bar)

vet(vet.exp, 42)
vet(scalar.num.pos || foo.or.bar, 42)  # equivalently
vet(vet.exp, "foo")
vet(vet.exp, "baz")

## Standard tokens that return strings see the string shown
## in the error message:
vet(all.equal(., 2), 1)
}
\seealso{
\code{\link[=vetr]{vetr()}} for a version optimized to vet function arguments,
\code{\link[=alike]{alike()}} for how templates are used, \code{\link[=vet_token]{vet_token()}} for how to specify
custom error messages and also for predefined validation tokens for common
use cases, \code{\link[=all_bw]{all_bw()}} for fast bounds checks.
}
